@@ -1,6 +1,7 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 from __future__ import print_function
-import os, sys, shutil, commands
+import re, os, sys, shutil
+import subprocess
 
 try:
   WindowsError
@@ -12,30 +13,59 @@ class Installer:
     if len(args)<6:
       print('********************************************************************')
       print('Installation script error - not enough arguments:')
-      print('./config/install.py SLEPC_DIR PETSC_DIR DESTDIR PETSC_ARCH LIB_SUFFIX RANLIB')
+      print('./config/install.py SLEPC_DIR PETSC_DIR SLEPC_INSTALLDIR -destDir=DESTDIR [-no-examples] PETSC_ARCH LIB_SUFFIX RANLIB')
       print('********************************************************************')
       sys.exit(1)
     self.rootDir     = args[0]
     self.petscDir    = args[1]
     self.destDir     = os.path.abspath(args[2])
-    self.arch        = args[3]
-    self.arLibSuffix = args[4]
-    self.ranlib      = ' '.join(args[5:])
+    if args[3].startswith('-destDir='):
+      if args[3][9:]:
+        # prepend to prefix dir
+        self.destDir   = os.path.join(args[3][9:], os.path.relpath(self.destDir, os.sep))
+    else:
+      print('********************************************************************')
+      print('Error in argument, should start with -destDir=')
+      print('********************************************************************')
+      sys.exit(1)
+    self.copyexamples = True
+    if args[4] == '-no-examples':
+      self.copyexamples = False
+      del args[4]
+    self.arch        = args[4]
+    self.arLibSuffix = args[5]
+    self.ranlib      = ' '.join(args[6:])
     self.copies = []
     return
 
-  def readDestDir(self, src):
+  def readInstallDir(self, src):
     try:
       f = open(src)
       for l in f.readlines():
         r = l.split('=',1)
         if len(r)!=2: continue
-        if r[0].strip() == 'SLEPC_DESTDIR':
+        if r[0].strip() == 'SLEPC_INSTALLDIR':
           break
       f.close()
     except:
       print('********************************************************************')
-      print('Error reading SLEPC_DESTDIR from slepcvariables')
+      print('Error reading SLEPC_INSTALLDIR from slepcvariables')
+      print('********************************************************************')
+      sys.exit(1)
+    return r[1].strip()
+
+  def readPetscCC(self, src):
+    try:
+      f = open(src)
+      for l in f.readlines():
+        r = l.split('=',1)
+        if len(r)!=2: continue
+        if r[0].strip() == 'CC':
+          break
+      f.close()
+    except:
+      print('********************************************************************')
+      print('Error reading CC from petscvariables')
       print('********************************************************************')
       sys.exit(1)
     return r[1].strip()
@@ -53,11 +83,15 @@ class Installer:
     self.destConfDir       = os.path.join(self.destDir, 'lib','slepc','conf')
     self.destLibDir        = os.path.join(self.destDir, 'lib')
     self.destBinDir        = os.path.join(self.destDir, 'lib','slepc','bin')
-    self.installDir        = self.readDestDir(os.path.join(self.archConfDir,'slepcvariables'))
+    self.installDir        = self.readInstallDir(os.path.join(self.archConfDir,'slepcvariables'))
     self.installIncludeDir = os.path.join(self.installDir, 'include')
     self.installBinDir     = os.path.join(self.installDir, 'lib','slepc','bin')
     self.rootShareDir      = os.path.join(self.rootDir, 'share')
     self.destShareDir      = os.path.join(self.destDir, 'share')
+    self.rootSrcDir        = os.path.join(self.rootDir, 'src')
+    arch = '' if self.arch.startswith('installed-') else self.arch
+    self.petscConfDir      = os.path.join(self.petscDir, arch, 'lib', 'petsc', 'conf')
+    self.petscCC           = self.readPetscCC(os.path.join(self.petscConfDir,'petscvariables'))
     return
 
   def checkDestdir(self):
@@ -104,7 +138,7 @@ class Installer:
     except (IOError, os.error) as why:
       errors.append((srcname, dstname, str(why)))
     except shutil.Error as err:
-      errors.extend((srcname,dstname,str(err.args[0])))
+      errors.append((srcname,dstname,str(err.args[0])))
     if errors:
       raise shutil.Error(errors)
     return copies
@@ -119,16 +153,27 @@ class Installer:
            self.copyfile(srcname,dst)
 
   def fixExamplesMakefile(self, src):
-    '''Change ./${PETSC_ARCH} in makefile in root petsc directory with ${SLEPC_DIR}'''
+    '''Change ./${PETSC_ARCH} in makefile in root slepc directory with ${SLEPC_DIR}'''
     lines   = []
     oldFile = open(src, 'r')
-    for line in oldFile.readlines():
-      # paths generated by configure could be different link-path than whats used by user, so fix both
-      line = line.replace('./${PETSC_ARCH}','${SLEPC_DIR}')
-      lines.append(line)
+    alllines=oldFile.read()
     oldFile.close()
+    newlines=alllines.split('\n')[0]+'\n'  # Firstline
+    # Hardcode SLEPC_DIR, PETSC_DIR and PETSC_ARCH to avoid users doing the wrong thing
+    newlines+='SLEPC_DIR='+self.installDir+'\n'
+    newlines+='PETSC_DIR='+self.petscDir+'\n'
+    newlines+='PETSC_ARCH=\n'
+    for line in alllines.split('\n')[1:]:
+      if line.startswith('TESTLOGTAPFILE'):
+        newlines+='TESTLOGTAPFILE = $(TESTDIR)/test_install_tap.log\n'
+      elif line.startswith('TESTLOGERRFILE'):
+        newlines+='TESTLOGERRFILE = $(TESTDIR)/test_install_err.log\n'
+      elif line.startswith('$(generatedtest)') and 'slepcvariables' in line:
+        newlines+='all: test\n\n'+line+'\n'
+      else:
+        newlines+=line+'\n'
     newFile = open(src, 'w')
-    newFile.write(''.join(lines))
+    newFile.write(newlines)
     newFile.close()
     return
 
@@ -147,7 +192,7 @@ class Installer:
       if not name.startswith('arch') and os.path.isdir(srcname) and os.path.isfile(os.path.join(srcname,'makefile')):
         os.mkdir(dstname)
         nret = self.copyExamples(srcname,dstname)
-        if name == 'tests' or name == 'tutorials' or name == 'nlevp':
+        if name == 'tests' or name == 'tutorials' or name == 'nlevp' or name == 'cnetwork':
           self.copyexamplefiles(srcname,dstname)
           if os.path.isdir(os.path.join(srcname,'output')):
             os.mkdir(os.path.join(dstname,'output'))
@@ -160,6 +205,20 @@ class Installer:
         nret2 = nret + nret2
     return nret2
 
+  def copyConfig(self, src, dst):
+    """Recursively copy the examples directories
+    """
+    if not os.path.isdir(dst):
+      raise shutil.Error('Destination is not a directory')
+
+    self.copies.extend(self.copyfile('gmakefile.test',dst))
+    #newConfigDir=os.path.join(dst,'config')  # Am not renaming at present
+    #if not os.path.isdir(newConfigDir): os.mkdir(newConfigDir)
+    #testConfFiles="".split()
+    #for tf in testConfFiles:
+    #  self.copies.extend(self.copyfile(os.path.join('config',tf),newConfigDir))
+    #return
+
   def copytree(self, src, dst, symlinks = False, copyFunc = shutil.copy2, exclude = [], exclude_ext= ['.DSYM','.o','.pyc','.html'], recurse = 1):
     """Recursively copy a directory tree using copyFunc, which defaults to shutil.copy2().
 
@@ -198,7 +257,7 @@ class Installer:
       # catch the Error from the recursive copytree so that we can
       # continue with other files
       except shutil.Error as err:
-        errors.extend((srcname,dstname,str(err.args[0])))
+        errors.append((srcname,dstname,str(err.args[0])))
     try:
       shutil.copystat(src, dst)
     except OSError as e:
@@ -206,7 +265,7 @@ class Installer:
         # Copying file access times may fail on Windows
         pass
       else:
-        errors.extend((src, dst, str(e)))
+        errors.append((src, dst, str(e)))
     if errors:
       raise shutil.Error(errors)
     return copies
@@ -216,7 +275,7 @@ class Installer:
     lines   = []
     oldFile = open(src, 'r')
     for line in oldFile.readlines():
-      # paths generated by configure could be different link-path than whats used by user, so fix both
+      # paths generated by configure could be different link-path than what's used by user, so fix both
       line = line.replace(os.path.join(self.rootDir, self.arch), self.installDir)
       line = line.replace(os.path.realpath(os.path.join(self.rootDir, self.arch)), self.installDir)
       line = line.replace(os.path.join(self.rootDir, 'bin'), self.installBinDir)
@@ -237,12 +296,150 @@ class Installer:
 
   def fixConf(self):
     import shutil
-    for file in ['slepc_rules', 'slepc_variables','slepcrules', 'slepcvariables']:
+    for file in ['slepc_rules', 'slepc_rules_doc.mk', 'slepc_rules_util.mk', 'slepc_variables', 'slepcrules', 'slepcvariables']:
       self.fixConfFile(os.path.join(self.destConfDir,file))
-    self.fixConfFile(os.path.join(self.destLibDir,'pkgconfig','SLEPc.pc'))
+    self.fixConfFile(os.path.join(self.destLibDir,'pkgconfig','slepc.pc'))
     self.fixConfFile(os.path.join(self.destIncludeDir,'slepcconf.h'))
     return
 
+  def fixPythonWheel(self):
+    import glob
+    import shutil
+    #
+    for pattern in (
+        self.destLibDir  + '/*.a',
+        self.destLibDir  + '/*.la',
+        self.destLibDir  + '/pkgconfig',  # TODO: keep?
+        self.destConfDir + '/configure-hash',
+        self.destConfDir + '/uninstall.py',
+        self.destConfDir + '/reconfigure-*.py',
+        self.destConfDir + '/pkg.conf.*',
+        self.destConfDir + '/pkg.git*.*',
+        self.destConfDir + '/modules',  # TODO: keep?
+        self.destShareDir + '/*/examples/src/*',
+        self.destShareDir + '/*/datafiles',
+    ):
+      for pathname in glob.glob(pattern):
+        if os.path.isdir(pathname):
+          shutil.rmtree(pathname)
+        elif os.path.exists(pathname):
+          os.remove(pathname)
+    #
+    for filename in (
+      self.destIncludeDir + '/slepcconf.h',
+      self.destShareDir + '/slepc/examples/gmakefile.test',
+      self.destConfDir + '/slepc_rules_doc.mk',
+      self.destConfDir + '/slepc_rules_util.mk',
+      self.destConfDir + '/slepc_rules',
+      self.destConfDir + '/slepcrules',
+      self.destConfDir + '/slepc_variables',
+      self.destConfDir + '/slepcvariables',
+    ):
+      with open(filename, 'r') as oldFile:
+        contents = oldFile.read()
+      contents = contents.replace(self.installDir, '${SLEPC_DIR}')
+      contents = contents.replace(self.rootDir, '${SLEPC_DIR}')
+      contents = contents.replace(self.petscDir, '${PETSC_DIR}')
+      with open(filename, 'w') as newFile:
+        newFile.write(contents)
+    #
+    def lsdir(dirname, *patterns):
+      return glob.glob(os.path.join(dirname, *patterns))
+    def shell(*args):
+      out  = subprocess.check_output(list(args), universal_newlines=True)
+      return out[:-1] if out[-1:] == '\n' else out
+    plibdir = os.path.join(self.petscDir, 'lib')
+    slibdir = os.path.join(self.installDir, 'lib')
+    if sys.platform == 'linux':
+      libraries = [
+        lib for lib in lsdir(self.destLibDir, 'lib*.so*')
+        if not os.path.islink(lib)
+      ]
+      for shlib in libraries:
+        # fix shared library rpath
+        rpath = shell('patchelf', '--print-rpath', shlib)
+        rpath = rpath.split(os.path.pathsep)
+        if plibdir in rpath:
+          rpath.insert(0, '$ORIGIN/../../petsc/lib')
+          while plibdir in rpath:
+            rpath.remove(plibdir)
+        if slibdir in rpath:
+          rpath.insert(0, '$ORIGIN')
+          while slibdir in rpath:
+            rpath.remove(slibdir)
+        if rpath:
+          rpath = os.path.pathsep.join(rpath)
+          shell('patchelf', '--set-rpath', rpath, shlib)
+        # fix shared library file and symlink
+        basename = os.path.basename(shlib)
+        libname, ext, _ = basename.partition('.so')
+        liblink = libname + ext
+        soname = shell('patchelf', '--print-soname', shlib)
+        for symlink in lsdir(self.destLibDir, liblink + '*'):
+          if os.path.islink(symlink):
+            os.unlink(symlink)
+        curdir = os.getcwd()
+        try:
+          os.chdir(os.path.dirname(shlib))
+          if soname != basename:
+            os.rename(basename, soname)
+          if soname != liblink:
+            os.symlink(soname, liblink)
+        finally:
+          os.chdir(curdir)
+    if sys.platform == 'darwin':
+      def otool(cmd, dylib):
+        pattern = r'''
+          ^\s+ cmd \s %s$\n
+          ^\s+ cmdsize \s \d+$\n
+          ^\s+ (?:name|path) \s (.*) \s \(offset \s \d+\)$
+        ''' % cmd
+        return re.findall(
+          pattern, shell('otool', '-l', dylib),
+          flags=re.VERBOSE | re.MULTILINE,
+        )
+      libraries = [
+        lib for lib in lsdir(self.destLibDir, 'lib*.dylib')
+        if not os.path.islink(lib)
+      ]
+      for dylib in libraries:
+        install_name = otool('LC_ID_DYLIB', dylib)[0]
+        dependencies = otool('LC_LOAD_DYLIB', dylib)
+        runtime_path = otool('LC_RPATH', dylib)
+        # fix shared library install name and rpath
+        install_name = '@rpath/' + os.path.basename(install_name)
+        shell('install_name_tool', '-id', install_name, dylib)
+        for libdir in (plibdir, slibdir):
+          if libdir in runtime_path:
+            shell('install_name_tool', '-delete_rpath', libdir, dylib)
+        for rpath in ('@loader_path', '@loader_path/../../petsc/lib'):
+          if rpath not in runtime_path:
+            shell('install_name_tool', '-add_rpath', rpath, dylib)
+        for dep in dependencies:
+          if os.path.dirname(dep) in (plibdir, slibdir):
+            newid = '@rpath/' + os.path.basename(dep)
+            shell('install_name_tool', '-change', dep, newid, dylib)
+        # fix shared library file and symlink
+        basename = os.path.basename(dylib)
+        libname, ext = os.path.splitext(basename)
+        libname = libname.partition('.')[0]
+        liblink = libname + ext
+        dyname = os.path.basename(install_name)
+        for symlink in lsdir(self.destLibDir, libname + '*' + ext):
+          if os.path.islink(symlink):
+            os.unlink(symlink)
+        curdir = os.getcwd()
+        try:
+          os.chdir(os.path.dirname(dylib))
+          if dyname != basename:
+            os.rename(basename, dyname)
+          if dyname != liblink:
+            os.symlink(dyname, liblink)
+        finally:
+          os.chdir(curdir)
+    #
+    return
+
   def createUninstaller(self):
     uninstallscript = os.path.join(self.destConfDir, 'uninstall.py')
     f = open(uninstallscript, 'w')
@@ -282,8 +479,8 @@ for dir in dirs:
     return
 
   def installConf(self):
-    self.copies.extend(self.copytree(self.rootConfDir, self.destConfDir, exclude = ['gmakegen.py','install.py','bfort-base.txt','bfort-mpi.txt','bfort-petsc.txt','bfort-slepc.txt']))
-    self.copies.extend(self.copytree(self.archConfDir, self.destConfDir, exclude = ['configure.log','error.log','files','gmake.log','make.log','test.log']))
+    self.copies.extend(self.copytree(self.rootConfDir, self.destConfDir))
+    self.copies.extend(self.copytree(self.archConfDir, self.destConfDir, exclude = ['configure-hash','configure.log','error.log','files','gmake.log','make.log','check.log','memoryerror.log']))
     return
 
   def installBin(self):
@@ -295,11 +492,15 @@ for dir in dirs:
 
   def installShare(self):
     self.copies.extend(self.copytree(self.rootShareDir, self.destShareDir))
-    if os.path.exists(os.path.join(self.destShareDir,'slepc','examples')):
-      shutil.rmtree(os.path.join(self.destShareDir,'slepc','examples'))
-    os.mkdir(os.path.join(self.destShareDir,'slepc','examples'))
-    self.copyExamples(self.rootDir,os.path.join(self.destShareDir,'slepc','examples'))
-    self.fixExamplesMakefile(os.path.join(self.destShareDir,'slepc','examples','makefile'))
+    if self.copyexamples:
+      examplesdir=os.path.join(self.destShareDir,'slepc','examples')
+      if os.path.exists(examplesdir):
+        shutil.rmtree(examplesdir)
+      os.mkdir(examplesdir)
+      os.mkdir(os.path.join(examplesdir,'src'))
+      self.copyExamples(self.rootSrcDir,os.path.join(examplesdir,'src'))
+      self.copyConfig(self.rootDir,examplesdir)
+      self.fixExamplesMakefile(os.path.join(examplesdir,'gmakefile.test'))
     return
 
   def copyLib(self, src, dst):
@@ -317,12 +518,13 @@ for dir in dirs:
     if not os.path.splitext(src)[1] == '.o':
       shutil.copy2(src, dst)
     if os.path.splitext(dst)[1] == '.'+self.arLibSuffix:
-      (result, output) = commands.getstatusoutput(self.ranlib+' '+dst)
-    if os.path.splitext(dst)[1] == '.dylib' and os.path.isfile('/usr/bin/install_name_tool'):
-      (result, output) = commands.getstatusoutput('otool -D '+src)
-      oldname = output[output.find("\n")+1:]
-      installName = oldname.replace(self.archDir, self.installDir)
-      (result, output) = commands.getstatusoutput('/usr/bin/install_name_tool -id ' + installName + ' ' + dst)
+      if not 'win32fe' in self.petscCC:
+        (result, output) = subprocess.getstatusoutput(self.ranlib+' '+dst)
+    if os.path.splitext(dst)[1] == '.dylib' and shutil.which('otool') and shutil.which('install_name_tool'):
+      output = subprocess.check_output(['otool', '-D', src], universal_newlines=True)
+      oldname = output.splitlines()[1]
+      installName = oldname.replace(os.path.realpath(self.archDir), self.installDir)
+      subprocess.check_output(['install_name_tool', '-id', installName, dst])
     # preserve the original timestamps - so that the .a vs .so time order is preserved
     shutil.copystat(src,dst)
     return
@@ -377,14 +579,20 @@ Before use - please copy/install over to specified prefix: %s
     self.installBin()
     self.installLib()
     self.installShare()
+    self.createUninstaller()
     return
 
   def runfix(self):
     self.fixConf()
+    using_build_backend = any(
+      os.environ.get(prefix + '_BUILD_BACKEND')
+      for prefix in ('_PYPROJECT_HOOKS', 'PEP517')
+    )
+    if using_build_backend:
+      self.fixPythonWheel()
     return
 
   def rundone(self):
-    self.createUninstaller()
     if self.destDir == self.installDir:
       self.outputInstallDone()
     else:
