@@ -8,12 +8,13 @@
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#if !defined(SLEPCMFNIMPL_H)
-#define SLEPCMFNIMPL_H
+#pragma once
 
 #include <slepcmfn.h>
 #include <slepc/private/slepcimpl.h>
 
+/* SUBMANSEC = MFN */
+
 SLEPC_EXTERN PetscBool MFNRegisterAllCalled;
 SLEPC_EXTERN PetscBool MFNMonitorRegisterAllCalled;
 SLEPC_EXTERN PetscErrorCode MFNRegisterAll(void);
@@ -25,7 +26,7 @@ typedef struct _MFNOps *MFNOps;
 struct _MFNOps {
   PetscErrorCode (*solve)(MFN,Vec,Vec);
   PetscErrorCode (*setup)(MFN);
-  PetscErrorCode (*setfromoptions)(PetscOptionItems*,MFN);
+  PetscErrorCode (*setfromoptions)(MFN,PetscOptionItems);
   PetscErrorCode (*publishoptions)(MFN);
   PetscErrorCode (*destroy)(MFN);
   PetscErrorCode (*reset)(MFN);
@@ -51,10 +52,10 @@ struct _p_MFN {
   PetscBool      errorifnotconverged;    /* error out if MFNSolve() does not converge */
 
   /*-------------- User-provided functions and contexts -----------------*/
-  PetscErrorCode (*monitor[MAXMFNMONITORS])(MFN,PetscInt,PetscReal,void*);
-  PetscErrorCode (*monitordestroy[MAXMFNMONITORS])(void**);
-  void           *monitorcontext[MAXMFNMONITORS];
-  PetscInt       numbermonitors;
+  PetscErrorCode    (*monitor[MAXMFNMONITORS])(MFN,PetscInt,PetscReal,void*);
+  PetscCtxDestroyFn *monitordestroy[MAXMFNMONITORS];
+  void              *monitorcontext[MAXMFNMONITORS];
+  PetscInt          numbermonitors;
 
   /*----------------- Child objects and working data -------------------*/
   BV             V;                /* set of basis vectors */
@@ -91,7 +92,7 @@ static inline PetscErrorCode MFN_CreateDenseMat(PetscInt k,Mat *A)
     }
   }
   if (create) PetscCall(MatCreateSeqDense(PETSC_COMM_SELF,k,k,NULL,A));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -112,7 +113,5 @@ static inline PetscErrorCode MFN_CreateVec(PetscInt k,Vec *v)
     }
   }
   if (create) PetscCall(VecCreateSeq(PETSC_COMM_SELF,k,v));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
-
-#endif
