@@ -11,8 +11,8 @@
    User interface for the SLEPc linear eigenvalue solvers
 */
 
-#if !defined(SLEPCEPS_H)
-#define SLEPCEPS_H
+#pragma once
+
 #include <slepcst.h>
 #include <slepcbv.h>
 #include <slepcds.h>
@@ -20,7 +20,10 @@
 #include <slepclme.h>
 #include <petscsnes.h>
 
+/* SUBMANSEC = EPS */
+
 SLEPC_EXTERN PetscErrorCode EPSInitializePackage(void);
+SLEPC_EXTERN PetscErrorCode EPSFinalizePackage(void);
 
 /*S
     EPS - Abstract SLEPc object that manages all the eigenvalue
@@ -39,7 +42,7 @@ typedef struct _p_EPS* EPS;
 
 .seealso: EPSSetType(), EPS
 J*/
-typedef const char* EPSType;
+typedef const char *EPSType;
 #define EPSPOWER       "power"
 #define EPSSUBSPACE    "subspace"
 #define EPSARNOLDI     "arnoldi"
@@ -61,6 +64,7 @@ typedef const char* EPSType;
 #define EPSELPA        "elpa"
 #define EPSELEMENTAL   "elemental"
 #define EPSEVSL        "evsl"
+#define EPSCHASE       "chase"
 
 /* Logging support */
 SLEPC_EXTERN PetscClassId EPS_CLASSID;
@@ -72,12 +76,13 @@ SLEPC_EXTERN PetscClassId EPS_CLASSID;
 
 .seealso: EPSSetProblemType(), EPSGetProblemType()
 E*/
-typedef enum { EPS_HEP=1,
-               EPS_GHEP,
-               EPS_NHEP,
-               EPS_GNHEP,
-               EPS_PGNHEP,
-               EPS_GHIEP } EPSProblemType;
+typedef enum { EPS_HEP    = 1,
+               EPS_GHEP   = 2,
+               EPS_NHEP   = 3,
+               EPS_GNHEP  = 4,
+               EPS_PGNHEP = 5,
+               EPS_GHIEP  = 6,
+               EPS_BSE    = 7 } EPSProblemType;
 
 /*E
     EPSExtraction - Determines the type of extraction technique employed
@@ -102,17 +107,17 @@ typedef enum { EPS_RITZ,
 
 .seealso: EPSSetWhichEigenpairs(), EPSGetWhichEigenpairs()
 E*/
-typedef enum { EPS_LARGEST_MAGNITUDE=1,
-               EPS_SMALLEST_MAGNITUDE,
-               EPS_LARGEST_REAL,
-               EPS_SMALLEST_REAL,
-               EPS_LARGEST_IMAGINARY,
-               EPS_SMALLEST_IMAGINARY,
-               EPS_TARGET_MAGNITUDE,
-               EPS_TARGET_REAL,
-               EPS_TARGET_IMAGINARY,
-               EPS_ALL,
-               EPS_WHICH_USER } EPSWhich;
+typedef enum { EPS_LARGEST_MAGNITUDE  = 1,
+               EPS_SMALLEST_MAGNITUDE = 2,
+               EPS_LARGEST_REAL       = 3,
+               EPS_SMALLEST_REAL      = 4,
+               EPS_LARGEST_IMAGINARY  = 5,
+               EPS_SMALLEST_IMAGINARY = 6,
+               EPS_TARGET_MAGNITUDE   = 7,
+               EPS_TARGET_REAL        = 8,
+               EPS_TARGET_IMAGINARY   = 9,
+               EPS_ALL                = 10,
+               EPS_WHICH_USER         = 11 } EPSWhich;
 
 /*E
     EPSBalance - The type of balancing used for non-Hermitian problems
@@ -159,7 +164,8 @@ typedef enum { EPS_CONV_ABS,
 .seealso: EPSSetStoppingTest(), EPSSetStoppingTestFunction()
 E*/
 typedef enum { EPS_STOP_BASIC,
-               EPS_STOP_USER } EPSStop;
+               EPS_STOP_USER,
+               EPS_STOP_THRESHOLD } EPSStop;
 
 /*E
     EPSConvergedReason - Reason an eigensolver was said to
@@ -179,6 +185,23 @@ typedef enum {/* converged */
               EPS_CONVERGED_ITERATING          =  0} EPSConvergedReason;
 SLEPC_EXTERN const char *const*EPSConvergedReasons;
 
+/*S
+   EPSStoppingCtx - Data structure (C struct) to hold additional information to
+   be used in some stopping test functions.
+
+   Level: advanced
+
+.seealso: EPSSetStoppingTestFunction()
+S*/
+struct _n_EPSStoppingCtx {
+  PetscReal firstev;    /* the (absolute) value of the first converged eigenvalue */
+  PetscReal lastev;     /* the (absolute) value of the last converged eigenvalue */
+  PetscReal thres;      /* threshold set with EPSSetThreshold() */
+  PetscBool threlative; /* threshold is relative */
+  EPSWhich  which;      /* which eigenvalues are being computed */
+};
+typedef struct _n_EPSStoppingCtx* EPSStoppingCtx;
+
 SLEPC_EXTERN PetscErrorCode EPSCreate(MPI_Comm,EPS*);
 SLEPC_EXTERN PetscErrorCode EPSDestroy(EPS*);
 SLEPC_EXTERN PetscErrorCode EPSReset(EPS);
@@ -193,17 +216,18 @@ SLEPC_EXTERN PetscErrorCode EPSGetBalance(EPS,EPSBalance*,PetscInt*,PetscReal*);
 SLEPC_EXTERN PetscErrorCode EPSSetOperators(EPS,Mat,Mat);
 SLEPC_EXTERN PetscErrorCode EPSGetOperators(EPS,Mat*,Mat*);
 SLEPC_EXTERN PetscErrorCode EPSSetFromOptions(EPS);
+SLEPC_EXTERN PetscErrorCode EPSSetDSType(EPS);
 SLEPC_EXTERN PetscErrorCode EPSSetUp(EPS);
 SLEPC_EXTERN PetscErrorCode EPSSolve(EPS);
 SLEPC_EXTERN PetscErrorCode EPSView(EPS,PetscViewer);
 SLEPC_EXTERN PetscErrorCode EPSViewFromOptions(EPS,PetscObject,const char[]);
 SLEPC_EXTERN PetscErrorCode EPSErrorView(EPS,EPSErrorType,PetscViewer);
-PETSC_DEPRECATED_FUNCTION("Use EPSErrorView()") static inline PetscErrorCode EPSPrintSolution(EPS eps,PetscViewer v) {return EPSErrorView(eps,EPS_ERROR_RELATIVE,v);}
+PETSC_DEPRECATED_FUNCTION(3, 6, 0, "EPSErrorView()", ) static inline PetscErrorCode EPSPrintSolution(EPS eps,PetscViewer v) {return EPSErrorView(eps,EPS_ERROR_RELATIVE,v);}
 SLEPC_EXTERN PetscErrorCode EPSErrorViewFromOptions(EPS);
 SLEPC_EXTERN PetscErrorCode EPSConvergedReasonView(EPS,PetscViewer);
 SLEPC_EXTERN PetscErrorCode EPSConvergedReasonViewFromOptions(EPS);
-PETSC_DEPRECATED_FUNCTION("Use EPSConvergedReasonView() (since version 3.14)") static inline PetscErrorCode EPSReasonView(EPS eps,PetscViewer v) {return EPSConvergedReasonView(eps,v);}
-PETSC_DEPRECATED_FUNCTION("Use EPSConvergedReasonViewFromOptions() (since version 3.14)") static inline PetscErrorCode EPSReasonViewFromOptions(EPS eps) {return EPSConvergedReasonViewFromOptions(eps);}
+PETSC_DEPRECATED_FUNCTION(3, 14, 0, "EPSConvergedReasonView()", ) static inline PetscErrorCode EPSReasonView(EPS eps,PetscViewer v) {return EPSConvergedReasonView(eps,v);}
+PETSC_DEPRECATED_FUNCTION(3, 14, 0, "EPSConvergedReasonViewFromOptions()", ) static inline PetscErrorCode EPSReasonViewFromOptions(EPS eps) {return EPSConvergedReasonViewFromOptions(eps);}
 SLEPC_EXTERN PetscErrorCode EPSValuesView(EPS,PetscViewer);
 SLEPC_EXTERN PetscErrorCode EPSValuesViewFromOptions(EPS);
 SLEPC_EXTERN PetscErrorCode EPSVectorsView(EPS,PetscViewer);
@@ -223,16 +247,15 @@ SLEPC_EXTERN PetscErrorCode EPSSetDS(EPS,DS);
 SLEPC_EXTERN PetscErrorCode EPSGetDS(EPS,DS*);
 SLEPC_EXTERN PetscErrorCode EPSSetTolerances(EPS,PetscReal,PetscInt);
 SLEPC_EXTERN PetscErrorCode EPSGetTolerances(EPS,PetscReal*,PetscInt*);
-SLEPC_EXTERN PetscErrorCode EPSSetConvergenceTestFunction(EPS,PetscErrorCode (*)(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*),void*,PetscErrorCode (*)(void*));
 SLEPC_EXTERN PetscErrorCode EPSSetConvergenceTest(EPS,EPSConv);
 SLEPC_EXTERN PetscErrorCode EPSGetConvergenceTest(EPS,EPSConv*);
 SLEPC_EXTERN PetscErrorCode EPSConvergedAbsolute(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
 SLEPC_EXTERN PetscErrorCode EPSConvergedRelative(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
 SLEPC_EXTERN PetscErrorCode EPSConvergedNorm(EPS,PetscScalar,PetscScalar,PetscReal,PetscReal*,void*);
-SLEPC_EXTERN PetscErrorCode EPSSetStoppingTestFunction(EPS,PetscErrorCode (*)(EPS,PetscInt,PetscInt,PetscInt,PetscInt,EPSConvergedReason*,void*),void*,PetscErrorCode (*)(void*));
 SLEPC_EXTERN PetscErrorCode EPSSetStoppingTest(EPS,EPSStop);
 SLEPC_EXTERN PetscErrorCode EPSGetStoppingTest(EPS,EPSStop*);
 SLEPC_EXTERN PetscErrorCode EPSStoppingBasic(EPS,PetscInt,PetscInt,PetscInt,PetscInt,EPSConvergedReason*,void*);
+SLEPC_EXTERN PetscErrorCode EPSStoppingThreshold(EPS,PetscInt,PetscInt,PetscInt,PetscInt,EPSConvergedReason*,void*);
 SLEPC_EXTERN PetscErrorCode EPSGetConvergedReason(EPS,EPSConvergedReason*);
 
 SLEPC_EXTERN PetscErrorCode EPSSetDimensions(EPS,PetscInt,PetscInt,PetscInt);
@@ -245,25 +268,26 @@ SLEPC_EXTERN PetscErrorCode EPSGetEigenvector(EPS,PetscInt,Vec,Vec);
 SLEPC_EXTERN PetscErrorCode EPSGetLeftEigenvector(EPS,PetscInt,Vec,Vec);
 
 SLEPC_EXTERN PetscErrorCode EPSComputeError(EPS,PetscInt,EPSErrorType,PetscReal*);
-PETSC_DEPRECATED_FUNCTION("Use EPSComputeError()") static inline PetscErrorCode EPSComputeRelativeError(EPS eps,PetscInt i,PetscReal *r) {return EPSComputeError(eps,i,EPS_ERROR_RELATIVE,r);}
-PETSC_DEPRECATED_FUNCTION("Use EPSComputeError() with EPS_ERROR_ABSOLUTE") static inline PetscErrorCode EPSComputeResidualNorm(EPS eps,PetscInt i,PetscReal *r) {return EPSComputeError(eps,i,EPS_ERROR_ABSOLUTE,r);}
+PETSC_DEPRECATED_FUNCTION(3, 6, 0, "EPSComputeError()", ) static inline PetscErrorCode EPSComputeRelativeError(EPS eps,PetscInt i,PetscReal *r) {return EPSComputeError(eps,i,EPS_ERROR_RELATIVE,r);}
+PETSC_DEPRECATED_FUNCTION(3, 6, 0, "EPSComputeError() with EPS_ERROR_ABSOLUTE", ) static inline PetscErrorCode EPSComputeResidualNorm(EPS eps,PetscInt i,PetscReal *r) {return EPSComputeError(eps,i,EPS_ERROR_ABSOLUTE,r);}
 SLEPC_EXTERN PetscErrorCode EPSGetInvariantSubspace(EPS,Vec[]);
 SLEPC_EXTERN PetscErrorCode EPSGetErrorEstimate(EPS,PetscInt,PetscReal*);
 SLEPC_EXTERN PetscErrorCode EPSGetIterationNumber(EPS,PetscInt*);
 
 SLEPC_EXTERN PetscErrorCode EPSSetWhichEigenpairs(EPS,EPSWhich);
 SLEPC_EXTERN PetscErrorCode EPSGetWhichEigenpairs(EPS,EPSWhich*);
+SLEPC_EXTERN PetscErrorCode EPSSetThreshold(EPS,PetscReal,PetscBool);
+SLEPC_EXTERN PetscErrorCode EPSGetThreshold(EPS,PetscReal*,PetscBool*);
 SLEPC_EXTERN PetscErrorCode EPSSetTwoSided(EPS,PetscBool);
 SLEPC_EXTERN PetscErrorCode EPSGetTwoSided(EPS,PetscBool*);
 SLEPC_EXTERN PetscErrorCode EPSSetTrueResidual(EPS,PetscBool);
 SLEPC_EXTERN PetscErrorCode EPSGetTrueResidual(EPS,PetscBool*);
 SLEPC_EXTERN PetscErrorCode EPSSetPurify(EPS,PetscBool);
 SLEPC_EXTERN PetscErrorCode EPSGetPurify(EPS,PetscBool*);
-SLEPC_EXTERN PetscErrorCode EPSSetEigenvalueComparison(EPS,PetscErrorCode (*func)(PetscScalar,PetscScalar,PetscScalar,PetscScalar,PetscInt*,void*),void*);
-SLEPC_EXTERN PetscErrorCode EPSSetArbitrarySelection(EPS,PetscErrorCode (*func)(PetscScalar,PetscScalar,Vec,Vec,PetscScalar*,PetscScalar*,void*),void*);
 SLEPC_EXTERN PetscErrorCode EPSIsGeneralized(EPS,PetscBool*);
 SLEPC_EXTERN PetscErrorCode EPSIsHermitian(EPS,PetscBool*);
 SLEPC_EXTERN PetscErrorCode EPSIsPositive(EPS,PetscBool*);
+SLEPC_EXTERN PetscErrorCode EPSIsStructured(EPS,PetscBool*);
 
 SLEPC_EXTERN PetscErrorCode EPSSetTrackAll(EPS,PetscBool);
 SLEPC_EXTERN PetscErrorCode EPSGetTrackAll(EPS,PetscBool*);
@@ -273,22 +297,21 @@ SLEPC_EXTERN PetscErrorCode EPSSetInitialSpace(EPS,PetscInt,Vec[]);
 SLEPC_EXTERN PetscErrorCode EPSSetLeftInitialSpace(EPS,PetscInt,Vec[]);
 
 SLEPC_EXTERN PetscErrorCode EPSMonitor(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt);
-SLEPC_EXTERN PetscErrorCode EPSMonitorSet(EPS,PetscErrorCode (*)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*),void*,PetscErrorCode (*)(void**));
+SLEPC_EXTERN PetscErrorCode EPSMonitorSet(EPS,PetscErrorCode (*)(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,void*),void*,PetscCtxDestroyFn*);
 SLEPC_EXTERN PetscErrorCode EPSMonitorCancel(EPS);
 SLEPC_EXTERN PetscErrorCode EPSGetMonitorContext(EPS,void*);
 
 SLEPC_EXTERN PetscErrorCode EPSMonitorSetFromOptions(EPS,const char[],const char[],void*,PetscBool);
-SLEPC_EXTERN PetscErrorCode EPSMonitorLGCreate(MPI_Comm,const char[],const char[],const char[],PetscInt,const char*[],int,int,int,int,PetscDrawLG*);
 SLEPC_EXTERN PetscErrorCode EPSMonitorFirst(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
 SLEPC_EXTERN PetscErrorCode EPSMonitorFirstDrawLG(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
-SLEPC_EXTERN PetscErrorCode EPSMonitorFirstDrawLGCreate(PetscViewer,PetscViewerFormat,void *,PetscViewerAndFormat**);
+SLEPC_EXTERN PetscErrorCode EPSMonitorFirstDrawLGCreate(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**);
 SLEPC_EXTERN PetscErrorCode EPSMonitorAll(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
 SLEPC_EXTERN PetscErrorCode EPSMonitorAllDrawLG(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
-SLEPC_EXTERN PetscErrorCode EPSMonitorAllDrawLGCreate(PetscViewer,PetscViewerFormat,void *,PetscViewerAndFormat**);
+SLEPC_EXTERN PetscErrorCode EPSMonitorAllDrawLGCreate(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**);
 SLEPC_EXTERN PetscErrorCode EPSMonitorConverged(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
-SLEPC_EXTERN PetscErrorCode EPSMonitorConvergedCreate(PetscViewer,PetscViewerFormat,void *,PetscViewerAndFormat**);
+SLEPC_EXTERN PetscErrorCode EPSMonitorConvergedCreate(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**);
 SLEPC_EXTERN PetscErrorCode EPSMonitorConvergedDrawLG(EPS,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscReal*,PetscInt,PetscViewerAndFormat*);
-SLEPC_EXTERN PetscErrorCode EPSMonitorConvergedDrawLGCreate(PetscViewer,PetscViewerFormat,void *,PetscViewerAndFormat**);
+SLEPC_EXTERN PetscErrorCode EPSMonitorConvergedDrawLGCreate(PetscViewer,PetscViewerFormat,void*,PetscViewerAndFormat**);
 SLEPC_EXTERN PetscErrorCode EPSMonitorConvergedDestroy(PetscViewerAndFormat**);
 
 SLEPC_EXTERN PetscErrorCode EPSSetOptionsPrefix(EPS,const char*);
@@ -304,6 +327,49 @@ SLEPC_EXTERN PetscErrorCode EPSMonitorRegister(const char[],PetscViewerType,Pets
 
 SLEPC_EXTERN PetscErrorCode EPSSetWorkVecs(EPS,PetscInt);
 SLEPC_EXTERN PetscErrorCode EPSAllocateSolution(EPS,PetscInt);
+SLEPC_EXTERN PetscErrorCode EPSReallocateSolution(EPS,PetscInt);
+
+/*S
+  EPSConvergenceTestFn - A prototype of an EPS convergence test function that would be passed to EPSSetConvergenceTestFunction()
+
+  Calling Sequence:
++   eps    - eigensolver context obtained from EPSCreate()
+.   eigr   - real part of the eigenvalue
+.   eigi   - imaginary part of the eigenvalue
+.   res    - residual norm associated to the eigenpair
+.   errest - [output] computed error estimate
+-   ctx    - [optional] user-defined context for private data for the
+             convergence test routine (may be NULL)
+
+  Level: advanced
+
+.seealso: EPSSetConvergenceTestFunction()
+S*/
+PETSC_EXTERN_TYPEDEF typedef PetscErrorCode(EPSConvergenceTestFn)(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscReal res,PetscReal *errest,void *ctx);
+
+/*S
+  EPSStoppingTestFn - A prototype of an EPS stopping test function that would be passed to EPSSetStoppingTestFunction()
+
+  Calling Sequence:
++   eps    - eigensolver context obtained from EPSCreate()
+.   its    - current number of iterations
+.   max_it - maximum number of iterations
+.   nconv  - number of currently converged eigenpairs
+.   nev    - number of requested eigenpairs
+.   reason - [output] result of the stopping test
+-   ctx    - [optional] user-defined context for private data for the
+             stopping test routine (may be NULL)
+
+  Level: advanced
+
+.seealso: EPSSetStoppingTestFunction()
+S*/
+PETSC_EXTERN_TYPEDEF typedef PetscErrorCode(EPSStoppingTestFn)(EPS eps,PetscInt its,PetscInt max_it,PetscInt nconv,PetscInt nev,EPSConvergedReason *reason,void *ctx);
+
+SLEPC_EXTERN PetscErrorCode EPSSetConvergenceTestFunction(EPS,EPSConvergenceTestFn*,void*,PetscCtxDestroyFn*);
+SLEPC_EXTERN PetscErrorCode EPSSetStoppingTestFunction(EPS,EPSStoppingTestFn*,void*,PetscCtxDestroyFn*);
+SLEPC_EXTERN PetscErrorCode EPSSetEigenvalueComparison(EPS,SlepcEigenvalueComparisonFn*,void*);
+SLEPC_EXTERN PetscErrorCode EPSSetArbitrarySelection(EPS,SlepcArbitrarySelectionFn*,void*);
 
 /* --------- options specific to particular eigensolvers -------- */
 
@@ -325,12 +391,29 @@ SLEPC_EXTERN PetscErrorCode EPSPowerSetNonlinear(EPS,PetscBool);
 SLEPC_EXTERN PetscErrorCode EPSPowerGetNonlinear(EPS,PetscBool*);
 SLEPC_EXTERN PetscErrorCode EPSPowerSetUpdate(EPS,PetscBool);
 SLEPC_EXTERN PetscErrorCode EPSPowerGetUpdate(EPS,PetscBool*);
+SLEPC_EXTERN PetscErrorCode EPSPowerSetSignNormalization(EPS,PetscBool);
+SLEPC_EXTERN PetscErrorCode EPSPowerGetSignNormalization(EPS,PetscBool*);
 SLEPC_EXTERN PetscErrorCode EPSPowerSetSNES(EPS,SNES);
 SLEPC_EXTERN PetscErrorCode EPSPowerGetSNES(EPS,SNES*);
 
 SLEPC_EXTERN PetscErrorCode EPSArnoldiSetDelayed(EPS,PetscBool);
 SLEPC_EXTERN PetscErrorCode EPSArnoldiGetDelayed(EPS,PetscBool*);
 
+/*E
+    EPSKrylovSchurBSEType - the method to be used in the Krylov-Schur solver
+    for the case of BSE structured eigenproblems
+
+    Level: advanced
+
+.seealso: EPSKrylovSchurSetBSEType(), EPSKrylovSchurGetBSEType()
+E*/
+typedef enum { EPS_KRYLOVSCHUR_BSE_SHAO,
+               EPS_KRYLOVSCHUR_BSE_GRUNING,
+               EPS_KRYLOVSCHUR_BSE_PROJECTEDBSE } EPSKrylovSchurBSEType;
+SLEPC_EXTERN const char *EPSKrylovSchurBSETypes[];
+
+SLEPC_EXTERN PetscErrorCode EPSKrylovSchurSetBSEType(EPS,EPSKrylovSchurBSEType);
+SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetBSEType(EPS,EPSKrylovSchurBSEType*);
 SLEPC_EXTERN PetscErrorCode EPSKrylovSchurSetRestart(EPS,PetscReal);
 SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetRestart(EPS,PetscReal*);
 SLEPC_EXTERN PetscErrorCode EPSKrylovSchurSetLocking(EPS,PetscBool);
@@ -342,8 +425,8 @@ SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetDetectZeros(EPS,PetscBool*);
 SLEPC_EXTERN PetscErrorCode EPSKrylovSchurSetDimensions(EPS,PetscInt,PetscInt,PetscInt);
 SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetDimensions(EPS,PetscInt*,PetscInt*,PetscInt*);
 SLEPC_EXTERN PetscErrorCode EPSKrylovSchurSetSubintervals(EPS,PetscReal*);
-SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetSubintervals(EPS,PetscReal**);
-SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetInertias(EPS,PetscInt*,PetscReal**,PetscInt**);
+SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetSubintervals(EPS,PetscReal*[]);
+SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetInertias(EPS,PetscInt*,PetscReal*[],PetscInt*[]);
 SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetSubcommInfo(EPS,PetscInt*,PetscInt*,Vec*);
 SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetSubcommPairs(EPS,PetscInt,PetscScalar*,Vec);
 SLEPC_EXTERN PetscErrorCode EPSKrylovSchurGetSubcommMats(EPS,Mat*,Mat*);
@@ -376,21 +459,21 @@ SLEPC_EXTERN PetscErrorCode EPSLanczosGetReorthog(EPS,EPSLanczosReorthogType*);
 
 .seealso: EPSPRIMMESetMethod(), EPSPRIMMEGetMethod()
 E*/
-typedef enum { EPS_PRIMME_DYNAMIC=1,
-               EPS_PRIMME_DEFAULT_MIN_TIME,
-               EPS_PRIMME_DEFAULT_MIN_MATVECS,
-               EPS_PRIMME_ARNOLDI,
-               EPS_PRIMME_GD,
-               EPS_PRIMME_GD_PLUSK,
-               EPS_PRIMME_GD_OLSEN_PLUSK,
-               EPS_PRIMME_JD_OLSEN_PLUSK,
-               EPS_PRIMME_RQI,
-               EPS_PRIMME_JDQR,
-               EPS_PRIMME_JDQMR,
-               EPS_PRIMME_JDQMR_ETOL,
-               EPS_PRIMME_SUBSPACE_ITERATION,
-               EPS_PRIMME_LOBPCG_ORTHOBASIS,
-               EPS_PRIMME_LOBPCG_ORTHOBASISW } EPSPRIMMEMethod;
+typedef enum { EPS_PRIMME_DYNAMIC             = 1,
+               EPS_PRIMME_DEFAULT_MIN_TIME    = 2,
+               EPS_PRIMME_DEFAULT_MIN_MATVECS = 3,
+               EPS_PRIMME_ARNOLDI             = 4,
+               EPS_PRIMME_GD                  = 5,
+               EPS_PRIMME_GD_PLUSK            = 6,
+               EPS_PRIMME_GD_OLSEN_PLUSK      = 7,
+               EPS_PRIMME_JD_OLSEN_PLUSK      = 8,
+               EPS_PRIMME_RQI                 = 9,
+               EPS_PRIMME_JDQR                = 10,
+               EPS_PRIMME_JDQMR               = 11,
+               EPS_PRIMME_JDQMR_ETOL          = 12,
+               EPS_PRIMME_SUBSPACE_ITERATION  = 13,
+               EPS_PRIMME_LOBPCG_ORTHOBASIS   = 14,
+               EPS_PRIMME_LOBPCG_ORTHOBASISW  = 15 } EPSPRIMMEMethod;
 SLEPC_EXTERN const char *EPSPRIMMEMethods[];
 
 SLEPC_EXTERN PetscErrorCode EPSPRIMMESetBlockSize(EPS,PetscInt);
@@ -443,8 +526,8 @@ SLEPC_EXTERN PetscErrorCode EPSLOBPCGGetLocking(EPS,PetscBool*);
 
 .seealso: EPSCISSSetQuadRule(), EPSCISSGetQuadRule()
 E*/
-typedef enum { EPS_CISS_QUADRULE_TRAPEZOIDAL=1,
-               EPS_CISS_QUADRULE_CHEBYSHEV } EPSCISSQuadRule;
+typedef enum { EPS_CISS_QUADRULE_TRAPEZOIDAL = 1,
+               EPS_CISS_QUADRULE_CHEBYSHEV   = 2 } EPSCISSQuadRule;
 SLEPC_EXTERN const char *EPSCISSQuadRules[];
 
 /*E
@@ -470,7 +553,7 @@ SLEPC_EXTERN PetscErrorCode EPSCISSSetRefinement(EPS,PetscInt,PetscInt);
 SLEPC_EXTERN PetscErrorCode EPSCISSGetRefinement(EPS,PetscInt*,PetscInt*);
 SLEPC_EXTERN PetscErrorCode EPSCISSSetUseST(EPS,PetscBool);
 SLEPC_EXTERN PetscErrorCode EPSCISSGetUseST(EPS,PetscBool*);
-SLEPC_EXTERN PetscErrorCode EPSCISSGetKSPs(EPS,PetscInt*,KSP**);
+SLEPC_EXTERN PetscErrorCode EPSCISSGetKSPs(EPS,PetscInt*,KSP*[]);
 
 SLEPC_EXTERN PetscErrorCode EPSLyapIISetLME(EPS,LME);
 SLEPC_EXTERN PetscErrorCode EPSLyapIIGetLME(EPS,LME*);
@@ -517,5 +600,5 @@ SLEPC_EXTERN PetscErrorCode EPSEVSLGetDamping(EPS,EPSEVSLDamping*);
 SLEPC_EXTERN PetscErrorCode EPSFEASTSetNumPoints(EPS,PetscInt);
 SLEPC_EXTERN PetscErrorCode EPSFEASTGetNumPoints(EPS,PetscInt*);
 
-#endif
-
+SLEPC_EXTERN PetscErrorCode EPSCHASESetDegree(EPS,PetscInt,PetscBool);
+SLEPC_EXTERN PetscErrorCode EPSCHASEGetDegree(EPS,PetscInt*,PetscBool*);
