@@ -37,34 +37,36 @@ typedef struct {
   PetscInt  guard;     /* number of guard vectors */
 } EPS_LOBPCG;
 
-PetscErrorCode EPSSetDimensions_LOBPCG(EPS eps,PetscInt nev,PetscInt *ncv,PetscInt *mpd)
+static PetscErrorCode EPSSetDimensions_LOBPCG(EPS eps,PetscInt *nev,PetscInt *ncv,PetscInt *mpd)
 {
   EPS_LOBPCG *ctx = (EPS_LOBPCG*)eps->data;
   PetscInt   k;
 
   PetscFunctionBegin;
-  k = PetscMax(3*ctx->bs,((eps->nev-1)/ctx->bs+3)*ctx->bs);
-  if (*ncv!=PETSC_DEFAULT) { /* ncv set */
+  if (*nev==0) *nev = 1;
+  k = PetscMax(3*ctx->bs,((*nev-1)/ctx->bs+3)*ctx->bs);
+  if (*ncv!=PETSC_DETERMINE) { /* ncv set */
     PetscCheck(*ncv>=k,PetscObjectComm((PetscObject)eps),PETSC_ERR_USER_INPUT,"The value of ncv is not sufficiently large");
   } else *ncv = k;
-  if (*mpd==PETSC_DEFAULT) *mpd = 3*ctx->bs;
+  if (*mpd==PETSC_DETERMINE) *mpd = 3*ctx->bs;
   else PetscCheck(*mpd==3*ctx->bs,PetscObjectComm((PetscObject)eps),PETSC_ERR_USER_INPUT,"This solver does not allow a value of mpd different from 3*blocksize");
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode EPSSetUp_LOBPCG(EPS eps)
+static PetscErrorCode EPSSetUp_LOBPCG(EPS eps)
 {
   EPS_LOBPCG     *ctx = (EPS_LOBPCG*)eps->data;
 
   PetscFunctionBegin;
   EPSCheckHermitianDefinite(eps);
-  if (!ctx->bs) ctx->bs = PetscMin(16,eps->nev);
+  EPSCheckNotStructured(eps);
+  if (!ctx->bs) ctx->bs = PetscMin(16,eps->nev?eps->nev:1);
   PetscCheck(eps->n-eps->nds>=5*ctx->bs,PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"The problem size is too small relative to the block size");
-  PetscCall(EPSSetDimensions_LOBPCG(eps,eps->nev,&eps->ncv,&eps->mpd));
-  if (eps->max_it==PETSC_DEFAULT) eps->max_it = PetscMax(100,2*eps->n/eps->ncv);
+  PetscCall(EPSSetDimensions_LOBPCG(eps,&eps->nev,&eps->ncv,&eps->mpd));
+  if (eps->max_it==PETSC_DETERMINE) eps->max_it = PetscMax(100,2*eps->n/eps->ncv);
   if (!eps->which) eps->which = EPS_SMALLEST_REAL;
   PetscCheck(eps->which==EPS_SMALLEST_REAL || eps->which==EPS_LARGEST_REAL,PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver supports only smallest real or largest real eigenvalues");
-  EPSCheckUnsupported(eps,EPS_FEATURE_ARBITRARY | EPS_FEATURE_REGION | EPS_FEATURE_EXTRACTION);
+  EPSCheckUnsupported(eps,EPS_FEATURE_ARBITRARY | EPS_FEATURE_REGION | EPS_FEATURE_EXTRACTION | EPS_FEATURE_THRESHOLD);
   EPSCheckIgnored(eps,EPS_FEATURE_BALANCE);
 
   if (!ctx->restart) ctx->restart = 0.9;
@@ -78,13 +80,13 @@ PetscErrorCode EPSSetUp_LOBPCG(EPS eps)
   PetscCall(DSSetType(eps->ds,DSGHEP));
   PetscCall(DSAllocate(eps->ds,eps->mpd));
   PetscCall(EPSSetWorkVecs(eps,1));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode EPSSolve_LOBPCG(EPS eps)
+static PetscErrorCode EPSSolve_LOBPCG(EPS eps)
 {
   EPS_LOBPCG     *ctx = (EPS_LOBPCG*)eps->data;
-  PetscInt       i,j,k,ld,nv,ini,nmat,nc,nconv,locked,its,prev=0;
+  PetscInt       i,j,k,nv,ini,nmat,nc,nconv,locked,its,prev=0;
   PetscReal      norm;
   PetscScalar    *eigr,dot;
   PetscBool      breakdown,countc,flip=PETSC_FALSE,checkprecond=PETSC_FALSE;
@@ -94,7 +96,6 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
   SlepcSC        sc;
 
   PetscFunctionBegin;
-  PetscCall(DSGetLeadingDimension(eps->ds,&ld));
   PetscCall(STGetNumMatrices(eps->st,&nmat));
   PetscCall(STGetMatrix(eps->st,0,&A));
   if (nmat>1) PetscCall(STGetMatrix(eps->st,1,&B));
@@ -157,7 +158,7 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
   PetscCall(DSGetMat(eps->ds,DS_MAT_X,&M));
   PetscCall(BVMultInPlace(X,M,0,nv));
   PetscCall(BVMultInPlace(AX,M,0,nv));
-  PetscCall(MatDestroy(&M));
+  PetscCall(DSRestoreMat(eps->ds,DS_MAT_X,&M));
 
   /* 5. Initialize range of active iterates */
   locked = 0;  /* hard-locked vectors, the leading locked columns of V are eigenvectors */
@@ -275,7 +276,7 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
       PetscCall(DSGetMat(eps->ds,DS_MAT_X,&M));
       PetscCall(BVMultInPlace(X,M,0,nv));
       PetscCall(BVMultInPlace(AX,M,0,nv));
-      PetscCall(MatDestroy(&M));
+      PetscCall(DSRestoreMat(eps->ds,DS_MAT_X,&M));
 
       continue;   /* skip the rest of the iteration */
     }
@@ -375,7 +376,7 @@ PetscErrorCode EPSSolve_LOBPCG(EPS eps)
     PetscCall(BVMult(X,1.0,1.0,Z,M));
     if (ctx->lock) PetscCall(BVSetActiveColumns(X,nconv,ctx->bs));
     PetscCall(BVMatMult(X,A,AX));
-    PetscCall(MatDestroy(&M));
+    PetscCall(DSRestoreMat(eps->ds,DS_MAT_X,&M));
   }
 
 diverged:
@@ -392,7 +393,7 @@ diverged:
   PetscCall(BVDestroy(&AX));
   if (B) PetscCall(BVDestroy(&BX));
   if (nc>0 || eps->nev>ctx->bs-ctx->guard) PetscCall(BVDestroy(&Y));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode EPSLOBPCGSetBlockSize_LOBPCG(EPS eps,PetscInt bs)
@@ -400,19 +401,19 @@ static PetscErrorCode EPSLOBPCGSetBlockSize_LOBPCG(EPS eps,PetscInt bs)
   EPS_LOBPCG *ctx = (EPS_LOBPCG*)eps->data;
 
   PetscFunctionBegin;
-  if (bs == PETSC_DEFAULT || bs == PETSC_DECIDE) bs = 1;
-  PetscCheck(bs>0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Invalid block size %" PetscInt_FMT,bs);
+  if (bs == PETSC_DEFAULT || bs == PETSC_DECIDE) bs = 0;
+  else PetscCheck(bs>0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Invalid block size %" PetscInt_FMT,bs);
   if (ctx->bs != bs) {
     ctx->bs = bs;
     eps->state = EPS_STATE_INITIAL;
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    EPSLOBPCGSetBlockSize - Sets the block size of the LOBPCG method.
 
-   Logically Collective on eps
+   Logically Collective
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -431,7 +432,7 @@ PetscErrorCode EPSLOBPCGSetBlockSize(EPS eps,PetscInt bs)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,bs,2);
   PetscTryMethod(eps,"EPSLOBPCGSetBlockSize_C",(EPS,PetscInt),(eps,bs));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode EPSLOBPCGGetBlockSize_LOBPCG(EPS eps,PetscInt *bs)
@@ -440,7 +441,7 @@ static PetscErrorCode EPSLOBPCGGetBlockSize_LOBPCG(EPS eps,PetscInt *bs)
 
   PetscFunctionBegin;
   *bs = ctx->bs;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -462,9 +463,9 @@ PetscErrorCode EPSLOBPCGGetBlockSize(EPS eps,PetscInt *bs)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidIntPointer(bs,2);
+  PetscAssertPointer(bs,2);
   PetscUseMethod(eps,"EPSLOBPCGGetBlockSize_C",(EPS,PetscInt*),(eps,bs));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode EPSLOBPCGSetRestart_LOBPCG(EPS eps,PetscReal restart)
@@ -472,19 +473,19 @@ static PetscErrorCode EPSLOBPCGSetRestart_LOBPCG(EPS eps,PetscReal restart)
   EPS_LOBPCG *ctx = (EPS_LOBPCG*)eps->data;
 
   PetscFunctionBegin;
-  if (restart==PETSC_DEFAULT) restart = 0.9;
+  if (restart==(PetscReal)PETSC_DEFAULT || restart==(PetscReal)PETSC_DECIDE) restart = 0.9;
   PetscCheck(restart>=0.1 && restart<=1.0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"The restart argument %g must be in the range [0.1,1.0]",(double)restart);
   if (restart != ctx->restart) {
     ctx->restart = restart;
     eps->state = EPS_STATE_INITIAL;
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    EPSLOBPCGSetRestart - Sets the restart parameter for the LOBPCG method.
 
-   Logically Collective on eps
+   Logically Collective
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -509,7 +510,7 @@ PetscErrorCode EPSLOBPCGSetRestart(EPS eps,PetscReal restart)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveReal(eps,restart,2);
   PetscTryMethod(eps,"EPSLOBPCGSetRestart_C",(EPS,PetscReal),(eps,restart));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode EPSLOBPCGGetRestart_LOBPCG(EPS eps,PetscReal *restart)
@@ -518,7 +519,7 @@ static PetscErrorCode EPSLOBPCGGetRestart_LOBPCG(EPS eps,PetscReal *restart)
 
   PetscFunctionBegin;
   *restart = ctx->restart;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -540,9 +541,9 @@ PetscErrorCode EPSLOBPCGGetRestart(EPS eps,PetscReal *restart)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidRealPointer(restart,2);
+  PetscAssertPointer(restart,2);
   PetscUseMethod(eps,"EPSLOBPCGGetRestart_C",(EPS,PetscReal*),(eps,restart));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode EPSLOBPCGSetLocking_LOBPCG(EPS eps,PetscBool lock)
@@ -551,14 +552,14 @@ static PetscErrorCode EPSLOBPCGSetLocking_LOBPCG(EPS eps,PetscBool lock)
 
   PetscFunctionBegin;
   ctx->lock = lock;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    EPSLOBPCGSetLocking - Choose between locking and non-locking variants of
    the LOBPCG method.
 
-   Logically Collective on eps
+   Logically Collective
 
    Input Parameters:
 +  eps  - the eigenproblem solver context
@@ -582,7 +583,7 @@ PetscErrorCode EPSLOBPCGSetLocking(EPS eps,PetscBool lock)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveBool(eps,lock,2);
   PetscTryMethod(eps,"EPSLOBPCGSetLocking_C",(EPS,PetscBool),(eps,lock));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode EPSLOBPCGGetLocking_LOBPCG(EPS eps,PetscBool *lock)
@@ -591,7 +592,7 @@ static PetscErrorCode EPSLOBPCGGetLocking_LOBPCG(EPS eps,PetscBool *lock)
 
   PetscFunctionBegin;
   *lock = ctx->lock;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -613,12 +614,12 @@ PetscErrorCode EPSLOBPCGGetLocking(EPS eps,PetscBool *lock)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidBoolPointer(lock,2);
+  PetscAssertPointer(lock,2);
   PetscUseMethod(eps,"EPSLOBPCGGetLocking_C",(EPS,PetscBool*),(eps,lock));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode EPSView_LOBPCG(EPS eps,PetscViewer viewer)
+static PetscErrorCode EPSView_LOBPCG(EPS eps,PetscViewer viewer)
 {
   EPS_LOBPCG     *ctx = (EPS_LOBPCG*)eps->data;
   PetscBool      isascii;
@@ -630,17 +631,17 @@ PetscErrorCode EPSView_LOBPCG(EPS eps,PetscViewer viewer)
     PetscCall(PetscViewerASCIIPrintf(viewer,"  restart parameter=%g (using %" PetscInt_FMT " guard vectors)\n",(double)ctx->restart,ctx->guard));
     PetscCall(PetscViewerASCIIPrintf(viewer,"  soft locking %sactivated\n",ctx->lock?"":"de"));
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode EPSSetFromOptions_LOBPCG(PetscOptionItems *PetscOptionsObject,EPS eps)
+static PetscErrorCode EPSSetFromOptions_LOBPCG(EPS eps,PetscOptionItems PetscOptionsObject)
 {
   PetscBool      lock,flg;
   PetscInt       bs;
   PetscReal      restart;
 
   PetscFunctionBegin;
-  PetscCall(PetscOptionsHead(PetscOptionsObject,"EPS LOBPCG Options"));
+  PetscOptionsHeadBegin(PetscOptionsObject,"EPS LOBPCG Options");
 
     PetscCall(PetscOptionsInt("-eps_lobpcg_blocksize","Block size","EPSLOBPCGSetBlockSize",20,&bs,&flg));
     if (flg) PetscCall(EPSLOBPCGSetBlockSize(eps,bs));
@@ -651,11 +652,11 @@ PetscErrorCode EPSSetFromOptions_LOBPCG(PetscOptionItems *PetscOptionsObject,EPS
     PetscCall(PetscOptionsBool("-eps_lobpcg_locking","Choose between locking and non-locking variants","EPSLOBPCGSetLocking",PETSC_TRUE,&lock,&flg));
     if (flg) PetscCall(EPSLOBPCGSetLocking(eps,lock));
 
-  PetscCall(PetscOptionsTail());
-  PetscFunctionReturn(0);
+  PetscOptionsHeadEnd();
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode EPSDestroy_LOBPCG(EPS eps)
+static PetscErrorCode EPSDestroy_LOBPCG(EPS eps)
 {
   PetscFunctionBegin;
   PetscCall(PetscFree(eps->data));
@@ -665,7 +666,7 @@ PetscErrorCode EPSDestroy_LOBPCG(EPS eps)
   PetscCall(PetscObjectComposeFunction((PetscObject)eps,"EPSLOBPCGGetRestart_C",NULL));
   PetscCall(PetscObjectComposeFunction((PetscObject)eps,"EPSLOBPCGSetLocking_C",NULL));
   PetscCall(PetscObjectComposeFunction((PetscObject)eps,"EPSLOBPCGGetLocking_C",NULL));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 SLEPC_EXTERN PetscErrorCode EPSCreate_LOBPCG(EPS eps)
@@ -673,7 +674,7 @@ SLEPC_EXTERN PetscErrorCode EPSCreate_LOBPCG(EPS eps)
   EPS_LOBPCG     *lobpcg;
 
   PetscFunctionBegin;
-  PetscCall(PetscNewLog(eps,&lobpcg));
+  PetscCall(PetscNew(&lobpcg));
   eps->data = (void*)lobpcg;
   lobpcg->lock = PETSC_TRUE;
 
@@ -695,5 +696,5 @@ SLEPC_EXTERN PetscErrorCode EPSCreate_LOBPCG(EPS eps)
   PetscCall(PetscObjectComposeFunction((PetscObject)eps,"EPSLOBPCGGetRestart_C",EPSLOBPCGGetRestart_LOBPCG));
   PetscCall(PetscObjectComposeFunction((PetscObject)eps,"EPSLOBPCGSetLocking_C",EPSLOBPCGSetLocking_LOBPCG));
   PetscCall(PetscObjectComposeFunction((PetscObject)eps,"EPSLOBPCGGetLocking_C",EPSLOBPCGGetLocking_LOBPCG));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
