@@ -81,14 +81,14 @@ static void monitorFun(void *basisEvals,int *basisSize,int *basisFlags,int *iblo
 #endif
                        primme_event *event,struct primme_params *primme,int *err)
 {
-  PetscErrorCode ierr = 0;
+  PetscErrorCode ierr = PETSC_SUCCESS;
   EPS            eps = (EPS)primme->commInfo;
   PetscInt       i,k,nerrest;
 
   switch (*event) {
     case primme_event_outer_iteration:
       /* Update EPS */
-      eps->its = primme->stats.numOuterIterations;
+      PetscCallVoid(PetscIntCast(primme->stats.numOuterIterations,&eps->its));
       eps->nconv = primme->initSize;
       k=0;
       if (lockedEvals && numLocked) for (i=0; i<*numLocked && k<eps->ncv; i++) eps->eigr[k++] = ((PetscReal*)lockedEvals)[i];
@@ -169,7 +169,7 @@ static void applyPreconditioner_PRIMME(void *xa,PRIMME_INT *ldx,void *ya,PRIMME_
   PetscFunctionReturnVoid();
 }
 
-PetscErrorCode EPSSetUp_PRIMME(EPS eps)
+static PetscErrorCode EPSSetUp_PRIMME(EPS eps)
 {
   PetscMPIInt    numProcs,procID;
   EPS_PRIMME     *ops = (EPS_PRIMME*)eps->data;
@@ -178,11 +178,13 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
 
   PetscFunctionBegin;
   EPSCheckHermitianDefinite(eps);
+  EPSCheckNotStructured(eps);
+  if (eps->nev==0) eps->nev = 1;
   PetscCallMPI(MPI_Comm_size(PetscObjectComm((PetscObject)eps),&numProcs));
   PetscCallMPI(MPI_Comm_rank(PetscObjectComm((PetscObject)eps),&procID));
 
   /* Check some constraints and set some default values */
-  if (eps->max_it==PETSC_DEFAULT) eps->max_it = PETSC_MAX_INT;
+  if (eps->max_it==PETSC_DETERMINE) eps->max_it = PETSC_INT_MAX;
   PetscCall(STGetMatrix(eps->st,0,&ops->A));
   if (eps->isgeneralized) {
 #if defined(SLEPC_HAVE_PRIMME3)
@@ -203,9 +205,9 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   /* Transfer SLEPc options to PRIMME options */
   primme_free(primme);
   primme_initialize(primme);
-  primme->n                             = eps->n;
-  primme->nLocal                        = eps->nloc;
-  primme->numEvals                      = eps->nev;
+  primme->n                             = (PRIMME_INT)eps->n;
+  primme->nLocal                        = (PRIMME_INT)eps->nloc;
+  primme->numEvals                      = (int)eps->nev;
   primme->matrix                        = ops;
   primme->matrixMatvec                  = matrixMatvec_PRIMME;
 #if defined(SLEPC_HAVE_PRIMME3)
@@ -215,7 +217,7 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   }
 #endif
   primme->commInfo                      = eps;
-  primme->maxOuterIterations            = eps->max_it;
+  primme->maxOuterIterations            = (PRIMME_INT)eps->max_it;
 #if !defined(SLEPC_HAVE_PRIMME2p2)
   primme->eps                           = SlepcDefaultTol(eps->tol);
 #endif
@@ -231,7 +233,7 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   primme->convTestFun                   = convTestFun;
   primme->monitorFun                    = monitorFun;
 #endif
-  if (ops->bs > 0) primme->maxBlockSize = ops->bs;
+  if (ops->bs > 0) primme->maxBlockSize = (int)ops->bs;
 
   switch (eps->which) {
     case EPS_LARGEST_REAL:
@@ -278,16 +280,16 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   }
 
   /* If user sets mpd or ncv, maxBasisSize is modified */
-  if (eps->mpd!=PETSC_DEFAULT) {
-    primme->maxBasisSize = eps->mpd;
-    if (eps->ncv!=PETSC_DEFAULT) PetscCall(PetscInfo(eps,"Warning: 'ncv' is ignored by PRIMME\n"));
-  } else if (eps->ncv!=PETSC_DEFAULT) primme->maxBasisSize = eps->ncv;
+  if (eps->mpd!=PETSC_DETERMINE) {
+    primme->maxBasisSize = (int)eps->mpd;
+    if (eps->ncv!=PETSC_DETERMINE) PetscCall(PetscInfo(eps,"Warning: 'ncv' is ignored by PRIMME\n"));
+  } else if (eps->ncv!=PETSC_DETERMINE) primme->maxBasisSize = (int)eps->ncv;
 
   PetscCheck(primme_set_method(ops->method,primme)>=0,PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"PRIMME method not valid");
 
-  eps->mpd = primme->maxBasisSize;
-  eps->ncv = (primme->locking?eps->nev:0)+primme->maxBasisSize;
-  ops->bs  = primme->maxBlockSize;
+  eps->mpd = (PetscInt)primme->maxBasisSize;
+  eps->ncv = (PetscInt)(primme->locking?eps->nev:0)+primme->maxBasisSize;
+  ops->bs  = (PetscInt)primme->maxBlockSize;
 
   /* Set workspace */
   PetscCall(EPSAllocateSolution(eps,0));
@@ -302,19 +304,15 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   }
 
   /* Prepare auxiliary vectors */
-  if (!ops->x) {
-    PetscCall(MatCreateVecsEmpty(ops->A,&ops->x,&ops->y));
-    PetscCall(PetscLogObjectParent((PetscObject)eps,(PetscObject)ops->x));
-    PetscCall(PetscLogObjectParent((PetscObject)eps,(PetscObject)ops->y));
-  }
-  PetscFunctionReturn(0);
+  if (!ops->x) PetscCall(MatCreateVecsEmpty(ops->A,&ops->x,&ops->y));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode EPSSolve_PRIMME(EPS eps)
+static PetscErrorCode EPSSolve_PRIMME(EPS eps)
 {
   EPS_PRIMME     *ops = (EPS_PRIMME*)eps->data;
   PetscScalar    *a;
-  PetscInt       i,ierrprimme;
+  PetscInt       i,ld,ierrprimme;
   PetscReal      *evals,*rnorms;
 
   PetscFunctionBegin;
@@ -325,11 +323,13 @@ PetscErrorCode EPSSolve_PRIMME(EPS eps)
   /* Force PRIMME to stop by absolute error */
   ops->primme.aNorm    = 1.0;
 #endif
-  ops->primme.initSize = eps->nini;
+  ops->primme.initSize = (int)eps->nini;
   ops->primme.iseed[0] = -1;
   ops->primme.iseed[1] = -1;
   ops->primme.iseed[2] = -1;
   ops->primme.iseed[3] = -1;
+  PetscCall(BVGetLeadingDimension(eps->V,&ld));
+  ops->primme.ldevecs  = (PRIMME_INT)ld;
 
   /* Call PRIMME solver */
   PetscCall(BVGetArray(eps->V,&a));
@@ -340,9 +340,9 @@ PetscErrorCode EPSSolve_PRIMME(EPS eps)
   PetscCall(PetscFree2(evals,rnorms));
   PetscCall(BVRestoreArray(eps->V,&a));
 
-  eps->nconv  = ops->primme.initSize >= 0 ? ops->primme.initSize : 0;
+  eps->nconv  = ops->primme.initSize >= 0 ? (PetscInt)ops->primme.initSize : 0;
   eps->reason = eps->nconv >= eps->nev ? EPS_CONVERGED_TOL: EPS_DIVERGED_ITS;
-  eps->its    = ops->primme.stats.numOuterIterations;
+  PetscCall(PetscIntCast(ops->primme.stats.numOuterIterations,&eps->its));
 
   /* Process PRIMME error code */
   switch (ierrprimme) {
@@ -358,10 +358,10 @@ PetscErrorCode EPSSolve_PRIMME(EPS eps)
       PetscCheck(ierrprimme<-39,PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"PRIMME library failed with error code=%" PetscInt_FMT ": configuration error; check PRIMME's manual",ierrprimme);
       PetscCheck(ierrprimme>=-39,PetscObjectComm((PetscObject)eps),PETSC_ERR_LIB,"PRIMME library failed with error code=%" PetscInt_FMT ": runtime error; check PRIMME's manual",ierrprimme);
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode EPSReset_PRIMME(EPS eps)
+static PetscErrorCode EPSReset_PRIMME(EPS eps)
 {
   EPS_PRIMME     *ops = (EPS_PRIMME*)eps->data;
 
@@ -369,10 +369,10 @@ PetscErrorCode EPSReset_PRIMME(EPS eps)
   primme_free(&ops->primme);
   PetscCall(VecDestroy(&ops->x));
   PetscCall(VecDestroy(&ops->y));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode EPSDestroy_PRIMME(EPS eps)
+static PetscErrorCode EPSDestroy_PRIMME(EPS eps)
 {
   PetscFunctionBegin;
   PetscCall(PetscFree(eps->data));
@@ -380,10 +380,10 @@ PetscErrorCode EPSDestroy_PRIMME(EPS eps)
   PetscCall(PetscObjectComposeFunction((PetscObject)eps,"EPSPRIMMESetMethod_C",NULL));
   PetscCall(PetscObjectComposeFunction((PetscObject)eps,"EPSPRIMMEGetBlockSize_C",NULL));
   PetscCall(PetscObjectComposeFunction((PetscObject)eps,"EPSPRIMMEGetMethod_C",NULL));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode EPSView_PRIMME(EPS eps,PetscViewer viewer)
+static PetscErrorCode EPSView_PRIMME(EPS eps,PetscViewer viewer)
 {
   PetscBool      isascii;
   EPS_PRIMME     *ctx = (EPS_PRIMME*)eps->data;
@@ -399,10 +399,10 @@ PetscErrorCode EPSView_PRIMME(EPS eps,PetscViewer viewer)
     PetscCallMPI(MPI_Comm_rank(PetscObjectComm((PetscObject)eps),&rank));
     if (!rank) primme_display_params(ctx->primme);
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode EPSSetFromOptions_PRIMME(PetscOptionItems *PetscOptionsObject,EPS eps)
+static PetscErrorCode EPSSetFromOptions_PRIMME(EPS eps,PetscOptionItems PetscOptionsObject)
 {
   EPS_PRIMME      *ctx = (EPS_PRIMME*)eps->data;
   PetscInt        bs;
@@ -410,7 +410,7 @@ PetscErrorCode EPSSetFromOptions_PRIMME(PetscOptionItems *PetscOptionsObject,EPS
   PetscBool       flg;
 
   PetscFunctionBegin;
-  PetscCall(PetscOptionsHead(PetscOptionsObject,"EPS PRIMME Options"));
+  PetscOptionsHeadBegin(PetscOptionsObject,"EPS PRIMME Options");
 
     PetscCall(PetscOptionsInt("-eps_primme_blocksize","Maximum block size","EPSPRIMMESetBlockSize",ctx->bs,&bs,&flg));
     if (flg) PetscCall(EPSPRIMMESetBlockSize(eps,bs));
@@ -418,8 +418,8 @@ PetscErrorCode EPSSetFromOptions_PRIMME(PetscOptionItems *PetscOptionsObject,EPS
     PetscCall(PetscOptionsEnum("-eps_primme_method","Method for solving the eigenproblem","EPSPRIMMESetMethod",EPSPRIMMEMethods,(PetscEnum)ctx->method,(PetscEnum*)&meth,&flg));
     if (flg) PetscCall(EPSPRIMMESetMethod(eps,meth));
 
-  PetscCall(PetscOptionsTail());
-  PetscFunctionReturn(0);
+  PetscOptionsHeadEnd();
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode EPSPRIMMESetBlockSize_PRIMME(EPS eps,PetscInt bs)
@@ -427,16 +427,18 @@ static PetscErrorCode EPSPRIMMESetBlockSize_PRIMME(EPS eps,PetscInt bs)
   EPS_PRIMME *ops = (EPS_PRIMME*)eps->data;
 
   PetscFunctionBegin;
-  if (bs == PETSC_DEFAULT) ops->bs = 0;
-  else PetscCheck(bs>0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"PRIMME: block size must be positive");
-  else ops->bs = bs;
-  PetscFunctionReturn(0);
+  if (bs == PETSC_DEFAULT || bs == PETSC_DECIDE) ops->bs = 0;
+  else {
+    PetscCheck(bs>0,PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"PRIMME: block size must be positive");
+    ops->bs = bs;
+  }
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    EPSPRIMMESetBlockSize - The maximum block size that PRIMME will try to use.
 
-   Logically Collective on eps
+   Logically Collective
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -464,7 +466,7 @@ PetscErrorCode EPSPRIMMESetBlockSize(EPS eps,PetscInt bs)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveInt(eps,bs,2);
   PetscTryMethod(eps,"EPSPRIMMESetBlockSize_C",(EPS,PetscInt),(eps,bs));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode EPSPRIMMEGetBlockSize_PRIMME(EPS eps,PetscInt *bs)
@@ -473,7 +475,7 @@ static PetscErrorCode EPSPRIMMEGetBlockSize_PRIMME(EPS eps,PetscInt *bs)
 
   PetscFunctionBegin;
   *bs = ops->bs;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -495,9 +497,9 @@ PetscErrorCode EPSPRIMMEGetBlockSize(EPS eps,PetscInt *bs)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidIntPointer(bs,2);
+  PetscAssertPointer(bs,2);
   PetscUseMethod(eps,"EPSPRIMMEGetBlockSize_C",(EPS,PetscInt*),(eps,bs));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode EPSPRIMMESetMethod_PRIMME(EPS eps,EPSPRIMMEMethod method)
@@ -506,13 +508,13 @@ static PetscErrorCode EPSPRIMMESetMethod_PRIMME(EPS eps,EPSPRIMMEMethod method)
 
   PetscFunctionBegin;
   ops->method = (primme_preset_method)method;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    EPSPRIMMESetMethod - Sets the method for the PRIMME library.
 
-   Logically Collective on eps
+   Logically Collective
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -534,7 +536,7 @@ PetscErrorCode EPSPRIMMESetMethod(EPS eps,EPSPRIMMEMethod method)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
   PetscValidLogicalCollectiveEnum(eps,method,2);
   PetscTryMethod(eps,"EPSPRIMMESetMethod_C",(EPS,EPSPRIMMEMethod),(eps,method));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode EPSPRIMMEGetMethod_PRIMME(EPS eps,EPSPRIMMEMethod *method)
@@ -543,7 +545,7 @@ static PetscErrorCode EPSPRIMMEGetMethod_PRIMME(EPS eps,EPSPRIMMEMethod *method)
 
   PetscFunctionBegin;
   *method = (EPSPRIMMEMethod)ops->method;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -565,9 +567,9 @@ PetscErrorCode EPSPRIMMEGetMethod(EPS eps,EPSPRIMMEMethod *method)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
-  PetscValidPointer(method,2);
+  PetscAssertPointer(method,2);
   PetscUseMethod(eps,"EPSPRIMMEGetMethod_C",(EPS,EPSPRIMMEMethod*),(eps,method));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 SLEPC_EXTERN PetscErrorCode EPSCreate_PRIMME(EPS eps)
@@ -575,7 +577,7 @@ SLEPC_EXTERN PetscErrorCode EPSCreate_PRIMME(EPS eps)
   EPS_PRIMME     *primme;
 
   PetscFunctionBegin;
-  PetscCall(PetscNewLog(eps,&primme));
+  PetscCall(PetscNew(&primme));
   eps->data = (void*)primme;
 
   primme_initialize(&primme->primme);
@@ -605,5 +607,5 @@ SLEPC_EXTERN PetscErrorCode EPSCreate_PRIMME(EPS eps)
   PetscCall(PetscObjectComposeFunction((PetscObject)eps,"EPSPRIMMESetMethod_C",EPSPRIMMESetMethod_PRIMME));
   PetscCall(PetscObjectComposeFunction((PetscObject)eps,"EPSPRIMMEGetBlockSize_C",EPSPRIMMEGetBlockSize_PRIMME));
   PetscCall(PetscObjectComposeFunction((PetscObject)eps,"EPSPRIMMEGetMethod_C",EPSPRIMMEGetMethod_PRIMME));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
