@@ -14,25 +14,25 @@
 
 #include <slepc/private/epsimpl.h>
 
-PetscErrorCode EPSSetUp_LAPACK(EPS eps)
+static PetscErrorCode EPSSetUp_LAPACK(EPS eps)
 {
-  PetscErrorCode ierra,ierrb;
+  int            ierra,ierrb;
   PetscBool      isshift,flg,denseok=PETSC_FALSE;
-  Mat            A,B,OP,shell,Ar,Br,Adense=NULL,Bdense=NULL;
-  PetscScalar    shift,*Ap,*Bp;
-  PetscInt       i,ld,nmat;
+  Mat            A,B,OP,shell,Ar,Br,Adense=NULL,Bdense=NULL,Ads,Bds;
+  PetscScalar    shift;
+  PetscInt       nmat;
   KSP            ksp;
   PC             pc;
-  Vec            v;
 
   PetscFunctionBegin;
+  if (eps->nev==0) eps->nev = 1;
   eps->ncv = eps->n;
-  if (eps->mpd!=PETSC_DEFAULT) PetscCall(PetscInfo(eps,"Warning: parameter mpd ignored\n"));
-  if (eps->max_it==PETSC_DEFAULT) eps->max_it = 1;
+  if (eps->mpd!=PETSC_DETERMINE) PetscCall(PetscInfo(eps,"Warning: parameter mpd ignored\n"));
+  if (eps->max_it==PETSC_DETERMINE) eps->max_it = 1;
   if (!eps->which) PetscCall(EPSSetWhichEigenpairs_Default(eps));
   PetscCheck(eps->which!=EPS_ALL || eps->inta==eps->intb,PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support interval computation");
-  EPSCheckUnsupported(eps,EPS_FEATURE_BALANCE | EPS_FEATURE_ARBITRARY | EPS_FEATURE_REGION | EPS_FEATURE_STOPPING);
-  EPSCheckIgnored(eps,EPS_FEATURE_EXTRACTION | EPS_FEATURE_CONVERGENCE);
+  EPSCheckUnsupported(eps,EPS_FEATURE_BALANCE | EPS_FEATURE_ARBITRARY | EPS_FEATURE_REGION);
+  EPSCheckIgnored(eps,EPS_FEATURE_EXTRACTION | EPS_FEATURE_CONVERGENCE | EPS_FEATURE_STOPPING);
   PetscCall(EPSAllocateSolution(eps,0));
 
   /* attempt to get dense representations of A and B separately */
@@ -42,21 +42,21 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
     PetscCall(STGetMatrix(eps->st,0,&A));
     PetscCall(MatHasOperation(A,MATOP_CREATE_SUBMATRICES,&flg));
     if (flg) {
-      PetscPushErrorHandler(PetscReturnErrorHandler,NULL);
+      PetscCall(PetscPushErrorHandler(PetscReturnErrorHandler,NULL));
       ierra  = MatCreateRedundantMatrix(A,0,PETSC_COMM_SELF,MAT_INITIAL_MATRIX,&Ar);
       if (!ierra) ierra |= MatConvert(Ar,MATSEQDENSE,MAT_INITIAL_MATRIX,&Adense);
       ierra |= MatDestroy(&Ar);
-      PetscPopErrorHandler();
+      PetscCall(PetscPopErrorHandler());
     } else ierra = 1;
     if (nmat>1) {
       PetscCall(STGetMatrix(eps->st,1,&B));
       PetscCall(MatHasOperation(B,MATOP_CREATE_SUBMATRICES,&flg));
       if (flg) {
-        PetscPushErrorHandler(PetscReturnErrorHandler,NULL);
+        PetscCall(PetscPushErrorHandler(PetscReturnErrorHandler,NULL));
         ierrb  = MatCreateRedundantMatrix(B,0,PETSC_COMM_SELF,MAT_INITIAL_MATRIX,&Br);
         if (!ierrb) ierrb |= MatConvert(Br,MATSEQDENSE,MAT_INITIAL_MATRIX,&Bdense);
         ierrb |= MatDestroy(&Br);
-        PetscPopErrorHandler();
+        PetscCall(PetscPopErrorHandler());
       } else ierrb = 1;
     } else ierrb = 0;
     denseok = PetscNot(ierra || ierrb);
@@ -75,7 +75,6 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
     }
   } else PetscCall(DSSetType(eps->ds,DSNHEP));
   PetscCall(DSAllocate(eps->ds,eps->ncv));
-  PetscCall(DSGetLeadingDimension(eps->ds,&ld));
   PetscCall(DSSetDimensions(eps->ds,eps->ncv,0,0));
 
   if (denseok) {
@@ -100,31 +99,21 @@ PetscErrorCode EPSSetUp_LAPACK(EPS eps)
   }
 
   /* fill DS matrices */
-  PetscCall(VecCreateSeqWithArray(PETSC_COMM_SELF,1,ld,NULL,&v));
-  PetscCall(DSGetArray(eps->ds,DS_MAT_A,&Ap));
-  for (i=0;i<ld;i++) {
-    PetscCall(VecPlaceArray(v,Ap+i*ld));
-    PetscCall(MatGetColumnVector(Adense,v,i));
-    PetscCall(VecResetArray(v));
-  }
-  PetscCall(DSRestoreArray(eps->ds,DS_MAT_A,&Ap));
+  PetscCall(DSGetMat(eps->ds,DS_MAT_A,&Ads));
+  PetscCall(MatCopy(Adense,Ads,SAME_NONZERO_PATTERN));
+  PetscCall(DSRestoreMat(eps->ds,DS_MAT_A,&Ads));
   if (denseok && eps->isgeneralized) {
-    PetscCall(DSGetArray(eps->ds,DS_MAT_B,&Bp));
-    for (i=0;i<ld;i++) {
-      PetscCall(VecPlaceArray(v,Bp+i*ld));
-      PetscCall(MatGetColumnVector(Bdense,v,i));
-      PetscCall(VecResetArray(v));
-    }
-    PetscCall(DSRestoreArray(eps->ds,DS_MAT_B,&Bp));
+    PetscCall(DSGetMat(eps->ds,DS_MAT_B,&Bds));
+    PetscCall(MatCopy(Bdense,Bds,SAME_NONZERO_PATTERN));
+    PetscCall(DSRestoreMat(eps->ds,DS_MAT_B,&Bds));
   }
-  PetscCall(VecDestroy(&v));
   PetscCall(DSSetState(eps->ds,DS_STATE_RAW));
   PetscCall(MatDestroy(&Adense));
   PetscCall(MatDestroy(&Bdense));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode EPSSolve_LAPACK(EPS eps)
+static PetscErrorCode EPSSolve_LAPACK(EPS eps)
 {
   PetscInt       n=eps->n,i,low,high;
   PetscScalar    *array,*pX,*pY;
@@ -166,7 +155,7 @@ PetscErrorCode EPSSolve_LAPACK(EPS eps)
   eps->nconv  = eps->ncv;
   eps->its    = 1;
   eps->reason = EPS_CONVERGED_TOL;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 SLEPC_EXTERN PetscErrorCode EPSCreate_LAPACK(EPS eps)
@@ -179,5 +168,5 @@ SLEPC_EXTERN PetscErrorCode EPSCreate_LAPACK(EPS eps)
   eps->ops->setup          = EPSSetUp_LAPACK;
   eps->ops->setupsort      = EPSSetUpSort_Default;
   eps->ops->backtransform  = EPSBackTransform_Default;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
