@@ -18,7 +18,7 @@ PetscErrorCode EPSBackTransform_Default(EPS eps)
 {
   PetscFunctionBegin;
   PetscCall(STBackTransform(eps->st,eps->nconv,eps->eigr,eps->eigi));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -46,7 +46,7 @@ PetscErrorCode EPSComputeVectors_Hermitian(EPS eps)
       PetscCall(BVSetMatrix(eps->V,C,PETSC_FALSE));  /* restore original matrix */
     }
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -63,14 +63,14 @@ PetscErrorCode EPSComputeVectors_Indefinite(EPS eps)
   PetscCall(DSVectors(eps->ds,DS_MAT_X,NULL,NULL));
   PetscCall(DSGetMat(eps->ds,DS_MAT_X,&X));
   PetscCall(BVMultInPlace(eps->V,X,0,n));
-  PetscCall(MatDestroy(&X));
+  PetscCall(DSRestoreMat(eps->ds,DS_MAT_X,&X));
 
   /* purification */
   if (eps->purify) PetscCall(EPS_Purify(eps,eps->nconv));
 
   /* normalization */
   PetscCall(BVNormalize(eps->V,eps->eigi));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -82,18 +82,16 @@ PetscErrorCode EPSComputeVectors_Twosided(EPS eps)
   Vec            w,y;
 
   PetscFunctionBegin;
-  if (!eps->twosided || !eps->isgeneralized) PetscFunctionReturn(0);
+  if (!eps->twosided || !eps->isgeneralized) PetscFunctionReturn(PETSC_SUCCESS);
   PetscCall(EPSSetWorkVecs(eps,1));
   w = eps->work[0];
   for (i=0;i<eps->nconv;i++) {
     PetscCall(BVCopyVec(eps->W,i,w));
-    PetscCall(VecConjugate(w));
     PetscCall(BVGetColumn(eps->W,i,&y));
-    PetscCall(STMatSolveTranspose(eps->st,w,y));
-    PetscCall(VecConjugate(y));
+    PetscCall(STMatSolveHermitianTranspose(eps->st,w,y));
     PetscCall(BVRestoreColumn(eps->W,i,&y));
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -114,7 +112,7 @@ PetscErrorCode EPSComputeVectors_Schur(EPS eps)
   if (eps->ishermitian) {
     if (eps->isgeneralized && !eps->ispositive) PetscCall(EPSComputeVectors_Indefinite(eps));
     else PetscCall(EPSComputeVectors_Hermitian(eps));
-    PetscFunctionReturn(0);
+    PetscFunctionReturn(PETSC_SUCCESS);
   }
 
   /* right eigenvectors */
@@ -123,7 +121,7 @@ PetscErrorCode EPSComputeVectors_Schur(EPS eps)
   /* V = V * Z */
   PetscCall(DSGetMat(eps->ds,DS_MAT_X,&Z));
   PetscCall(BVMultInPlace(eps->V,Z,0,eps->nconv));
-  PetscCall(MatDestroy(&Z));
+  PetscCall(DSRestoreMat(eps->ds,DS_MAT_X,&Z));
 
   /* Purify eigenvectors */
   if (eps->purify) PetscCall(EPS_Purify(eps,eps->nconv));
@@ -146,7 +144,7 @@ PetscErrorCode EPSComputeVectors_Schur(EPS eps)
     /* W = W * Z */
     PetscCall(DSGetMat(eps->ds,DS_MAT_Y,&Z));
     PetscCall(BVMultInPlace(eps->W,Z,0,eps->nconv));
-    PetscCall(MatDestroy(&Z));
+    PetscCall(DSRestoreMat(eps->ds,DS_MAT_Y,&Z));
     /* Fix left eigenvectors if balancing was used */
     if (eps->balance!=EPS_BALANCE_NONE && eps->D) {
       for (i=0;i<eps->nconv;i++) {
@@ -167,13 +165,13 @@ PetscErrorCode EPSComputeVectors_Schur(EPS eps)
     }
 #endif
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    EPSSetWorkVecs - Sets a number of work vectors into an EPS object.
 
-   Collective on eps
+   Collective
 
    Input Parameters:
 +  eps - eigensolver context
@@ -201,9 +199,8 @@ PetscErrorCode EPSSetWorkVecs(EPS eps,PetscInt nw)
     PetscCall(BVGetColumn(eps->V,0,&t));
     PetscCall(VecDuplicateVecs(t,nw,&eps->work));
     PetscCall(BVRestoreColumn(eps->V,0,&t));
-    PetscCall(PetscLogObjectParents(eps,nw,eps->work));
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -218,7 +215,7 @@ PetscErrorCode EPSSetWhichEigenpairs_Default(EPS eps)
   PetscCall(PetscObjectTypeCompareAny((PetscObject)eps->st,&target,STSINVERT,STCAYLEY,""));
   if (target) eps->which = EPS_TARGET_MAGNITUDE;
   else eps->which = EPS_LARGEST_MAGNITUDE;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -230,8 +227,8 @@ PetscErrorCode EPSConvergedRelative(EPS eps,PetscScalar eigr,PetscScalar eigi,Pe
 
   PetscFunctionBegin;
   w = SlepcAbsEigenvalue(eigr,eigi);
-  *errest = res/w;
-  PetscFunctionReturn(0);
+  *errest = (w!=0.0)? res/w: PETSC_MAX_REAL;
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -241,7 +238,7 @@ PetscErrorCode EPSConvergedAbsolute(EPS eps,PetscScalar eigr,PetscScalar eigi,Pe
 {
   PetscFunctionBegin;
   *errest = res;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -255,14 +252,14 @@ PetscErrorCode EPSConvergedNorm(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscR
   PetscFunctionBegin;
   w = SlepcAbsEigenvalue(eigr,eigi);
   *errest = res / (eps->nrma + w*eps->nrmb);
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
    EPSStoppingBasic - Default routine to determine whether the outer eigensolver
    iteration must be stopped.
 
-   Collective on eps
+   Collective
 
    Input Parameters:
 +  eps    - eigensolver context obtained from EPSCreate()
@@ -288,7 +285,7 @@ PetscErrorCode EPSConvergedNorm(EPS eps,PetscScalar eigr,PetscScalar eigi,PetscR
 
    Level: advanced
 
-.seealso: EPSSetStoppingTest(), EPSConvergedReason, EPSGetConvergedReason()
+.seealso: EPSSetStoppingTest(), EPSStoppingThreshold(), EPSConvergedReason, EPSGetConvergedReason()
 @*/
 PetscErrorCode EPSStoppingBasic(EPS eps,PetscInt its,PetscInt max_it,PetscInt nconv,PetscInt nev,EPSConvergedReason *reason,void *ctx)
 {
@@ -301,7 +298,80 @@ PetscErrorCode EPSStoppingBasic(EPS eps,PetscInt its,PetscInt max_it,PetscInt nc
     *reason = EPS_DIVERGED_ITS;
     PetscCall(PetscInfo(eps,"Linear eigensolver iteration reached maximum number of iterations (%" PetscInt_FMT ")\n",its));
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
+}
+
+/*@C
+   EPSStoppingThreshold - Routine to determine whether the outer eigenvalue solver
+   iteration must be stopped, according to some threshold for the computed values.
+
+   Collective
+
+   Input Parameters:
++  eps    - eigenvalue solver context obtained from EPSCreate()
+.  its    - current number of iterations
+.  max_it - maximum number of iterations
+.  nconv  - number of currently converged eigenpairs (ignored here)
+.  nev    - number of requested eigenpairs (ignored here)
+-  ctx    - context containing additional data (EPSStoppingCtx)
+
+   Output Parameter:
+.  reason - result of the stopping test
+
+   Notes:
+   A positive value of reason indicates that the iteration has finished successfully
+   (converged), and a negative value indicates an error condition (diverged). If
+   the iteration needs to be continued, reason must be set to EPS_CONVERGED_ITERATING
+   (zero).
+
+   EPSStoppingThreshold() will stop when one of the computed eigenvalues is not
+   above/below the threshold given at EPSSetThreshold(). If a number of wanted
+   eigenvalues has been specified via EPSSetDimensions() then it is also taken into
+   account, and the solver will stop when one of the two conditions (threshold or
+   number of converged values) is met.
+
+   Use EPSSetStoppingTest() to provide your own test instead of using this one.
+
+   Level: advanced
+
+.seealso: EPSSetStoppingTest(), EPSStoppingBasic(), EPSSetThreshold(), EPSSetDimensions(), EPSConvergedReason, EPSGetConvergedReason()
+@*/
+PetscErrorCode EPSStoppingThreshold(EPS eps,PetscInt its,PetscInt max_it,PetscInt nconv,PetscInt nev,EPSConvergedReason *reason,void *ctx)
+{
+  PetscReal thres,firstev,lastev;
+  PetscBool magnit,rel;
+  EPSWhich  which;
+
+  PetscFunctionBegin;
+  *reason = EPS_CONVERGED_ITERATING;
+  firstev = ((EPSStoppingCtx)ctx)->firstev;
+  lastev  = ((EPSStoppingCtx)ctx)->lastev;
+  thres   = ((EPSStoppingCtx)ctx)->thres;
+  rel     = ((EPSStoppingCtx)ctx)->threlative;
+  which   = ((EPSStoppingCtx)ctx)->which;
+  magnit  = (which==EPS_SMALLEST_MAGNITUDE || which==EPS_LARGEST_MAGNITUDE || which==EPS_TARGET_MAGNITUDE)? PETSC_TRUE: PETSC_FALSE;
+  if (nconv && magnit && which==EPS_TARGET_MAGNITUDE && ((rel && ((thres>1.0 && lastev>thres*firstev) || (thres<1.0 && lastev<thres*firstev))) || (!rel && lastev>thres))) {
+    if (!rel) PetscCall(PetscInfo(eps,"Linear eigensolver finished successfully: the eigenvalue magnitude %g is above the threshold %g\n",(double)lastev,(double)thres));
+    else if (thres>1.0) PetscCall(PetscInfo(eps,"Linear eigensolver finished successfully: the ratio %g/%g is above the threshold %g\n",(double)lastev,(double)firstev,(double)thres));
+    else PetscCall(PetscInfo(eps,"Linear eigensolver finished successfully: the ratio %g/%g is below the threshold %g\n",(double)lastev,(double)firstev,(double)thres));
+    *reason = EPS_CONVERGED_TOL;
+  } else if (nconv && magnit && ((which==EPS_LARGEST_MAGNITUDE && ((rel && lastev<thres*firstev) || (!rel && lastev<thres))) || (which==EPS_SMALLEST_MAGNITUDE && lastev>thres))) {
+    if (which==EPS_SMALLEST_MAGNITUDE) PetscCall(PetscInfo(eps,"Linear eigensolver finished successfully: the eigenvalue magnitude %g is above the threshold %g\n",(double)lastev,(double)thres));
+    else if (!rel) PetscCall(PetscInfo(eps,"Linear eigensolver finished successfully: the eigenvalue magnitude %g is below the threshold %g\n",(double)lastev,(double)thres));
+    else PetscCall(PetscInfo(eps,"Linear eigensolver finished successfully: the ratio %g/%g is below the threshold %g\n",(double)lastev,(double)firstev,(double)thres));
+    *reason = EPS_CONVERGED_TOL;
+  } else if (nconv && !magnit && ((which==EPS_LARGEST_REAL && lastev<thres) || (which==EPS_SMALLEST_REAL && lastev>thres))) {
+    if (which==EPS_LARGEST_REAL) PetscCall(PetscInfo(eps,"Linear eigensolver finished successfully: eigenvalue %g is below the threshold %g\n",(double)lastev,(double)thres));
+    else PetscCall(PetscInfo(eps,"Linear eigensolver finished successfully: eigenvalue %g is above the threshold %g\n",(double)lastev,(double)thres));
+    *reason = EPS_CONVERGED_TOL;
+  } else if (nev && nconv >= nev) {
+    PetscCall(PetscInfo(eps,"Linear eigensolver finished successfully: %" PetscInt_FMT " eigenpairs converged at iteration %" PetscInt_FMT "\n",nconv,its));
+    *reason = EPS_CONVERGED_TOL;
+  } else if (its >= max_it) {
+    *reason = EPS_DIVERGED_ITS;
+    PetscCall(PetscInfo(eps,"Linear eigensolver iteration reached maximum number of iterations (%" PetscInt_FMT ")\n",its));
+  }
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -363,7 +433,7 @@ PetscErrorCode EPSComputeRitzVector(EPS eps,PetscScalar *Zr,PetscScalar *Zi,BV V
     PetscCall(VecNormalize(x,NULL));
   }
   PetscCall(BVSetActiveColumns(V,l,k));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -430,5 +500,5 @@ PetscErrorCode EPSBuildBalance_Krylov(EPS eps)
     PetscCall(VecRestoreArrayRead(p,&pp));
     PetscCall(VecRestoreArray(eps->D,&pD));
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
