@@ -56,7 +56,8 @@ int main(int argc,char **argv)
   PetscBool      terse;
   PetscViewer    viewer;
 
-  PetscCall(SlepcInitialize(&argc,&argv,(char*)0,help));
+  PetscFunctionBeginUser;
+  PetscCall(SlepcInitialize(&argc,&argv,NULL,help));
 
   PetscCall(PetscOptionsGetInt(NULL,NULL,"-n",&N,NULL));
   PetscCall(PetscPrintf(PETSC_COMM_WORLD,"\nBrusselator wave model, n=%" PetscInt_FMT "\n\n",N));
@@ -90,7 +91,6 @@ int main(int argc,char **argv)
   PetscCall(MatCreate(PETSC_COMM_WORLD,&ctx->T));
   PetscCall(MatSetSizes(ctx->T,PETSC_DECIDE,PETSC_DECIDE,N,N));
   PetscCall(MatSetFromOptions(ctx->T));
-  PetscCall(MatSetUp(ctx->T));
 
   PetscCall(MatGetOwnershipRange(ctx->T,&Istart,&Iend));
   for (i=Istart;i<Iend;i++) {
@@ -221,7 +221,7 @@ PetscErrorCode MatMult_Brussel(Mat A,Vec x,Vec y)
   PetscCall(VecResetArray(ctx->x2));
   PetscCall(VecResetArray(ctx->y1));
   PetscCall(VecResetArray(ctx->y2));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 PetscErrorCode MatMultTranspose_Brussel(Mat A,Vec x,Vec y)
@@ -257,7 +257,7 @@ PetscErrorCode MatMultTranspose_Brussel(Mat A,Vec x,Vec y)
   PetscCall(VecResetArray(ctx->x2));
   PetscCall(VecResetArray(ctx->y1));
   PetscCall(VecResetArray(ctx->y2));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 PetscErrorCode MatGetDiagonal_Brussel(Mat A,Vec diag)
@@ -282,7 +282,7 @@ PetscErrorCode MatGetDiagonal_Brussel(Mat A,Vec diag)
   PetscCall(VecDestroy(&d1));
   PetscCall(VecDestroy(&d2));
   PetscCall(VecRestoreArray(diag,&pd));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*TEST
@@ -327,7 +327,7 @@ PetscErrorCode MatGetDiagonal_Brussel(Mat A,Vec diag)
 
    test:
       suffix: 7
-      args: -n 40 -eps_nev 1 -eps_type arnoldi -eps_smallest_real -eps_refined -eps_ncv 40 -eps_max_it 300 -terse
+      args: -n 40 -eps_nev 1 -eps_type arnoldi -eps_smallest_real -eps_refined -eps_ncv 42 -eps_max_it 300 -terse
       requires: double
 
    test:
@@ -335,6 +335,7 @@ PetscErrorCode MatGetDiagonal_Brussel(Mat A,Vec diag)
       args: -eps_nev 2 -eps_target -30 -eps_type jd -st_matmode shell -eps_jd_fix 0.0001 -eps_jd_const_correction_tol 0 -terse
       requires: !single
       filter: sed -e "s/[+-]0\.0*i//g"
+      timeoutfactor: 2
 
    test:
       suffix: 9
