@@ -18,7 +18,7 @@ PetscClassId      NEP_CLASSID = 0;
 PetscLogEvent     NEP_SetUp = 0,NEP_Solve = 0,NEP_Refine = 0,NEP_FunctionEval = 0,NEP_JacobianEval = 0,NEP_Resolvent = 0,NEP_CISS_SVD = 0;
 
 /* List of registered NEP routines */
-PetscFunctionList NEPList = 0;
+PetscFunctionList NEPList = NULL;
 PetscBool         NEPRegisterAllCalled = PETSC_FALSE;
 
 /* List of registered NEP monitors */
@@ -47,26 +47,25 @@ PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
   NEP            nep;
 
   PetscFunctionBegin;
-  PetscValidPointer(outnep,2);
-  *outnep = 0;
+  PetscAssertPointer(outnep,2);
   PetscCall(NEPInitializePackage());
   PetscCall(SlepcHeaderCreate(nep,NEP_CLASSID,"NEP","Nonlinear Eigenvalue Problem","NEP",comm,NEPDestroy,NEPView));
 
-  nep->max_it          = PETSC_DEFAULT;
+  nep->max_it          = PETSC_DETERMINE;
   nep->nev             = 1;
-  nep->ncv             = PETSC_DEFAULT;
-  nep->mpd             = PETSC_DEFAULT;
+  nep->ncv             = PETSC_DETERMINE;
+  nep->mpd             = PETSC_DETERMINE;
   nep->nini            = 0;
   nep->target          = 0.0;
-  nep->tol             = PETSC_DEFAULT;
+  nep->tol             = PETSC_DETERMINE;
   nep->conv            = NEP_CONV_REL;
   nep->stop            = NEP_STOP_BASIC;
   nep->which           = (NEPWhich)0;
   nep->problem_type    = (NEPProblemType)0;
   nep->refine          = NEP_REFINE_NONE;
   nep->npart           = 1;
-  nep->rtol            = PETSC_DEFAULT;
-  nep->rits            = PETSC_DEFAULT;
+  nep->rtol            = PETSC_DETERMINE;
+  nep->rits            = PETSC_DETERMINE;
   nep->scheme          = (NEPRefineScheme)0;
   nep->trackall        = PETSC_FALSE;
   nep->twosided        = PETSC_FALSE;
@@ -118,15 +117,15 @@ PetscErrorCode NEPCreate(MPI_Comm comm,NEP *outnep)
   nep->resolvent       = NULL;
   nep->reason          = NEP_CONVERGED_ITERATING;
 
-  PetscCall(PetscNewLog(nep,&nep->sc));
+  PetscCall(PetscNew(&nep->sc));
   *outnep = nep;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    NEPSetType - Selects the particular solver to be used in the NEP object.
 
-   Logically Collective on nep
+   Logically Collective
 
    Input Parameters:
 +  nep      - the nonlinear eigensolver context
@@ -158,24 +157,24 @@ PetscErrorCode NEPSetType(NEP nep,NEPType type)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidCharPointer(type,2);
+  PetscAssertPointer(type,2);
 
   PetscCall(PetscObjectTypeCompare((PetscObject)nep,type,&match));
-  if (match) PetscFunctionReturn(0);
+  if (match) PetscFunctionReturn(PETSC_SUCCESS);
 
   PetscCall(PetscFunctionListFind(NEPList,type,&r));
   PetscCheck(r,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown NEP type given: %s",type);
 
-  if (nep->ops->destroy) PetscCall((*nep->ops->destroy)(nep));
+  PetscTryTypeMethod(nep,destroy);
   PetscCall(PetscMemzero(nep->ops,sizeof(struct _NEPOps)));
 
   nep->state = NEP_STATE_INITIAL;
   PetscCall(PetscObjectChangeTypeName((PetscObject)nep,type));
   PetscCall((*r)(nep));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    NEPGetType - Gets the NEP type as a string from the NEP object.
 
    Not Collective
@@ -194,9 +193,9 @@ PetscErrorCode NEPGetType(NEP nep,NEPType *type)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidPointer(type,2);
+  PetscAssertPointer(type,2);
   *type = ((PetscObject)nep)->type_name;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
@@ -211,7 +210,7 @@ PetscErrorCode NEPGetType(NEP nep,NEPType *type)
    Notes:
    NEPRegister() may be called multiple times to add several user-defined solvers.
 
-   Sample usage:
+   Example Usage:
 .vb
     NEPRegister("my_solver",MySolverCreate);
 .ve
@@ -230,7 +229,7 @@ PetscErrorCode NEPRegister(const char *name,PetscErrorCode (*function)(NEP))
   PetscFunctionBegin;
   PetscCall(NEPInitializePackage());
   PetscCall(PetscFunctionListAdd(&NEPList,name,function));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
@@ -249,7 +248,7 @@ PetscErrorCode NEPRegister(const char *name,PetscErrorCode (*function)(NEP))
    Notes:
    NEPMonitorRegister() may be called multiple times to add several user-defined monitors.
 
-   Sample usage:
+   Example Usage:
 .vb
    NEPMonitorRegister("my_monitor",PETSCVIEWERASCII,PETSC_VIEWER_ASCII_INFO_DETAIL,MyMonitor,NULL,NULL);
 .ve
@@ -273,7 +272,7 @@ PetscErrorCode NEPMonitorRegister(const char name[],PetscViewerType vtype,PetscV
   PetscCall(PetscFunctionListAdd(&NEPMonitorList,key,monitor));
   if (create)  PetscCall(PetscFunctionListAdd(&NEPMonitorCreateList,key,create));
   if (destroy) PetscCall(PetscFunctionListAdd(&NEPMonitorDestroyList,key,destroy));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*
@@ -296,13 +295,13 @@ PetscErrorCode NEPReset_Problem(NEP nep)
     if (nep->P) PetscCall(MatDestroyMatrices(nep->nt,&nep->P));
     nep->nt = 0;
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 /*@
    NEPReset - Resets the NEP context to the initial state (prior to setup)
    and destroys any allocated Vecs and Mats.
 
-   Collective on nep
+   Collective
 
    Input Parameter:
 .  nep - eigensolver context obtained from NEPCreate()
@@ -315,8 +314,8 @@ PetscErrorCode NEPReset(NEP nep)
 {
   PetscFunctionBegin;
   if (nep) PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  if (!nep) PetscFunctionReturn(0);
-  if (nep->ops->reset) PetscCall((nep->ops->reset)(nep));
+  if (!nep) PetscFunctionReturn(PETSC_SUCCESS);
+  PetscTryTypeMethod(nep,reset);
   if (nep->refineksp) PetscCall(KSPReset(nep->refineksp));
   PetscCall(NEPReset_Problem(nep));
   PetscCall(BVDestroy(&nep->V));
@@ -325,13 +324,13 @@ PetscErrorCode NEPReset(NEP nep)
   PetscCall(MatDestroy(&nep->resolvent));
   nep->nwork = 0;
   nep->state = NEP_STATE_INITIAL;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    NEPDestroy - Destroys the NEP context.
 
-   Collective on nep
+   Collective
 
    Input Parameter:
 .  nep - eigensolver context obtained from NEPCreate()
@@ -343,11 +342,11 @@ PetscErrorCode NEPReset(NEP nep)
 PetscErrorCode NEPDestroy(NEP *nep)
 {
   PetscFunctionBegin;
-  if (!*nep) PetscFunctionReturn(0);
+  if (!*nep) PetscFunctionReturn(PETSC_SUCCESS);
   PetscValidHeaderSpecific(*nep,NEP_CLASSID,1);
-  if (--((PetscObject)(*nep))->refct > 0) { *nep = 0; PetscFunctionReturn(0); }
+  if (--((PetscObject)*nep)->refct > 0) { *nep = NULL; PetscFunctionReturn(PETSC_SUCCESS); }
   PetscCall(NEPReset(*nep));
-  if ((*nep)->ops->destroy) PetscCall((*(*nep)->ops->destroy)(*nep));
+  PetscTryTypeMethod(*nep,destroy);
   if ((*nep)->eigr) PetscCall(PetscFree4((*nep)->eigr,(*nep)->eigi,(*nep)->errest,(*nep)->perm));
   PetscCall(RGDestroy(&(*nep)->rg));
   PetscCall(DSDestroy(&(*nep)->ds));
@@ -356,16 +355,17 @@ PetscErrorCode NEPDestroy(NEP *nep)
   PetscCall(PetscFree((*nep)->sc));
   /* just in case the initial vectors have not been used */
   PetscCall(SlepcBasisDestroy_Private(&(*nep)->nini,&(*nep)->IS));
-  if ((*nep)->convergeddestroy) PetscCall((*(*nep)->convergeddestroy)((*nep)->convergedctx));
+  if ((*nep)->convergeddestroy) PetscCall((*(*nep)->convergeddestroy)(&(*nep)->convergedctx));
+  if ((*nep)->stoppingdestroy) PetscCall((*(*nep)->stoppingdestroy)(&(*nep)->stoppingctx));
   PetscCall(NEPMonitorCancel(*nep));
   PetscCall(PetscHeaderDestroy(nep));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    NEPSetBV - Associates a basis vectors object to the nonlinear eigensolver.
 
-   Collective on nep
+   Collective
 
    Input Parameters:
 +  nep - eigensolver context obtained from NEPCreate()
@@ -388,8 +388,7 @@ PetscErrorCode NEPSetBV(NEP nep,BV bv)
   PetscCall(PetscObjectReference((PetscObject)bv));
   PetscCall(BVDestroy(&nep->V));
   nep->V = bv;
-  PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->V));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -412,21 +411,20 @@ PetscErrorCode NEPGetBV(NEP nep,BV *bv)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidPointer(bv,2);
+  PetscAssertPointer(bv,2);
   if (!nep->V) {
     PetscCall(BVCreate(PetscObjectComm((PetscObject)nep),&nep->V));
     PetscCall(PetscObjectIncrementTabLevel((PetscObject)nep->V,(PetscObject)nep,0));
-    PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->V));
     PetscCall(PetscObjectSetOptions((PetscObject)nep->V,((PetscObject)nep)->options));
   }
   *bv = nep->V;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    NEPSetRG - Associates a region object to the nonlinear eigensolver.
 
-   Collective on nep
+   Collective
 
    Input Parameters:
 +  nep - eigensolver context obtained from NEPCreate()
@@ -451,8 +449,7 @@ PetscErrorCode NEPSetRG(NEP nep,RG rg)
   PetscCall(PetscObjectReference((PetscObject)rg));
   PetscCall(RGDestroy(&nep->rg));
   nep->rg = rg;
-  PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->rg));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -475,21 +472,20 @@ PetscErrorCode NEPGetRG(NEP nep,RG *rg)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidPointer(rg,2);
+  PetscAssertPointer(rg,2);
   if (!nep->rg) {
     PetscCall(RGCreate(PetscObjectComm((PetscObject)nep),&nep->rg));
     PetscCall(PetscObjectIncrementTabLevel((PetscObject)nep->rg,(PetscObject)nep,0));
-    PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->rg));
     PetscCall(PetscObjectSetOptions((PetscObject)nep->rg,((PetscObject)nep)->options));
   }
   *rg = nep->rg;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    NEPSetDS - Associates a direct solver object to the nonlinear eigensolver.
 
-   Collective on nep
+   Collective
 
    Input Parameters:
 +  nep - eigensolver context obtained from NEPCreate()
@@ -512,8 +508,7 @@ PetscErrorCode NEPSetDS(NEP nep,DS ds)
   PetscCall(PetscObjectReference((PetscObject)ds));
   PetscCall(DSDestroy(&nep->ds));
   nep->ds = ds;
-  PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->ds));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -536,22 +531,21 @@ PetscErrorCode NEPGetDS(NEP nep,DS *ds)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidPointer(ds,2);
+  PetscAssertPointer(ds,2);
   if (!nep->ds) {
     PetscCall(DSCreate(PetscObjectComm((PetscObject)nep),&nep->ds));
     PetscCall(PetscObjectIncrementTabLevel((PetscObject)nep->ds,(PetscObject)nep,0));
-    PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->ds));
     PetscCall(PetscObjectSetOptions((PetscObject)nep->ds,((PetscObject)nep)->options));
   }
   *ds = nep->ds;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    NEPRefineGetKSP - Obtain the ksp object used by the eigensolver
    object in the refinement phase.
 
-   Not Collective
+   Collective
 
    Input Parameters:
 .  nep - eigensolver context obtained from NEPCreate()
@@ -569,32 +563,30 @@ PetscErrorCode NEPRefineGetKSP(NEP nep,KSP *ksp)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidPointer(ksp,2);
+  PetscAssertPointer(ksp,2);
   if (!nep->refineksp) {
     if (nep->npart>1) {
       /* Split in subcomunicators */
       PetscCall(PetscSubcommCreate(PetscObjectComm((PetscObject)nep),&nep->refinesubc));
       PetscCall(PetscSubcommSetNumber(nep->refinesubc,nep->npart));
       PetscCall(PetscSubcommSetType(nep->refinesubc,PETSC_SUBCOMM_CONTIGUOUS));
-      PetscCall(PetscLogObjectMemory((PetscObject)nep,sizeof(PetscSubcomm)));
       PetscCall(PetscSubcommGetChild(nep->refinesubc,&comm));
     } else PetscCall(PetscObjectGetComm((PetscObject)nep,&comm));
     PetscCall(KSPCreate(comm,&nep->refineksp));
     PetscCall(PetscObjectIncrementTabLevel((PetscObject)nep->refineksp,(PetscObject)nep,0));
-    PetscCall(PetscLogObjectParent((PetscObject)nep,(PetscObject)nep->refineksp));
     PetscCall(PetscObjectSetOptions((PetscObject)nep->refineksp,((PetscObject)nep)->options));
     PetscCall(KSPSetOptionsPrefix(*ksp,((PetscObject)nep)->prefix));
     PetscCall(KSPAppendOptionsPrefix(*ksp,"nep_refine_"));
-    PetscCall(KSPSetTolerances(nep->refineksp,SlepcDefaultTol(nep->rtol),PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT));
+    PetscCall(KSPSetTolerances(nep->refineksp,SlepcDefaultTol(nep->rtol),PETSC_CURRENT,PETSC_CURRENT,PETSC_CURRENT));
   }
   *ksp = nep->refineksp;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    NEPSetTarget - Sets the value of the target.
 
-   Logically Collective on nep
+   Logically Collective
 
    Input Parameters:
 +  nep    - eigensolver context
@@ -621,7 +613,7 @@ PetscErrorCode NEPSetTarget(NEP nep,PetscScalar target)
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidLogicalCollectiveScalar(nep,target,2);
   nep->target = target;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -646,41 +638,32 @@ PetscErrorCode NEPGetTarget(NEP nep,PetscScalar* target)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
-  PetscValidScalarPointer(target,2);
+  PetscAssertPointer(target,2);
   *target = nep->target;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
    NEPSetFunction - Sets the function to compute the nonlinear Function T(lambda)
    as well as the location to store the matrix.
 
-   Logically Collective on nep
+   Collective
 
    Input Parameters:
 +  nep - the NEP context
 .  A   - Function matrix
 .  B   - preconditioner matrix (usually same as A)
 .  fun - Function evaluation routine (if NULL then NEP retains any
-         previously set value)
+         previously set value), see NEPFunctionFn for the calling sequence
 -  ctx - [optional] user-defined context for private data for the Function
          evaluation routine (may be NULL) (if NULL then NEP retains any
          previously set value)
 
-   Calling Sequence of fun:
-$   fun(NEP nep,PetscScalar lambda,Mat F,Mat P,void *ctx)
-
-+  nep    - the NEP context
-.  lambda - the scalar argument where T(.) must be evaluated
-.  T      - matrix that will contain T(lambda)
-.  P      - (optional) different matrix to build the preconditioner
--  ctx    - (optional) user-defined context, as set by NEPSetFunction()
-
    Level: beginner
 
 .seealso: NEPGetFunction(), NEPSetJacobian()
 @*/
-PetscErrorCode NEPSetFunction(NEP nep,Mat A,Mat B,PetscErrorCode (*fun)(NEP,PetscScalar,Mat,Mat,void*),void *ctx)
+PetscErrorCode NEPSetFunction(NEP nep,Mat A,Mat B,NEPFunctionFn *fun,void *ctx)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
@@ -706,14 +689,14 @@ PetscErrorCode NEPSetFunction(NEP nep,Mat A,Mat B,PetscErrorCode (*fun)(NEP,Pets
   }
   nep->fui   = NEP_USER_INTERFACE_CALLBACK;
   nep->state = NEP_STATE_INITIAL;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
    NEPGetFunction - Returns the Function matrix and optionally the user
    provided context for evaluating the Function.
 
-   Not Collective, but Mat object will be parallel if NEP object is
+   Not Collective
 
    Input Parameter:
 .  nep - the nonlinear eigensolver context
@@ -728,7 +711,7 @@ PetscErrorCode NEPSetFunction(NEP nep,Mat A,Mat B,PetscErrorCode (*fun)(NEP,Pets
 
 .seealso: NEPSetFunction()
 @*/
-PetscErrorCode NEPGetFunction(NEP nep,Mat *A,Mat *B,PetscErrorCode (**fun)(NEP,PetscScalar,Mat,Mat,void*),void **ctx)
+PetscErrorCode NEPGetFunction(NEP nep,Mat *A,Mat *B,NEPFunctionFn **fun,void **ctx)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
@@ -737,37 +720,29 @@ PetscErrorCode NEPGetFunction(NEP nep,Mat *A,Mat *B,PetscErrorCode (**fun)(NEP,P
   if (B)   *B   = nep->function_pre;
   if (fun) *fun = nep->computefunction;
   if (ctx) *ctx = nep->functionctx;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
    NEPSetJacobian - Sets the function to compute the Jacobian T'(lambda) as well
    as the location to store the matrix.
 
-   Logically Collective on nep
+   Collective
 
    Input Parameters:
 +  nep - the NEP context
 .  A   - Jacobian matrix
 .  jac - Jacobian evaluation routine (if NULL then NEP retains any
-         previously set value)
+         previously set value), see NEPJacobianFn for the calling sequence
 -  ctx - [optional] user-defined context for private data for the Jacobian
          evaluation routine (may be NULL) (if NULL then NEP retains any
          previously set value)
 
-   Calling Sequence of jac:
-$   jac(NEP nep,PetscScalar lambda,Mat J,void *ctx)
-
-+  nep    - the NEP context
-.  lambda - the scalar argument where T'(.) must be evaluated
-.  J      - matrix that will contain T'(lambda)
--  ctx    - (optional) user-defined context, as set by NEPSetJacobian()
-
    Level: beginner
 
 .seealso: NEPSetFunction(), NEPGetJacobian()
 @*/
-PetscErrorCode NEPSetJacobian(NEP nep,Mat A,PetscErrorCode (*jac)(NEP,PetscScalar,Mat,void*),void *ctx)
+PetscErrorCode NEPSetJacobian(NEP nep,Mat A,NEPJacobianFn *jac,void *ctx)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
@@ -786,14 +761,14 @@ PetscErrorCode NEPSetJacobian(NEP nep,Mat A,PetscErrorCode (*jac)(NEP,PetscScala
   }
   nep->fui   = NEP_USER_INTERFACE_CALLBACK;
   nep->state = NEP_STATE_INITIAL;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
    NEPGetJacobian - Returns the Jacobian matrix and optionally the user
    provided routine and context for evaluating the Jacobian.
 
-   Not Collective, but Mat object will be parallel if NEP object is
+   Not Collective
 
    Input Parameter:
 .  nep - the nonlinear eigensolver context
@@ -807,7 +782,7 @@ PetscErrorCode NEPSetJacobian(NEP nep,Mat A,PetscErrorCode (*jac)(NEP,PetscScala
 
 .seealso: NEPSetJacobian()
 @*/
-PetscErrorCode NEPGetJacobian(NEP nep,Mat *A,PetscErrorCode (**jac)(NEP,PetscScalar,Mat,void*),void **ctx)
+PetscErrorCode NEPGetJacobian(NEP nep,Mat *A,NEPJacobianFn **jac,void **ctx)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
@@ -815,14 +790,14 @@ PetscErrorCode NEPGetJacobian(NEP nep,Mat *A,PetscErrorCode (**jac)(NEP,PetscSca
   if (A)   *A   = nep->jacobian;
   if (jac) *jac = nep->computejacobian;
   if (ctx) *ctx = nep->jacobianctx;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    NEPSetSplitOperator - Sets the operator of the nonlinear eigenvalue problem
    in split form.
 
-   Collective on nep
+   Collective
 
    Input Parameters:
 +  nep - the nonlinear eigensolver context
@@ -850,7 +825,7 @@ PetscErrorCode NEPGetJacobian(NEP nep,Mat *A,PetscErrorCode (**jac)(NEP,PetscSca
    Level: beginner
 
 .seealso: NEPGetSplitOperatorTerm(), NEPGetSplitOperatorInfo(), NEPSetSplitPreconditioner()
- @*/
+@*/
 PetscErrorCode NEPSetSplitOperator(NEP nep,PetscInt nt,Mat A[],FN f[],MatStructure str)
 {
   PetscInt       i,n=0,m,m0=0,mloc,nloc,mloc0=0;
@@ -859,8 +834,8 @@ PetscErrorCode NEPSetSplitOperator(NEP nep,PetscInt nt,Mat A[],FN f[],MatStructu
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidLogicalCollectiveInt(nep,nt,2);
   PetscCheck(nt>0,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Must have one or more terms, you have %" PetscInt_FMT,nt);
-  PetscValidPointer(A,3);
-  PetscValidPointer(f,4);
+  PetscAssertPointer(A,3);
+  PetscAssertPointer(f,4);
   PetscValidLogicalCollectiveEnum(nep,str,5);
 
   for (i=0;i<nt;i++) {
@@ -884,25 +859,22 @@ PetscErrorCode NEPSetSplitOperator(NEP nep,PetscInt nt,Mat A[],FN f[],MatStructu
 
   /* allocate space and copy matrices and functions */
   PetscCall(PetscMalloc1(nt,&nep->A));
-  PetscCall(PetscLogObjectMemory((PetscObject)nep,nt*sizeof(Mat)));
   for (i=0;i<nt;i++) nep->A[i] = A[i];
   PetscCall(PetscMalloc1(nt,&nep->f));
-  PetscCall(PetscLogObjectMemory((PetscObject)nep,nt*sizeof(FN)));
   for (i=0;i<nt;i++) nep->f[i] = f[i];
   PetscCall(PetscCalloc1(nt,&nep->nrma));
-  PetscCall(PetscLogObjectMemory((PetscObject)nep,nt*sizeof(PetscReal)));
   nep->nt    = nt;
   nep->mstr  = str;
   nep->fui   = NEP_USER_INTERFACE_SPLIT;
   nep->state = NEP_STATE_INITIAL;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    NEPGetSplitOperatorTerm - Gets the matrices and functions associated with
    the nonlinear operator in split form.
 
-   Not collective, though parallel Mats and FNs are returned if the NEP is parallel
+   Not Collective
 
    Input Parameters:
 +  nep - the nonlinear eigensolver context
@@ -925,14 +897,14 @@ PetscErrorCode NEPGetSplitOperatorTerm(NEP nep,PetscInt k,Mat *A,FN *f)
   PetscCheck(k>=0 && k<nep->nt,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %" PetscInt_FMT,nep->nt-1);
   if (A) *A = nep->A[k];
   if (f) *f = nep->f[k];
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    NEPGetSplitOperatorInfo - Returns the number of terms of the split form of
    the nonlinear operator, as well as the structure flag for matrices.
 
-   Not collective
+   Not Collective
 
    Input Parameter:
 .  nep - the nonlinear eigensolver context
@@ -952,7 +924,7 @@ PetscErrorCode NEPGetSplitOperatorInfo(NEP nep,PetscInt *n,MatStructure *str)
   NEPCheckSplit(nep,1);
   if (n)   *n = nep->nt;
   if (str) *str = nep->mstr;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -960,7 +932,7 @@ PetscErrorCode NEPGetSplitOperatorInfo(NEP nep,PetscInt *n,MatStructure *str)
    to build the preconditioner to be used when solving the nonlinear
    eigenvalue problem in split form.
 
-   Collective on nep
+   Collective
 
    Input Parameters:
 +  nep  - the nonlinear eigensolver context
@@ -988,7 +960,7 @@ PetscErrorCode NEPGetSplitOperatorInfo(NEP nep,PetscInt *n,MatStructure *str)
    Level: advanced
 
 .seealso: NEPGetSplitPreconditionerTerm(), NEPGetSplitPreconditionerInfo(), NEPSetSplitOperator()
- @*/
+@*/
 PetscErrorCode NEPSetSplitPreconditioner(NEP nep,PetscInt ntp,Mat P[],MatStructure strp)
 {
   PetscInt       i,n=0,m,m0=0,mloc,nloc,mloc0=0;
@@ -999,7 +971,7 @@ PetscErrorCode NEPSetSplitPreconditioner(NEP nep,PetscInt ntp,Mat P[],MatStructu
   PetscCheck(ntp>=0,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Negative value of ntp = %" PetscInt_FMT,ntp);
   PetscCheck(nep->fui==NEP_USER_INTERFACE_SPLIT,PetscObjectComm((PetscObject)nep),PETSC_ERR_ORDER,"Must call NEPSetSplitOperator first");
   PetscCheck(ntp==0 || nep->nt==ntp,PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"The number of terms must be the same as in NEPSetSplitOperator()");
-  if (ntp) PetscValidPointer(P,3);
+  if (ntp) PetscAssertPointer(P,3);
   PetscValidLogicalCollectiveEnum(nep,strp,4);
 
   for (i=0;i<ntp;i++) {
@@ -1021,19 +993,18 @@ PetscErrorCode NEPSetSplitPreconditioner(NEP nep,PetscInt ntp,Mat P[],MatStructu
   /* allocate space and copy matrices */
   if (ntp) {
     PetscCall(PetscMalloc1(ntp,&nep->P));
-    PetscCall(PetscLogObjectMemory((PetscObject)nep,ntp*sizeof(Mat)));
     for (i=0;i<ntp;i++) nep->P[i] = P[i];
   }
   nep->mstrp = strp;
   nep->state = NEP_STATE_INITIAL;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    NEPGetSplitPreconditionerTerm - Gets the matrices associated with
    the split preconditioner.
 
-   Not collective, though parallel Mats are returned if the NEP is parallel
+   Not Collective
 
    Input Parameters:
 +  nep - the nonlinear eigensolver context
@@ -1051,19 +1022,19 @@ PetscErrorCode NEPGetSplitPreconditionerTerm(NEP nep,PetscInt k,Mat *P)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidLogicalCollectiveInt(nep,k,2);
-  PetscValidPointer(P,3);
+  PetscAssertPointer(P,3);
   NEPCheckSplit(nep,1);
   PetscCheck(k>=0 && k<nep->nt,PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"k must be between 0 and %" PetscInt_FMT,nep->nt-1);
   PetscCheck(nep->P,PetscObjectComm((PetscObject)nep),PETSC_ERR_ORDER,"You have not called NEPSetSplitPreconditioner()");
   *P = nep->P[k];
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    NEPGetSplitPreconditionerInfo - Returns the number of terms of the split
    preconditioner, as well as the structure flag for matrices.
 
-   Not collective
+   Not Collective
 
    Input Parameter:
 .  nep - the nonlinear eigensolver context
@@ -1083,5 +1054,5 @@ PetscErrorCode NEPGetSplitPreconditionerInfo(NEP nep,PetscInt *n,MatStructure *s
   NEPCheckSplit(nep,1);
   if (n)    *n    = nep->P? nep->nt: 0;
   if (strp) *strp = nep->mstrp;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
