@@ -18,7 +18,7 @@ PetscClassId      PEP_CLASSID = 0;
 PetscLogEvent     PEP_SetUp = 0,PEP_Solve = 0,PEP_Refine = 0,PEP_CISS_SVD = 0;
 
 /* List of registered PEP routines */
-PetscFunctionList PEPList = 0;
+PetscFunctionList PEPList = NULL;
 PetscBool         PEPRegisterAllCalled = PETSC_FALSE;
 
 /* List of registered PEP monitors */
@@ -50,18 +50,17 @@ PetscErrorCode PEPCreate(MPI_Comm comm,PEP *outpep)
   PEP            pep;
 
   PetscFunctionBegin;
-  PetscValidPointer(outpep,2);
-  *outpep = 0;
+  PetscAssertPointer(outpep,2);
   PetscCall(PEPInitializePackage());
   PetscCall(SlepcHeaderCreate(pep,PEP_CLASSID,"PEP","Polynomial Eigenvalue Problem","PEP",comm,PEPDestroy,PEPView));
 
-  pep->max_it          = PETSC_DEFAULT;
+  pep->max_it          = PETSC_DETERMINE;
   pep->nev             = 1;
-  pep->ncv             = PETSC_DEFAULT;
-  pep->mpd             = PETSC_DEFAULT;
+  pep->ncv             = PETSC_DETERMINE;
+  pep->mpd             = PETSC_DETERMINE;
   pep->nini            = 0;
   pep->target          = 0.0;
-  pep->tol             = PETSC_DEFAULT;
+  pep->tol             = PETSC_DETERMINE;
   pep->conv            = PEP_CONV_REL;
   pep->stop            = PEP_STOP_BASIC;
   pep->which           = (PEPWhich)0;
@@ -74,8 +73,8 @@ PetscErrorCode PEPCreate(MPI_Comm comm,PEP *outpep)
   pep->slambda         = 1.0;
   pep->refine          = PEP_REFINE_NONE;
   pep->npart           = 1;
-  pep->rtol            = PETSC_DEFAULT;
-  pep->rits            = PETSC_DEFAULT;
+  pep->rtol            = PETSC_DETERMINE;
+  pep->rits            = PETSC_DETERMINE;
   pep->scheme          = (PEPRefineScheme)0;
   pep->extract         = (PEPExtract)0;
   pep->trackall        = PETSC_FALSE;
@@ -121,15 +120,15 @@ PetscErrorCode PEPCreate(MPI_Comm comm,PEP *outpep)
   pep->lineariz        = PETSC_FALSE;
   pep->reason          = PEP_CONVERGED_ITERATING;
 
-  PetscCall(PetscNewLog(pep,&pep->sc));
+  PetscCall(PetscNew(&pep->sc));
   *outpep = pep;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    PEPSetType - Selects the particular solver to be used in the PEP object.
 
-   Logically Collective on pep
+   Logically Collective
 
    Input Parameters:
 +  pep      - the polynomial eigensolver context
@@ -162,24 +161,24 @@ PetscErrorCode PEPSetType(PEP pep,PEPType type)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidCharPointer(type,2);
+  PetscAssertPointer(type,2);
 
   PetscCall(PetscObjectTypeCompare((PetscObject)pep,type,&match));
-  if (match) PetscFunctionReturn(0);
+  if (match) PetscFunctionReturn(PETSC_SUCCESS);
 
   PetscCall(PetscFunctionListFind(PEPList,type,&r));
   PetscCheck(r,PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown PEP type given: %s",type);
 
-  if (pep->ops->destroy) PetscCall((*pep->ops->destroy)(pep));
+  PetscTryTypeMethod(pep,destroy);
   PetscCall(PetscMemzero(pep->ops,sizeof(struct _PEPOps)));
 
   pep->state = PEP_STATE_INITIAL;
   PetscCall(PetscObjectChangeTypeName((PetscObject)pep,type));
   PetscCall((*r)(pep));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    PEPGetType - Gets the PEP type as a string from the PEP object.
 
    Not Collective
@@ -198,9 +197,9 @@ PetscErrorCode PEPGetType(PEP pep,PEPType *type)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidPointer(type,2);
+  PetscAssertPointer(type,2);
   *type = ((PetscObject)pep)->type_name;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
@@ -215,7 +214,7 @@ PetscErrorCode PEPGetType(PEP pep,PEPType *type)
    Notes:
    PEPRegister() may be called multiple times to add several user-defined solvers.
 
-   Sample usage:
+   Example Usage:
 .vb
     PEPRegister("my_solver",MySolverCreate);
 .ve
@@ -234,7 +233,7 @@ PetscErrorCode PEPRegister(const char *name,PetscErrorCode (*function)(PEP))
   PetscFunctionBegin;
   PetscCall(PEPInitializePackage());
   PetscCall(PetscFunctionListAdd(&PEPList,name,function));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
@@ -253,7 +252,7 @@ PetscErrorCode PEPRegister(const char *name,PetscErrorCode (*function)(PEP))
    Notes:
    PEPMonitorRegister() may be called multiple times to add several user-defined monitors.
 
-   Sample usage:
+   Example Usage:
 .vb
    PEPMonitorRegister("my_monitor",PETSCVIEWERASCII,PETSC_VIEWER_ASCII_INFO_DETAIL,MyMonitor,NULL,NULL);
 .ve
@@ -277,14 +276,14 @@ PetscErrorCode PEPMonitorRegister(const char name[],PetscViewerType vtype,PetscV
   PetscCall(PetscFunctionListAdd(&PEPMonitorList,key,monitor));
   if (create)  PetscCall(PetscFunctionListAdd(&PEPMonitorCreateList,key,create));
   if (destroy) PetscCall(PetscFunctionListAdd(&PEPMonitorDestroyList,key,destroy));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    PEPReset - Resets the PEP context to the initial state (prior to setup)
    and destroys any allocated Vecs and Mats.
 
-   Collective on pep
+   Collective
 
    Input Parameter:
 .  pep - eigensolver context obtained from PEPCreate()
@@ -297,8 +296,8 @@ PetscErrorCode PEPReset(PEP pep)
 {
   PetscFunctionBegin;
   if (pep) PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  if (!pep) PetscFunctionReturn(0);
-  if (pep->ops->reset) PetscCall((pep->ops->reset)(pep));
+  if (!pep) PetscFunctionReturn(PETSC_SUCCESS);
+  PetscTryTypeMethod(pep,reset);
   if (pep->st) PetscCall(STReset(pep->st));
   if (pep->refineksp) PetscCall(KSPReset(pep->refineksp));
   if (pep->nmat) {
@@ -313,13 +312,13 @@ PetscErrorCode PEPReset(PEP pep)
   PetscCall(VecDestroyVecs(pep->nwork,&pep->work));
   pep->nwork = 0;
   pep->state = PEP_STATE_INITIAL;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
    PEPDestroy - Destroys the PEP context.
 
-   Collective on pep
+   Collective
 
    Input Parameter:
 .  pep - eigensolver context obtained from PEPCreate()
@@ -331,11 +330,11 @@ PetscErrorCode PEPReset(PEP pep)
 PetscErrorCode PEPDestroy(PEP *pep)
 {
   PetscFunctionBegin;
-  if (!*pep) PetscFunctionReturn(0);
+  if (!*pep) PetscFunctionReturn(PETSC_SUCCESS);
   PetscValidHeaderSpecific(*pep,PEP_CLASSID,1);
-  if (--((PetscObject)(*pep))->refct > 0) { *pep = 0; PetscFunctionReturn(0); }
+  if (--((PetscObject)*pep)->refct > 0) { *pep = NULL; PetscFunctionReturn(PETSC_SUCCESS); }
   PetscCall(PEPReset(*pep));
-  if ((*pep)->ops->destroy) PetscCall((*(*pep)->ops->destroy)(*pep));
+  PetscTryTypeMethod(*pep,destroy);
   if ((*pep)->eigr) PetscCall(PetscFree4((*pep)->eigr,(*pep)->eigi,(*pep)->errest,(*pep)->perm));
   PetscCall(STDestroy(&(*pep)->st));
   PetscCall(RGDestroy(&(*pep)->rg));
@@ -345,16 +344,17 @@ PetscErrorCode PEPDestroy(PEP *pep)
   PetscCall(PetscFree((*pep)->sc));
   /* just in case the initial vectors have not been used */
   PetscCall(SlepcBasisDestroy_Private(&(*pep)->nini,&(*pep)->IS));
-  if ((*pep)->convergeddestroy) PetscCall((*(*pep)->convergeddestroy)((*pep)->convergedctx));
+  if ((*pep)->convergeddestroy) PetscCall((*(*pep)->convergeddestroy)(&(*pep)->convergedctx));
+  if ((*pep)->stoppingdestroy) PetscCall((*(*pep)->stoppingdestroy)(&(*pep)->stoppingctx));
   PetscCall(PEPMonitorCancel(*pep));
   PetscCall(PetscHeaderDestroy(pep));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    PEPSetBV - Associates a basis vectors object to the polynomial eigensolver.
 
-   Collective on pep
+   Collective
 
    Input Parameters:
 +  pep - eigensolver context obtained from PEPCreate()
@@ -377,8 +377,7 @@ PetscErrorCode PEPSetBV(PEP pep,BV bv)
   PetscCall(PetscObjectReference((PetscObject)bv));
   PetscCall(BVDestroy(&pep->V));
   pep->V = bv;
-  PetscCall(PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->V));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -401,21 +400,20 @@ PetscErrorCode PEPGetBV(PEP pep,BV *bv)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidPointer(bv,2);
+  PetscAssertPointer(bv,2);
   if (!pep->V) {
     PetscCall(BVCreate(PetscObjectComm((PetscObject)pep),&pep->V));
     PetscCall(PetscObjectIncrementTabLevel((PetscObject)pep->V,(PetscObject)pep,0));
-    PetscCall(PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->V));
     PetscCall(PetscObjectSetOptions((PetscObject)pep->V,((PetscObject)pep)->options));
   }
   *bv = pep->V;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    PEPSetRG - Associates a region object to the polynomial eigensolver.
 
-   Collective on pep
+   Collective
 
    Input Parameters:
 +  pep - eigensolver context obtained from PEPCreate()
@@ -440,8 +438,7 @@ PetscErrorCode PEPSetRG(PEP pep,RG rg)
   PetscCall(PetscObjectReference((PetscObject)rg));
   PetscCall(RGDestroy(&pep->rg));
   pep->rg = rg;
-  PetscCall(PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->rg));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -464,21 +461,20 @@ PetscErrorCode PEPGetRG(PEP pep,RG *rg)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidPointer(rg,2);
+  PetscAssertPointer(rg,2);
   if (!pep->rg) {
     PetscCall(RGCreate(PetscObjectComm((PetscObject)pep),&pep->rg));
     PetscCall(PetscObjectIncrementTabLevel((PetscObject)pep->rg,(PetscObject)pep,0));
-    PetscCall(PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->rg));
     PetscCall(PetscObjectSetOptions((PetscObject)pep->rg,((PetscObject)pep)->options));
   }
   *rg = pep->rg;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    PEPSetDS - Associates a direct solver object to the polynomial eigensolver.
 
-   Collective on pep
+   Collective
 
    Input Parameters:
 +  pep - eigensolver context obtained from PEPCreate()
@@ -501,8 +497,7 @@ PetscErrorCode PEPSetDS(PEP pep,DS ds)
   PetscCall(PetscObjectReference((PetscObject)ds));
   PetscCall(DSDestroy(&pep->ds));
   pep->ds = ds;
-  PetscCall(PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->ds));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -525,21 +520,20 @@ PetscErrorCode PEPGetDS(PEP pep,DS *ds)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidPointer(ds,2);
+  PetscAssertPointer(ds,2);
   if (!pep->ds) {
     PetscCall(DSCreate(PetscObjectComm((PetscObject)pep),&pep->ds));
     PetscCall(PetscObjectIncrementTabLevel((PetscObject)pep->ds,(PetscObject)pep,0));
-    PetscCall(PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->ds));
     PetscCall(PetscObjectSetOptions((PetscObject)pep->ds,((PetscObject)pep)->options));
   }
   *ds = pep->ds;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    PEPSetST - Associates a spectral transformation object to the eigensolver.
 
-   Collective on pep
+   Collective
 
    Input Parameters:
 +  pep - eigensolver context obtained from PEPCreate()
@@ -562,8 +556,7 @@ PetscErrorCode PEPSetST(PEP pep,ST st)
   PetscCall(PetscObjectReference((PetscObject)st));
   PetscCall(STDestroy(&pep->st));
   pep->st = st;
-  PetscCall(PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->st));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -586,22 +579,21 @@ PetscErrorCode PEPGetST(PEP pep,ST *st)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidPointer(st,2);
+  PetscAssertPointer(st,2);
   if (!pep->st) {
     PetscCall(STCreate(PetscObjectComm((PetscObject)pep),&pep->st));
     PetscCall(PetscObjectIncrementTabLevel((PetscObject)pep->st,(PetscObject)pep,0));
-    PetscCall(PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->st));
     PetscCall(PetscObjectSetOptions((PetscObject)pep->st,((PetscObject)pep)->options));
   }
   *st = pep->st;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    PEPRefineGetKSP - Obtain the ksp object used by the eigensolver
    object in the refinement phase.
 
-   Not Collective
+   Collective
 
    Input Parameters:
 .  pep - eigensolver context obtained from PEPCreate()
@@ -619,32 +611,30 @@ PetscErrorCode PEPRefineGetKSP(PEP pep,KSP *ksp)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidPointer(ksp,2);
+  PetscAssertPointer(ksp,2);
   if (!pep->refineksp) {
     if (pep->npart>1) {
       /* Split in subcomunicators */
       PetscCall(PetscSubcommCreate(PetscObjectComm((PetscObject)pep),&pep->refinesubc));
       PetscCall(PetscSubcommSetNumber(pep->refinesubc,pep->npart));
       PetscCall(PetscSubcommSetType(pep->refinesubc,PETSC_SUBCOMM_CONTIGUOUS));
-      PetscCall(PetscLogObjectMemory((PetscObject)pep,sizeof(PetscSubcomm)));
       PetscCall(PetscSubcommGetChild(pep->refinesubc,&comm));
     } else PetscCall(PetscObjectGetComm((PetscObject)pep,&comm));
     PetscCall(KSPCreate(comm,&pep->refineksp));
     PetscCall(PetscObjectIncrementTabLevel((PetscObject)pep->refineksp,(PetscObject)pep,0));
-    PetscCall(PetscLogObjectParent((PetscObject)pep,(PetscObject)pep->refineksp));
     PetscCall(PetscObjectSetOptions((PetscObject)pep->refineksp,((PetscObject)pep)->options));
     PetscCall(KSPSetOptionsPrefix(*ksp,((PetscObject)pep)->prefix));
     PetscCall(KSPAppendOptionsPrefix(*ksp,"pep_refine_"));
-    PetscCall(KSPSetTolerances(pep->refineksp,SlepcDefaultTol(pep->rtol),PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT));
+    PetscCall(KSPSetTolerances(pep->refineksp,SlepcDefaultTol(pep->rtol),PETSC_CURRENT,PETSC_CURRENT,PETSC_CURRENT));
   }
   *ksp = pep->refineksp;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    PEPSetTarget - Sets the value of the target.
 
-   Logically Collective on pep
+   Logically Collective
 
    Input Parameters:
 +  pep    - eigensolver context
@@ -673,7 +663,7 @@ PetscErrorCode PEPSetTarget(PEP pep,PetscScalar target)
   pep->target = target;
   if (!pep->st) PetscCall(PEPGetST(pep,&pep->st));
   PetscCall(STSetDefaultShift(pep->st,target));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -698,15 +688,15 @@ PetscErrorCode PEPGetTarget(PEP pep,PetscScalar* target)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
-  PetscValidScalarPointer(target,2);
+  PetscAssertPointer(target,2);
   *target = pep->target;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    PEPSetInterval - Defines the computational interval for spectrum slicing.
 
-   Logically Collective on pep
+   Logically Collective
 
    Input Parameters:
 +  pep  - eigensolver context
@@ -743,7 +733,7 @@ PetscErrorCode PEPSetInterval(PEP pep,PetscReal inta,PetscReal intb)
     pep->intb = intb;
     pep->state = PEP_STATE_INITIAL;
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -771,5 +761,5 @@ PetscErrorCode PEPGetInterval(PEP pep,PetscReal* inta,PetscReal* intb)
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
   if (inta) *inta = pep->inta;
   if (intb) *intb = pep->intb;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
