@@ -14,7 +14,6 @@
 */
 
 #include <slepc/private/svdimpl.h>                /*I "slepcsvd.h" I*/
-#include <slepc/private/epsimpl.h>                /*I "slepceps.h" I*/
 
 typedef struct {
   PetscBool explicitmatrix;
@@ -25,7 +24,7 @@ typedef struct {
 
 typedef struct {
   Mat       A,AT;
-  Vec       w,diag;
+  Vec       w,diag,omega;
   PetscBool swapped;
 } SVD_CROSS_SHELL;
 
@@ -36,8 +35,9 @@ static PetscErrorCode MatMult_Cross(Mat B,Vec x,Vec y)
   PetscFunctionBegin;
   PetscCall(MatShellGetContext(B,&ctx));
   PetscCall(MatMult(ctx->A,x,ctx->w));
+  if (ctx->omega && !ctx->swapped) PetscCall(VecPointwiseMult(ctx->w,ctx->w,ctx->omega));
   PetscCall(MatMult(ctx->AT,ctx->w,y));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode MatGetDiagonal_Cross(Mat B,Vec d)
@@ -73,7 +73,7 @@ static PetscErrorCode MatGetDiagonal_Cross(Mat B,Vec d)
       }
     }
     PetscCall(PetscMPIIntCast(N,&len));
-    PetscCall(MPIU_Allreduce(work1,work2,len,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)B)));
+    PetscCallMPI(MPIU_Allreduce(work1,work2,len,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)B)));
     PetscCall(VecGetOwnershipRange(ctx->diag,&start,&end));
     PetscCall(VecGetArrayWrite(ctx->diag,&diag));
     for (i=start;i<end;i++) diag[i-start] = work2[i];
@@ -81,7 +81,7 @@ static PetscErrorCode MatGetDiagonal_Cross(Mat B,Vec d)
     PetscCall(PetscFree2(work1,work2));
   }
   PetscCall(VecCopy(ctx->diag,d));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode MatDestroy_Cross(Mat B)
@@ -93,7 +93,7 @@ static PetscErrorCode MatDestroy_Cross(Mat B)
   PetscCall(VecDestroy(&ctx->w));
   PetscCall(VecDestroy(&ctx->diag));
   PetscCall(PetscFree(ctx));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode SVDCrossGetProductMat(SVD svd,Mat A,Mat AT,Mat *C)
@@ -102,21 +102,27 @@ static PetscErrorCode SVDCrossGetProductMat(SVD svd,Mat A,Mat AT,Mat *C)
   SVD_CROSS_SHELL *ctx;
   PetscInt        n;
   VecType         vtype;
+  Mat             B;
 
   PetscFunctionBegin;
   if (cross->explicitmatrix) {
+    if (!svd->ishyperbolic || svd->swapped) B = (!svd->expltrans && svd->swapped)? AT: A;
+    else {  /* duplicate A and scale by signature */
+      PetscCall(MatDuplicate(A,MAT_COPY_VALUES,&B));
+      PetscCall(MatDiagonalScale(B,svd->omega,NULL));
+    }
     if (svd->expltrans) {  /* explicit transpose */
-      PetscCall(MatProductCreate(AT,A,NULL,C));
+      PetscCall(MatProductCreate(AT,B,NULL,C));
       PetscCall(MatProductSetType(*C,MATPRODUCT_AB));
     } else {  /* implicit transpose */
 #if defined(PETSC_USE_COMPLEX)
       SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"Must use explicit transpose with complex scalars");
 #else
       if (!svd->swapped) {
-        PetscCall(MatProductCreate(A,A,NULL,C));
+        PetscCall(MatProductCreate(A,B,NULL,C));
         PetscCall(MatProductSetType(*C,MATPRODUCT_AtB));
       } else {
-        PetscCall(MatProductCreate(AT,AT,NULL,C));
+        PetscCall(MatProductCreate(B,AT,NULL,C));
         PetscCall(MatProductSetType(*C,MATPRODUCT_ABt));
       }
 #endif
@@ -124,23 +130,23 @@ static PetscErrorCode SVDCrossGetProductMat(SVD svd,Mat A,Mat AT,Mat *C)
     PetscCall(MatProductSetFromOptions(*C));
     PetscCall(MatProductSymbolic(*C));
     PetscCall(MatProductNumeric(*C));
+    if (svd->ishyperbolic && !svd->swapped) PetscCall(MatDestroy(&B));
   } else {
     PetscCall(PetscNew(&ctx));
     ctx->A       = A;
     ctx->AT      = AT;
+    ctx->omega   = svd->omega;
     ctx->swapped = svd->swapped;
     PetscCall(MatCreateVecs(A,NULL,&ctx->w));
-    PetscCall(PetscLogObjectParent((PetscObject)svd,(PetscObject)ctx->w));
     PetscCall(MatGetLocalSize(A,NULL,&n));
     PetscCall(MatCreateShell(PetscObjectComm((PetscObject)svd),n,n,PETSC_DETERMINE,PETSC_DETERMINE,(void*)ctx,C));
     PetscCall(MatShellSetOperation(*C,MATOP_MULT,(void(*)(void))MatMult_Cross));
-    PetscCall(MatShellSetOperation(*C,MATOP_GET_DIAGONAL,(void(*)(void))MatGetDiagonal_Cross));
+    if (!svd->ishyperbolic || svd->swapped) PetscCall(MatShellSetOperation(*C,MATOP_GET_DIAGONAL,(void(*)(void))MatGetDiagonal_Cross));
     PetscCall(MatShellSetOperation(*C,MATOP_DESTROY,(void(*)(void))MatDestroy_Cross));
     PetscCall(MatGetVecType(A,&vtype));
     PetscCall(MatSetVecType(*C,vtype));
   }
-  PetscCall(PetscLogObjectParent((PetscObject)svd,(PetscObject)*C));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /* Convergence test relative to the norm of R (used in GSVD only) */
@@ -150,16 +156,22 @@ static PetscErrorCode EPSConv_Cross(EPS eps,PetscScalar eigr,PetscScalar eigi,Pe
 
   PetscFunctionBegin;
   *errest = res/PetscMax(svd->nrma,svd->nrmb);
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode SVDSetUp_Cross(SVD svd)
+static PetscErrorCode SVDSetUp_Cross(SVD svd)
 {
   SVD_CROSS      *cross = (SVD_CROSS*)svd->data;
   ST             st;
-  PetscBool      trackall,issinv;
+  PetscBool      trackall,issinv,isks;
+  EPSProblemType ptype;
+  EPSWhich       which;
+  Mat            Omega;
+  MatType        Atype;
+  PetscInt       n,N;
 
   PetscFunctionBegin;
+  if (svd->nsv==0 && svd->stop!=SVD_STOP_THRESHOLD) svd->nsv = 1;
   if (!cross->eps) PetscCall(SVDCrossGetEPS(svd,&cross->eps));
   PetscCall(MatDestroy(&cross->C));
   PetscCall(MatDestroy(&cross->D));
@@ -167,24 +179,42 @@ PetscErrorCode SVDSetUp_Cross(SVD svd)
   if (svd->isgeneralized) {
     PetscCall(SVDCrossGetProductMat(svd,svd->B,svd->BT,&cross->D));
     PetscCall(EPSSetOperators(cross->eps,cross->C,cross->D));
-    PetscCall(EPSSetProblemType(cross->eps,EPS_GHEP));
+    PetscCall(EPSGetProblemType(cross->eps,&ptype));
+    if (!ptype) PetscCall(EPSSetProblemType(cross->eps,EPS_GHEP));
+  } else if (svd->ishyperbolic && svd->swapped) {
+    PetscCall(MatGetType(svd->OP,&Atype));
+    PetscCall(MatGetSize(svd->A,NULL,&N));
+    PetscCall(MatGetLocalSize(svd->A,NULL,&n));
+    PetscCall(MatCreate(PetscObjectComm((PetscObject)svd),&Omega));
+    PetscCall(MatSetSizes(Omega,n,n,N,N));
+    PetscCall(MatSetType(Omega,Atype));
+    PetscCall(MatDiagonalSet(Omega,svd->omega,INSERT_VALUES));
+    PetscCall(EPSSetOperators(cross->eps,cross->C,Omega));
+    PetscCall(EPSSetProblemType(cross->eps,EPS_GHIEP));
+    PetscCall(MatDestroy(&Omega));
   } else {
     PetscCall(EPSSetOperators(cross->eps,cross->C,NULL));
     PetscCall(EPSSetProblemType(cross->eps,EPS_HEP));
   }
   if (!cross->usereps) {
     PetscCall(EPSGetST(cross->eps,&st));
+    PetscCall(PetscObjectTypeCompare((PetscObject)st,STSINVERT,&issinv));
+    PetscCall(PetscObjectTypeCompare((PetscObject)cross->eps,EPSKRYLOVSCHUR,&isks));
     if (svd->isgeneralized && svd->which==SVD_SMALLEST) {
-      PetscCall(STSetType(st,STSINVERT));
-      PetscCall(EPSSetTarget(cross->eps,0.0));
-      PetscCall(EPSSetWhichEigenpairs(cross->eps,EPS_TARGET_REAL));
+      if (cross->explicitmatrix && isks && !issinv) {  /* default to shift-and-invert */
+        PetscCall(STSetType(st,STSINVERT));
+        PetscCall(EPSSetTarget(cross->eps,0.0));
+        which = EPS_TARGET_REAL;
+      } else which = issinv?EPS_TARGET_REAL:EPS_SMALLEST_REAL;
     } else {
-      PetscCall(PetscObjectTypeCompare((PetscObject)st,STSINVERT,&issinv));
-      if (issinv) PetscCall(EPSSetWhichEigenpairs(cross->eps,EPS_TARGET_MAGNITUDE));
-      else PetscCall(EPSSetWhichEigenpairs(cross->eps,svd->which==SVD_LARGEST?EPS_LARGEST_REAL:EPS_SMALLEST_REAL));
+      if (issinv) which = EPS_TARGET_MAGNITUDE;
+      else if (svd->ishyperbolic) which = svd->which==SVD_LARGEST?EPS_LARGEST_MAGNITUDE:EPS_SMALLEST_MAGNITUDE;
+      else which = svd->which==SVD_LARGEST?EPS_LARGEST_MAGNITUDE:EPS_SMALLEST_MAGNITUDE;
     }
-    PetscCall(EPSSetDimensions(cross->eps,svd->nsv,svd->ncv,svd->mpd));
-    PetscCall(EPSSetTolerances(cross->eps,svd->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL/10.0:svd->tol,svd->max_it));
+    PetscCall(EPSSetWhichEigenpairs(cross->eps,which));
+    PetscCall(EPSSetDimensions(cross->eps,svd->nsv?svd->nsv:PETSC_CURRENT,svd->ncv,svd->mpd));
+    if (svd->stop==SVD_STOP_THRESHOLD) PetscCall(EPSSetThreshold(cross->eps,svd->thres*svd->thres,svd->threlative));
+    PetscCall(EPSSetTolerances(cross->eps,svd->tol==(PetscReal)PETSC_DETERMINE?SLEPC_DEFAULT_TOL/10.0:svd->tol,svd->max_it));
     switch (svd->conv) {
     case SVD_CONV_ABS:
       PetscCall(EPSSetConvergenceTest(cross->eps,EPS_CONV_ABS));break;
@@ -205,7 +235,6 @@ PetscErrorCode SVDSetUp_Cross(SVD svd)
       SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"User-defined convergence test not supported in this solver");
     }
   }
-  SVDCheckUnsupported(svd,SVD_FEATURE_STOPPING);
   /* Transfer the trackall option from svd to eps */
   PetscCall(SVDGetTrackAll(svd,&trackall));
   PetscCall(EPSSetTrackAll(cross->eps,trackall));
@@ -217,14 +246,14 @@ PetscErrorCode SVDSetUp_Cross(SVD svd)
   PetscCall(EPSSetUp(cross->eps));
   PetscCall(EPSGetDimensions(cross->eps,NULL,&svd->ncv,&svd->mpd));
   PetscCall(EPSGetTolerances(cross->eps,NULL,&svd->max_it));
-  if (svd->tol==PETSC_DEFAULT) svd->tol = SLEPC_DEFAULT_TOL;
+  if (svd->tol==(PetscReal)PETSC_DETERMINE) svd->tol = SLEPC_DEFAULT_TOL;
 
   svd->leftbasis = PETSC_FALSE;
   PetscCall(SVDAllocateSolution(svd,0));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode SVDSolve_Cross(SVD svd)
+static PetscErrorCode SVDSolve_Cross(SVD svd)
 {
   SVD_CROSS      *cross = (SVD_CROSS*)svd->data;
   PetscInt       i;
@@ -239,22 +268,26 @@ PetscErrorCode SVDSolve_Cross(SVD svd)
   for (i=0;i<svd->nconv;i++) {
     PetscCall(EPSGetEigenvalue(cross->eps,i,&lambda,NULL));
     sigma = PetscRealPart(lambda);
-    PetscCheck(sigma>-10*PETSC_MACHINE_EPSILON,PetscObjectComm((PetscObject)svd),PETSC_ERR_FP,"Negative eigenvalue computed by EPS: %g",(double)sigma);
-    if (sigma<0.0) {
-      PetscCall(PetscInfo(svd,"Negative eigenvalue computed by EPS: %g, resetting to 0\n",(double)sigma));
-      sigma = 0.0;
+    if (svd->ishyperbolic) svd->sigma[i] = PetscSqrtReal(PetscAbsReal(sigma));
+    else {
+      PetscCheck(sigma>-10*PETSC_MACHINE_EPSILON,PetscObjectComm((PetscObject)svd),PETSC_ERR_FP,"Negative eigenvalue computed by EPS: %g",(double)sigma);
+      if (sigma<0.0) {
+        PetscCall(PetscInfo(svd,"Negative eigenvalue computed by EPS: %g, resetting to 0\n",(double)sigma));
+        sigma = 0.0;
+      }
+      svd->sigma[i] = PetscSqrtReal(sigma);
     }
-    svd->sigma[i] = PetscSqrtReal(sigma);
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode SVDComputeVectors_Cross(SVD svd)
+static PetscErrorCode SVDComputeVectors_Cross(SVD svd)
 {
   SVD_CROSS         *cross = (SVD_CROSS*)svd->data;
   PetscInt          i,mloc,ploc;
-  Vec               u,v,x,uv;
-  PetscScalar       *dst,alpha,lambda;
+  Vec               u,v,x,uv,w,omega2=NULL;
+  Mat               Omega;
+  PetscScalar       *dst,alpha,lambda,*varray;
   const PetscScalar *src;
   PetscReal         nrm;
 
@@ -288,6 +321,28 @@ PetscErrorCode SVDComputeVectors_Cross(SVD svd)
     }
     PetscCall(VecDestroy(&v));
     PetscCall(VecDestroy(&u));
+  } else if (svd->ishyperbolic && svd->swapped) {  /* was solved as GHIEP, set u=Omega*u and normalize */
+    PetscCall(EPSGetOperators(cross->eps,NULL,&Omega));
+    PetscCall(MatCreateVecs(Omega,&w,NULL));
+    PetscCall(VecCreateSeq(PETSC_COMM_SELF,svd->ncv,&omega2));
+    PetscCall(VecGetArrayWrite(omega2,&varray));
+    for (i=0;i<svd->nconv;i++) {
+      PetscCall(BVGetColumn(svd->V,i,&v));
+      PetscCall(EPSGetEigenvector(cross->eps,i,v,NULL));
+      PetscCall(MatMult(Omega,v,w));
+      PetscCall(VecDot(v,w,&alpha));
+      svd->sign[i] = PetscSign(PetscRealPart(alpha));
+      varray[i] = svd->sign[i];
+      alpha = 1.0/PetscSqrtScalar(PetscAbsScalar(alpha));
+      PetscCall(VecScale(w,alpha));
+      PetscCall(VecCopy(w,v));
+      PetscCall(BVRestoreColumn(svd->V,i,&v));
+    }
+    PetscCall(BVSetSignature(svd->V,omega2));
+    PetscCall(VecRestoreArrayWrite(omega2,&varray));
+    PetscCall(VecDestroy(&omega2));
+    PetscCall(VecDestroy(&w));
+    PetscCall(SVDComputeVectors_Left(svd));
   } else {
     for (i=0;i<svd->nconv;i++) {
       PetscCall(BVGetColumn(svd->V,i,&v));
@@ -296,39 +351,51 @@ PetscErrorCode SVDComputeVectors_Cross(SVD svd)
     }
     PetscCall(SVDComputeVectors_Left(svd));
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode EPSMonitor_Cross(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *ctx)
 {
-  PetscInt       i;
+  PetscInt       i,ncv;
   SVD            svd = (SVD)ctx;
+  SVD_CROSS      *cross;
   PetscScalar    er,ei;
+  ST             st;
 
   PetscFunctionBegin;
+  if (svd->stop==SVD_STOP_THRESHOLD) {
+    cross = (SVD_CROSS*)svd->data;
+    PetscCall(EPSGetDimensions(cross->eps,NULL,&ncv,NULL));
+    if (ncv!=svd->ncv) {  /* reallocate */
+      PetscCall(SVDReallocateSolution(svd,ncv));
+      for (i=svd->ncv;i<ncv;i++) svd->perm[i] = i;
+      svd->ncv = ncv;
+    }
+  }
+  PetscCall(EPSGetST(eps,&st));
   for (i=0;i<PetscMin(nest,svd->ncv);i++) {
     er = eigr[i]; ei = eigi[i];
-    PetscCall(STBackTransform(eps->st,1,&er,&ei));
+    PetscCall(STBackTransform(st,1,&er,&ei));
     svd->sigma[i] = PetscSqrtReal(PetscAbsReal(PetscRealPart(er)));
     svd->errest[i] = errest[i];
   }
   PetscCall(SVDMonitor(svd,its,nconv,svd->sigma,svd->errest,nest));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode SVDSetFromOptions_Cross(PetscOptionItems *PetscOptionsObject,SVD svd)
+static PetscErrorCode SVDSetFromOptions_Cross(SVD svd,PetscOptionItems PetscOptionsObject)
 {
   PetscBool      set,val;
   SVD_CROSS      *cross = (SVD_CROSS*)svd->data;
   ST             st;
 
   PetscFunctionBegin;
-  PetscCall(PetscOptionsHead(PetscOptionsObject,"SVD Cross Options"));
+  PetscOptionsHeadBegin(PetscOptionsObject,"SVD Cross Options");
 
     PetscCall(PetscOptionsBool("-svd_cross_explicitmatrix","Use cross explicit matrix","SVDCrossSetExplicitMatrix",cross->explicitmatrix,&val,&set));
     if (set) PetscCall(SVDCrossSetExplicitMatrix(svd,val));
 
-  PetscCall(PetscOptionsTail());
+  PetscOptionsHeadEnd();
 
   if (!cross->eps) PetscCall(SVDCrossGetEPS(svd,&cross->eps));
   if (!cross->explicitmatrix && !cross->usereps) {
@@ -337,7 +404,7 @@ PetscErrorCode SVDSetFromOptions_Cross(PetscOptionItems *PetscOptionsObject,SVD
     PetscCall(STSetMatMode(st,ST_MATMODE_SHELL));
   }
   PetscCall(EPSSetFromOptions(cross->eps));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode SVDCrossSetExplicitMatrix_Cross(SVD svd,PetscBool explicitmatrix)
@@ -349,14 +416,14 @@ static PetscErrorCode SVDCrossSetExplicitMatrix_Cross(SVD svd,PetscBool explicit
     cross->explicitmatrix = explicitmatrix;
     svd->state = SVD_STATE_INITIAL;
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    SVDCrossSetExplicitMatrix - Indicate if the eigensolver operator A^T*A must
    be computed explicitly.
 
-   Logically Collective on svd
+   Logically Collective
 
    Input Parameters:
 +  svd         - singular value solver
@@ -375,7 +442,7 @@ PetscErrorCode SVDCrossSetExplicitMatrix(SVD svd,PetscBool explicitmat)
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidLogicalCollectiveBool(svd,explicitmat,2);
   PetscTryMethod(svd,"SVDCrossSetExplicitMatrix_C",(SVD,PetscBool),(svd,explicitmat));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode SVDCrossGetExplicitMatrix_Cross(SVD svd,PetscBool *explicitmat)
@@ -384,7 +451,7 @@ static PetscErrorCode SVDCrossGetExplicitMatrix_Cross(SVD svd,PetscBool *explici
 
   PetscFunctionBegin;
   *explicitmat = cross->explicitmatrix;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -406,9 +473,9 @@ PetscErrorCode SVDCrossGetExplicitMatrix(SVD svd,PetscBool *explicitmat)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  PetscValidBoolPointer(explicitmat,2);
+  PetscAssertPointer(explicitmat,2);
   PetscUseMethod(svd,"SVDCrossGetExplicitMatrix_C",(SVD,PetscBool*),(svd,explicitmat));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode SVDCrossSetEPS_Cross(SVD svd,EPS eps)
@@ -418,18 +485,17 @@ static PetscErrorCode SVDCrossSetEPS_Cross(SVD svd,EPS eps)
   PetscFunctionBegin;
   PetscCall(PetscObjectReference((PetscObject)eps));
   PetscCall(EPSDestroy(&cross->eps));
-  cross->eps = eps;
+  cross->eps     = eps;
   cross->usereps = PETSC_TRUE;
-  PetscCall(PetscLogObjectParent((PetscObject)svd,(PetscObject)cross->eps));
-  svd->state = SVD_STATE_INITIAL;
-  PetscFunctionReturn(0);
+  svd->state     = SVD_STATE_INITIAL;
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    SVDCrossSetEPS - Associate an eigensolver object (EPS) to the
    singular value solver.
 
-   Collective on svd
+   Collective
 
    Input Parameters:
 +  svd - singular value solver
@@ -446,7 +512,7 @@ PetscErrorCode SVDCrossSetEPS(SVD svd,EPS eps)
   PetscValidHeaderSpecific(eps,EPS_CLASSID,2);
   PetscCheckSameComm(svd,1,eps,2);
   PetscTryMethod(svd,"SVDCrossSetEPS_C",(SVD,EPS),(svd,eps));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode SVDCrossGetEPS_Cross(SVD svd,EPS *eps)
@@ -459,20 +525,19 @@ static PetscErrorCode SVDCrossGetEPS_Cross(SVD svd,EPS *eps)
     PetscCall(PetscObjectIncrementTabLevel((PetscObject)cross->eps,(PetscObject)svd,1));
     PetscCall(EPSSetOptionsPrefix(cross->eps,((PetscObject)svd)->prefix));
     PetscCall(EPSAppendOptionsPrefix(cross->eps,"svd_cross_"));
-    PetscCall(PetscLogObjectParent((PetscObject)svd,(PetscObject)cross->eps));
     PetscCall(PetscObjectSetOptions((PetscObject)cross->eps,((PetscObject)svd)->options));
-    PetscCall(EPSSetWhichEigenpairs(cross->eps,EPS_LARGEST_REAL));
+    PetscCall(EPSSetWhichEigenpairs(cross->eps,EPS_LARGEST_MAGNITUDE));
     PetscCall(EPSMonitorSet(cross->eps,EPSMonitor_Cross,svd,NULL));
   }
   *eps = cross->eps;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    SVDCrossGetEPS - Retrieve the eigensolver object (EPS) associated
    to the singular value solver.
 
-   Not Collective
+   Collective
 
    Input Parameter:
 .  svd - singular value solver
@@ -488,12 +553,12 @@ PetscErrorCode SVDCrossGetEPS(SVD svd,EPS *eps)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  PetscValidPointer(eps,2);
+  PetscAssertPointer(eps,2);
   PetscUseMethod(svd,"SVDCrossGetEPS_C",(SVD,EPS*),(svd,eps));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode SVDView_Cross(SVD svd,PetscViewer viewer)
+static PetscErrorCode SVDView_Cross(SVD svd,PetscViewer viewer)
 {
   SVD_CROSS      *cross = (SVD_CROSS*)svd->data;
   PetscBool      isascii;
@@ -507,10 +572,10 @@ PetscErrorCode SVDView_Cross(SVD svd,PetscViewer viewer)
     PetscCall(EPSView(cross->eps,viewer));
     PetscCall(PetscViewerASCIIPopTab(viewer));
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode SVDReset_Cross(SVD svd)
+static PetscErrorCode SVDReset_Cross(SVD svd)
 {
   SVD_CROSS      *cross = (SVD_CROSS*)svd->data;
 
@@ -518,10 +583,10 @@ PetscErrorCode SVDReset_Cross(SVD svd)
   PetscCall(EPSReset(cross->eps));
   PetscCall(MatDestroy(&cross->C));
   PetscCall(MatDestroy(&cross->D));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode SVDDestroy_Cross(SVD svd)
+static PetscErrorCode SVDDestroy_Cross(SVD svd)
 {
   SVD_CROSS      *cross = (SVD_CROSS*)svd->data;
 
@@ -532,7 +597,7 @@ PetscErrorCode SVDDestroy_Cross(SVD svd)
   PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDCrossGetEPS_C",NULL));
   PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDCrossSetExplicitMatrix_C",NULL));
   PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDCrossGetExplicitMatrix_C",NULL));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 SLEPC_EXTERN PetscErrorCode SVDCreate_Cross(SVD svd)
@@ -540,11 +605,12 @@ SLEPC_EXTERN PetscErrorCode SVDCreate_Cross(SVD svd)
   SVD_CROSS      *cross;
 
   PetscFunctionBegin;
-  PetscCall(PetscNewLog(svd,&cross));
+  PetscCall(PetscNew(&cross));
   svd->data = (void*)cross;
 
   svd->ops->solve          = SVDSolve_Cross;
   svd->ops->solveg         = SVDSolve_Cross;
+  svd->ops->solveh         = SVDSolve_Cross;
   svd->ops->setup          = SVDSetUp_Cross;
   svd->ops->setfromoptions = SVDSetFromOptions_Cross;
   svd->ops->destroy        = SVDDestroy_Cross;
@@ -555,5 +621,5 @@ SLEPC_EXTERN PetscErrorCode SVDCreate_Cross(SVD svd)
   PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDCrossGetEPS_C",SVDCrossGetEPS_Cross));
   PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDCrossSetExplicitMatrix_C",SVDCrossSetExplicitMatrix_Cross));
   PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDCrossGetExplicitMatrix_C",SVDCrossGetExplicitMatrix_Cross));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
