@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -46,16 +46,16 @@ static void multMatvec_PRIMME(void*,PRIMME_INT*,void*,PRIMME_INT*,int*,int*,stru
 static void par_GlobalSumReal(void *sendBuf,void *recvBuf,int *count,primme_svds_params *primme,int *ierr)
 {
   if (sendBuf == recvBuf) {
-    *ierr = MPIU_Allreduce(MPI_IN_PLACE,recvBuf,*count,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)primme->commInfo));CHKERRABORT(PetscObjectComm((PetscObject)primme->commInfo),*ierr);
+    *ierr = MPI_Allreduce(MPI_IN_PLACE,recvBuf,*count,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)primme->commInfo));
   } else {
-    *ierr = MPIU_Allreduce(sendBuf,recvBuf,*count,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)primme->commInfo));CHKERRABORT(PetscObjectComm((PetscObject)primme->commInfo),*ierr);
+    *ierr = MPI_Allreduce(sendBuf,recvBuf,*count,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)primme->commInfo));
   }
 }
 
 #if defined(SLEPC_HAVE_PRIMME3)
 static void par_broadcastReal(void *buf,int *count,primme_svds_params *primme,int *ierr)
 {
-  *ierr = MPI_Bcast(buf,*count,MPIU_REAL,0/*root*/,PetscObjectComm((PetscObject)primme->commInfo));CHKERRABORT(PetscObjectComm((PetscObject)primme->commInfo),*ierr);
+  *ierr = MPI_Bcast(buf,*count,MPIU_REAL,0/*root*/,PetscObjectComm((PetscObject)primme->commInfo));
 }
 #endif
 
@@ -71,10 +71,12 @@ static void convTestFun(double *sval,void *leftsvec,void *rightsvec,double *resN
   PetscReal      sigma = sval?*sval:0.0;
   PetscReal      r = resNorm?*resNorm:HUGE_VAL,errest;
 
-  *err = 1;
-  ierr = (*svd->converged)(svd,sigma,r,&errest,svd->convergedctx);CHKERRABORT(PetscObjectComm((PetscObject)svd),ierr);
-  *isconv = (errest<=svd->tol?1:0);
-  *err = 0;
+  ierr = (*svd->converged)(svd,sigma,r,&errest,svd->convergedctx);
+  if (ierr) *err = 1;
+  else {
+    *isconv = (errest<=svd->tol?1:0);
+    *err = 0;
+  }
 }
 
 static void monitorFun(void *basisSvals,int *basisSize,int *basisFlags,int *iblock,int *blockSize,void *basisNorms,int *numConverged,void *lockedSvals,int *numLocked,int *lockedFlags,void *lockedNorms,int *inner_its,void *LSRes,
@@ -84,7 +86,7 @@ static void monitorFun(void *basisSvals,int *basisSize,int *basisFlags,int *iblo
                        primme_event *event,int *stage,struct primme_svds_params *primme,int *err)
 {
 
-  PetscErrorCode ierr;
+  PetscErrorCode ierr = PETSC_SUCCESS;
   SVD            svd = (SVD)primme->commInfo;
   PetscInt       i,k,nerrest;
 
@@ -92,7 +94,7 @@ static void monitorFun(void *basisSvals,int *basisSize,int *basisFlags,int *iblo
   switch (*event) {
     case primme_event_outer_iteration:
       /* Update SVD */
-      svd->its = primme->stats.numOuterIterations;
+      PetscCallVoid(PetscIntCast(primme->stats.numOuterIterations,&svd->its));
       if (numConverged) svd->nconv = *numConverged;
       k = 0;
       if (lockedSvals && numLocked) for (i=0; i<*numLocked && k<svd->ncv; i++) svd->sigma[k++] = ((PetscReal*)lockedSvals)[i];
@@ -103,18 +105,18 @@ static void monitorFun(void *basisSvals,int *basisSize,int *basisFlags,int *iblo
       }
       if (basisSvals && basisSize) for (i=0; i<*basisSize && k<svd->ncv; i++) svd->sigma[k++] = ((PetscReal*)basisSvals)[i];
       /* Show progress */
-      ierr = SVDMonitor(svd,svd->its,numConverged?*numConverged:0,svd->sigma,svd->errest,nerrest);CHKERRABORT(PetscObjectComm((PetscObject)svd),ierr);
+      ierr = SVDMonitor(svd,svd->its,numConverged?*numConverged:0,svd->sigma,svd->errest,nerrest);
       break;
 #if defined(SLEPC_HAVE_PRIMME3)
     case primme_event_message:
       /* Print PRIMME information messages */
-      ierr = PetscInfo(svd,msg);CHKERRABORT(PetscObjectComm((PetscObject)svd),ierr);
+      ierr = PetscInfo(svd,"%s\n",msg);
       break;
 #endif
     default:
       break;
   }
-  *err = 0;
+  *err = (ierr!=0)? 1: 0;
 }
 #endif /* SLEPC_HAVE_PRIMME2p2 */
 
@@ -128,56 +130,57 @@ static void multMatvec_PRIMME(void *xa,PRIMME_INT *ldx,void *ya,PRIMME_INT *ldy,
   PetscFunctionBegin;
   for (i=0;i<*blockSize;i++) {
     if (*transpose) {
-      *ierr = VecPlaceArray(y,(PetscScalar*)xa+(*ldx)*i);CHKERRABORT(PetscObjectComm((PetscObject)svd),*ierr);
-      *ierr = VecPlaceArray(x,(PetscScalar*)ya+(*ldy)*i);CHKERRABORT(PetscObjectComm((PetscObject)svd),*ierr);
-      *ierr = SVDMatMult(svd,PETSC_TRUE,y,x);CHKERRABORT(PetscObjectComm((PetscObject)svd),*ierr);
+      PetscCallAbort(PetscObjectComm((PetscObject)svd),VecPlaceArray(y,(PetscScalar*)xa+(*ldx)*i));
+      PetscCallAbort(PetscObjectComm((PetscObject)svd),VecPlaceArray(x,(PetscScalar*)ya+(*ldy)*i));
+      PetscCallAbort(PetscObjectComm((PetscObject)svd),MatMult(svd->AT,y,x));
     } else {
-      *ierr = VecPlaceArray(x,(PetscScalar*)xa+(*ldx)*i);CHKERRABORT(PetscObjectComm((PetscObject)svd),*ierr);
-      *ierr = VecPlaceArray(y,(PetscScalar*)ya+(*ldy)*i);CHKERRABORT(PetscObjectComm((PetscObject)svd),*ierr);
-      *ierr = SVDMatMult(svd,PETSC_FALSE,x,y);CHKERRABORT(PetscObjectComm((PetscObject)svd),*ierr);
+      PetscCallAbort(PetscObjectComm((PetscObject)svd),VecPlaceArray(x,(PetscScalar*)xa+(*ldx)*i));
+      PetscCallAbort(PetscObjectComm((PetscObject)svd),VecPlaceArray(y,(PetscScalar*)ya+(*ldy)*i));
+      PetscCallAbort(PetscObjectComm((PetscObject)svd),MatMult(svd->A,x,y));
     }
-    *ierr = VecResetArray(x);CHKERRABORT(PetscObjectComm((PetscObject)svd),*ierr);
-    *ierr = VecResetArray(y);CHKERRABORT(PetscObjectComm((PetscObject)svd),*ierr);
+    PetscCallAbort(PetscObjectComm((PetscObject)svd),VecResetArray(x));
+    PetscCallAbort(PetscObjectComm((PetscObject)svd),VecResetArray(y));
   }
   PetscFunctionReturnVoid();
 }
 
-PetscErrorCode SVDSetUp_PRIMME(SVD svd)
+static PetscErrorCode SVDSetUp_PRIMME(SVD svd)
 {
-  PetscErrorCode     ierr;
   PetscMPIInt        numProcs,procID;
   PetscInt           n,m,nloc,mloc;
   SVD_PRIMME         *ops = (SVD_PRIMME*)svd->data;
   primme_svds_params *primme = &ops->primme;
 
   PetscFunctionBegin;
-  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)svd),&numProcs);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)svd),&procID);CHKERRQ(ierr);
+  SVDCheckStandard(svd);
+  SVDCheckDefinite(svd);
+  PetscCallMPI(MPI_Comm_size(PetscObjectComm((PetscObject)svd),&numProcs));
+  PetscCallMPI(MPI_Comm_rank(PetscObjectComm((PetscObject)svd),&procID));
 
   /* Check some constraints and set some default values */
-  ierr = SVDMatGetSize(svd,&m,&n);CHKERRQ(ierr);
-  ierr = SVDMatGetLocalSize(svd,&mloc,&nloc);CHKERRQ(ierr);
-  ierr = SVDSetDimensions_Default(svd);CHKERRQ(ierr);
-  if (svd->max_it==PETSC_DEFAULT) svd->max_it = PETSC_MAX_INT;
+  PetscCall(MatGetSize(svd->A,&m,&n));
+  PetscCall(MatGetLocalSize(svd->A,&mloc,&nloc));
+  PetscCall(SVDSetDimensions_Default(svd));
+  if (svd->max_it==PETSC_DETERMINE) svd->max_it = PETSC_INT_MAX;
   svd->leftbasis = PETSC_TRUE;
   SVDCheckUnsupported(svd,SVD_FEATURE_STOPPING);
 #if !defined(SLEPC_HAVE_PRIMME2p2)
-  if (svd->converged != SVDConvergedAbsolute) { ierr = PetscInfo(svd,"Warning: using absolute convergence test\n");CHKERRQ(ierr); }
+  if (svd->converged != SVDConvergedAbsolute) PetscCall(PetscInfo(svd,"Warning: using absolute convergence test\n"));
 #endif
 
   /* Transfer SLEPc options to PRIMME options */
   primme_svds_free(primme);
   primme_svds_initialize(primme);
-  primme->m             = m;
-  primme->n             = n;
-  primme->mLocal        = mloc;
-  primme->nLocal        = nloc;
-  primme->numSvals      = svd->nsv;
+  primme->m             = (PRIMME_INT)m;
+  primme->n             = (PRIMME_INT)n;
+  primme->mLocal        = (PRIMME_INT)mloc;
+  primme->nLocal        = (PRIMME_INT)nloc;
+  primme->numSvals      = (int)svd->nsv;
   primme->matrix        = ops;
   primme->commInfo      = svd;
-  primme->maxMatvecs    = svd->max_it;
+  primme->maxMatvecs    = (PRIMME_INT)svd->max_it;
 #if !defined(SLEPC_HAVE_PRIMME2p2)
-  primme->eps           = svd->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL:svd->tol;
+  primme->eps           = SlepcDefaultTol(svd->tol);
 #endif
   primme->numProcs      = numProcs;
   primme->procID        = procID;
@@ -191,7 +194,7 @@ PetscErrorCode SVDSetUp_PRIMME(SVD svd)
   primme->convTestFun   = convTestFun;
   primme->monitorFun    = monitorFun;
 #endif
-  if (ops->bs > 0) primme->maxBlockSize = ops->bs;
+  if (ops->bs > 0) primme->maxBlockSize = (int)ops->bs;
 
   switch (svd->which) {
     case SVD_LARGEST:
@@ -200,154 +203,147 @@ PetscErrorCode SVDSetUp_PRIMME(SVD svd)
     case SVD_SMALLEST:
       primme->target = primme_svds_smallest;
       break;
-    default:
-      SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"'which' value not supported by PRIMME");
-      break;
   }
 
   /* If user sets mpd or ncv, maxBasisSize is modified */
-  if (svd->mpd!=PETSC_DEFAULT) {
-    primme->maxBasisSize = svd->mpd;
-    if (svd->ncv!=PETSC_DEFAULT) { ierr = PetscInfo(svd,"Warning: 'ncv' is ignored by PRIMME\n");CHKERRQ(ierr); }
-  } else if (svd->ncv!=PETSC_DEFAULT) primme->maxBasisSize = svd->ncv;
+  if (svd->mpd!=PETSC_DETERMINE) {
+    primme->maxBasisSize = (int)svd->mpd;
+    if (svd->ncv!=PETSC_DETERMINE) PetscCall(PetscInfo(svd,"Warning: 'ncv' is ignored by PRIMME\n"));
+  } else if (svd->ncv!=PETSC_DETERMINE) primme->maxBasisSize = (int)svd->ncv;
 
-  if (primme_svds_set_method(ops->method,(primme_preset_method)EPS_PRIMME_DEFAULT_MIN_TIME,PRIMME_DEFAULT_METHOD,primme) < 0) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"PRIMME method not valid");
+  PetscCheck(primme_svds_set_method(ops->method,(primme_preset_method)EPS_PRIMME_DEFAULT_MIN_TIME,PRIMME_DEFAULT_METHOD,primme)>=0,PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"PRIMME method not valid");
 
-  svd->mpd = primme->maxBasisSize;
-  svd->ncv = (primme->locking?svd->nsv:0)+primme->maxBasisSize;
-  ops->bs  = primme->maxBlockSize;
+  svd->mpd = (PetscInt)primme->maxBasisSize;
+  svd->ncv = (PetscInt)(primme->locking?svd->nsv:0)+primme->maxBasisSize;
+  ops->bs  = (PetscInt)primme->maxBlockSize;
 
   /* Set workspace */
-  ierr = SVDAllocateSolution(svd,0);CHKERRQ(ierr);
+  PetscCall(SVDAllocateSolution(svd,0));
 
   /* Prepare auxiliary vectors */
-  if (!ops->x) {
-    ierr = MatCreateVecsEmpty(svd->A,&ops->x,&ops->y);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)svd,(PetscObject)ops->x);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)svd,(PetscObject)ops->y);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
+  if (!ops->x) PetscCall(MatCreateVecsEmpty(svd->A,&ops->x,&ops->y));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode SVDSolve_PRIMME(SVD svd)
+static PetscErrorCode SVDSolve_PRIMME(SVD svd)
 {
-  PetscErrorCode ierr;
   SVD_PRIMME     *ops = (SVD_PRIMME*)svd->data;
   PetscScalar    *svecs, *a;
-  PetscInt       i,ierrprimme;
+  PetscInt       i,ierrprimme,ld;
   PetscReal      *svals,*rnorms;
 
   PetscFunctionBegin;
   /* Reset some parameters left from previous runs */
   ops->primme.aNorm    = 0.0;
-  ops->primme.initSize = svd->nini;
+  ops->primme.initSize = (int)svd->nini;
   ops->primme.iseed[0] = -1;
   ops->primme.iseed[1] = -1;
   ops->primme.iseed[2] = -1;
   ops->primme.iseed[3] = -1;
 
   /* Allocating left and right singular vectors contiguously */
-  ierr = PetscCalloc1(ops->primme.numSvals*(ops->primme.mLocal+ops->primme.nLocal),&svecs);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory((PetscObject)svd,sizeof(PetscReal)*ops->primme.numSvals*(ops->primme.mLocal+ops->primme.nLocal));CHKERRQ(ierr);
+  PetscCall(PetscCalloc1(ops->primme.numSvals*(ops->primme.mLocal+ops->primme.nLocal),&svecs));
 
   /* Call PRIMME solver */
-  ierr = PetscMalloc2(svd->ncv,&svals,svd->ncv,&rnorms);CHKERRQ(ierr);
+  PetscCall(PetscMalloc2(svd->ncv,&svals,svd->ncv,&rnorms));
   ierrprimme = PRIMME_DRIVER(svals,svecs,rnorms,&ops->primme);
   for (i=0;i<svd->ncv;i++) svd->sigma[i] = svals[i];
   for (i=0;i<svd->ncv;i++) svd->errest[i] = rnorms[i];
-  ierr = PetscFree2(svals,rnorms);CHKERRQ(ierr);
+  PetscCall(PetscFree2(svals,rnorms));
 
   /* Copy left and right singular vectors into svd */
-  ierr = BVGetArray(svd->U,&a);CHKERRQ(ierr);
-  ierr = PetscArraycpy(a,svecs,ops->primme.mLocal*ops->primme.initSize);CHKERRQ(ierr);
-  ierr = BVRestoreArray(svd->U,&a);CHKERRQ(ierr);
+  PetscCall(BVGetLeadingDimension(svd->U,&ld));
+  PetscCall(BVGetArray(svd->U,&a));
+  for (i=0;i<ops->primme.initSize;i++) PetscCall(PetscArraycpy(a+i*ld,svecs+i*ops->primme.mLocal,ops->primme.mLocal));
+  PetscCall(BVRestoreArray(svd->U,&a));
 
-  ierr = BVGetArray(svd->V,&a);CHKERRQ(ierr);
-  ierr = PetscArraycpy(a,svecs+ops->primme.mLocal*ops->primme.initSize,ops->primme.nLocal*ops->primme.initSize);CHKERRQ(ierr);
-  ierr = BVRestoreArray(svd->V,&a);CHKERRQ(ierr);
+  PetscCall(BVGetLeadingDimension(svd->V,&ld));
+  PetscCall(BVGetArray(svd->V,&a));
+  for (i=0;i<ops->primme.initSize;i++) PetscCall(PetscArraycpy(a+i*ld,svecs+ops->primme.mLocal*ops->primme.initSize+i*ops->primme.nLocal,ops->primme.nLocal));
+  PetscCall(BVRestoreArray(svd->V,&a));
 
-  ierr = PetscFree(svecs);CHKERRQ(ierr);
+  PetscCall(PetscFree(svecs));
 
-  svd->nconv  = ops->primme.initSize >= 0 ? ops->primme.initSize : 0;
+  svd->nconv  = ops->primme.initSize >= 0 ? (PetscInt)ops->primme.initSize : 0;
   svd->reason = svd->nconv >= svd->nsv ? SVD_CONVERGED_TOL: SVD_DIVERGED_ITS;
-  svd->its    = ops->primme.stats.numOuterIterations;
+  PetscCall(PetscIntCast(ops->primme.stats.numOuterIterations,&svd->its));
 
   /* Process PRIMME error code */
-  if (ierrprimme == 0) {
-    /* no error */
-  } else if (ierrprimme%100 == -1) SETERRQ1(PetscObjectComm((PetscObject)svd),PETSC_ERR_LIB,"PRIMME library failed with error code=%d: unexpected error",ierrprimme);
-  else if (ierrprimme%100 == -2) SETERRQ1(PetscObjectComm((PetscObject)svd),PETSC_ERR_LIB,"PRIMME library failed with error code=%d: allocation error",ierrprimme);
-  else if (ierrprimme%100 == -3) {
-    /* stop by maximum number of iteration or matvecs */
-  } else if (ierrprimme%100 >= -39) SETERRQ1(PetscObjectComm((PetscObject)svd),PETSC_ERR_LIB,"PRIMME library failed with error code=%d: configuration error; check PRIMME's manual",ierrprimme);
-  else SETERRQ1(PetscObjectComm((PetscObject)svd),PETSC_ERR_LIB,"PRIMME library failed with error code=%d: runtime error; check PRIMME's manual",ierrprimme);
-  PetscFunctionReturn(0);
+  if (ierrprimme != 0) {
+    switch (ierrprimme%100) {
+      case -1:
+        SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_LIB,"PRIMME library failed with error code=%" PetscInt_FMT ": unexpected error",ierrprimme);
+      case -2:
+        SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_LIB,"PRIMME library failed with error code=%" PetscInt_FMT ": allocation error",ierrprimme);
+      case -3: /* stop due to maximum number of iterations or matvecs */
+        break;
+      default:
+        PetscCheck(ierrprimme<-39,PetscObjectComm((PetscObject)svd),PETSC_ERR_LIB,"PRIMME library failed with error code=%" PetscInt_FMT ": configuration error; check PRIMME's manual",ierrprimme);
+        PetscCheck(ierrprimme>=-39,PetscObjectComm((PetscObject)svd),PETSC_ERR_LIB,"PRIMME library failed with error code=%" PetscInt_FMT ": runtime error; check PRIMME's manual",ierrprimme);
+    }
+  }
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode SVDReset_PRIMME(SVD svd)
+static PetscErrorCode SVDReset_PRIMME(SVD svd)
 {
-  PetscErrorCode ierr;
   SVD_PRIMME     *ops = (SVD_PRIMME*)svd->data;
 
   PetscFunctionBegin;
   primme_svds_free(&ops->primme);
-  ierr = VecDestroy(&ops->x);CHKERRQ(ierr);
-  ierr = VecDestroy(&ops->y);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
+  PetscCall(VecDestroy(&ops->x));
+  PetscCall(VecDestroy(&ops->y));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode SVDDestroy_PRIMME(SVD svd)
+static PetscErrorCode SVDDestroy_PRIMME(SVD svd)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
-  ierr = PetscFree(svd->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMESetBlockSize_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMEGetBlockSize_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMESetMethod_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMEGetMethod_C",NULL);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
+  PetscCall(PetscFree(svd->data));
+  PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMESetBlockSize_C",NULL));
+  PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMEGetBlockSize_C",NULL));
+  PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMESetMethod_C",NULL));
+  PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMEGetMethod_C",NULL));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode SVDView_PRIMME(SVD svd,PetscViewer viewer)
+static PetscErrorCode SVDView_PRIMME(SVD svd,PetscViewer viewer)
 {
-  PetscErrorCode ierr;
   PetscBool      isascii;
   SVD_PRIMME     *ctx = (SVD_PRIMME*)svd->data;
   PetscMPIInt    rank;
 
   PetscFunctionBegin;
-  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  PetscCall(PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii));
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  block size=%D\n",ctx->bs);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  solver method: %s\n",SVDPRIMMEMethods[(SVDPRIMMEMethod)ctx->method]);CHKERRQ(ierr);
+    PetscCall(PetscViewerASCIIPrintf(viewer,"  block size=%" PetscInt_FMT "\n",ctx->bs));
+    PetscCall(PetscViewerASCIIPrintf(viewer,"  solver method: %s\n",SVDPRIMMEMethods[(SVDPRIMMEMethod)ctx->method]));
 
     /* Display PRIMME params */
-    ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)svd),&rank);CHKERRQ(ierr);
+    PetscCallMPI(MPI_Comm_rank(PetscObjectComm((PetscObject)svd),&rank));
     if (!rank) primme_svds_display_params(ctx->primme);
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode SVDSetFromOptions_PRIMME(PetscOptionItems *PetscOptionsObject,SVD svd)
+static PetscErrorCode SVDSetFromOptions_PRIMME(SVD svd,PetscOptionItems PetscOptionsObject)
 {
-  PetscErrorCode  ierr;
   SVD_PRIMME      *ctx = (SVD_PRIMME*)svd->data;
   PetscInt        bs;
   SVDPRIMMEMethod meth;
   PetscBool       flg;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsHead(PetscOptionsObject,"SVD PRIMME Options");CHKERRQ(ierr);
+  PetscOptionsHeadBegin(PetscOptionsObject,"SVD PRIMME Options");
 
-    ierr = PetscOptionsInt("-svd_primme_blocksize","Maximum block size","SVDPRIMMESetBlockSize",ctx->bs,&bs,&flg);CHKERRQ(ierr);
-    if (flg) { ierr = SVDPRIMMESetBlockSize(svd,bs);CHKERRQ(ierr); }
+    PetscCall(PetscOptionsInt("-svd_primme_blocksize","Maximum block size","SVDPRIMMESetBlockSize",ctx->bs,&bs,&flg));
+    if (flg) PetscCall(SVDPRIMMESetBlockSize(svd,bs));
 
-    ierr = PetscOptionsEnum("-svd_primme_method","Method for solving the singular value problem","SVDPRIMMESetMethod",SVDPRIMMEMethods,(PetscEnum)ctx->method,(PetscEnum*)&meth,&flg);CHKERRQ(ierr);
-    if (flg) { ierr = SVDPRIMMESetMethod(svd,meth);CHKERRQ(ierr); }
+    PetscCall(PetscOptionsEnum("-svd_primme_method","Method for solving the singular value problem","SVDPRIMMESetMethod",SVDPRIMMEMethods,(PetscEnum)ctx->method,(PetscEnum*)&meth,&flg));
+    if (flg) PetscCall(SVDPRIMMESetMethod(svd,meth));
 
-  ierr = PetscOptionsTail();CHKERRQ(ierr);
-  PetscFunctionReturn(0);
+  PetscOptionsHeadEnd();
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode SVDPRIMMESetBlockSize_PRIMME(SVD svd,PetscInt bs)
@@ -355,16 +351,18 @@ static PetscErrorCode SVDPRIMMESetBlockSize_PRIMME(SVD svd,PetscInt bs)
   SVD_PRIMME *ops = (SVD_PRIMME*)svd->data;
 
   PetscFunctionBegin;
-  if (bs == PETSC_DEFAULT) ops->bs = 0;
-  else if (bs <= 0) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"PRIMME: block size must be positive");
-  else ops->bs = bs;
-  PetscFunctionReturn(0);
+  if (bs == PETSC_DEFAULT || bs == PETSC_DECIDE) ops->bs = 0;
+  else {
+    PetscCheck(bs>0,PetscObjectComm((PetscObject)svd),PETSC_ERR_ARG_OUTOFRANGE,"PRIMME: block size must be positive");
+    ops->bs = bs;
+  }
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    SVDPRIMMESetBlockSize - The maximum block size that PRIMME will try to use.
 
-   Logically Collective on svd
+   Logically Collective
 
    Input Parameters:
 +  svd - the singular value solver context
@@ -388,13 +386,11 @@ static PetscErrorCode SVDPRIMMESetBlockSize_PRIMME(SVD svd,PetscInt bs)
 @*/
 PetscErrorCode SVDPRIMMESetBlockSize(SVD svd,PetscInt bs)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidLogicalCollectiveInt(svd,bs,2);
-  ierr = PetscTryMethod(svd,"SVDPRIMMESetBlockSize_C",(SVD,PetscInt),(svd,bs));CHKERRQ(ierr);
-  PetscFunctionReturn(0);
+  PetscTryMethod(svd,"SVDPRIMMESetBlockSize_C",(SVD,PetscInt),(svd,bs));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode SVDPRIMMEGetBlockSize_PRIMME(SVD svd,PetscInt *bs)
@@ -403,7 +399,7 @@ static PetscErrorCode SVDPRIMMEGetBlockSize_PRIMME(SVD svd,PetscInt *bs)
 
   PetscFunctionBegin;
   *bs = ops->bs;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -423,13 +419,11 @@ static PetscErrorCode SVDPRIMMEGetBlockSize_PRIMME(SVD svd,PetscInt *bs)
 @*/
 PetscErrorCode SVDPRIMMEGetBlockSize(SVD svd,PetscInt *bs)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  PetscValidIntPointer(bs,2);
-  ierr = PetscUseMethod(svd,"SVDPRIMMEGetBlockSize_C",(SVD,PetscInt*),(svd,bs));CHKERRQ(ierr);
-  PetscFunctionReturn(0);
+  PetscAssertPointer(bs,2);
+  PetscUseMethod(svd,"SVDPRIMMEGetBlockSize_C",(SVD,PetscInt*),(svd,bs));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode SVDPRIMMESetMethod_PRIMME(SVD svd,SVDPRIMMEMethod method)
@@ -438,13 +432,13 @@ static PetscErrorCode SVDPRIMMESetMethod_PRIMME(SVD svd,SVDPRIMMEMethod method)
 
   PetscFunctionBegin;
   ops->method = (primme_svds_preset_method)method;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    SVDPRIMMESetMethod - Sets the method for the PRIMME SVD solver.
 
-   Logically Collective on svd
+   Logically Collective
 
    Input Parameters:
 +  svd - the singular value solver context
@@ -462,13 +456,11 @@ static PetscErrorCode SVDPRIMMESetMethod_PRIMME(SVD svd,SVDPRIMMEMethod method)
 @*/
 PetscErrorCode SVDPRIMMESetMethod(SVD svd,SVDPRIMMEMethod method)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
   PetscValidLogicalCollectiveEnum(svd,method,2);
-  ierr = PetscTryMethod(svd,"SVDPRIMMESetMethod_C",(SVD,SVDPRIMMEMethod),(svd,method));CHKERRQ(ierr);
-  PetscFunctionReturn(0);
+  PetscTryMethod(svd,"SVDPRIMMESetMethod_C",(SVD,SVDPRIMMEMethod),(svd,method));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 static PetscErrorCode SVDPRIMMEGetMethod_PRIMME(SVD svd,SVDPRIMMEMethod *method)
@@ -477,7 +469,7 @@ static PetscErrorCode SVDPRIMMEGetMethod_PRIMME(SVD svd,SVDPRIMMEMethod *method)
 
   PetscFunctionBegin;
   *method = (SVDPRIMMEMethod)ops->method;
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
@@ -497,22 +489,19 @@ static PetscErrorCode SVDPRIMMEGetMethod_PRIMME(SVD svd,SVDPRIMMEMethod *method)
 @*/
 PetscErrorCode SVDPRIMMEGetMethod(SVD svd,SVDPRIMMEMethod *method)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(svd,SVD_CLASSID,1);
-  PetscValidPointer(method,2);
-  ierr = PetscUseMethod(svd,"SVDPRIMMEGetMethod_C",(SVD,SVDPRIMMEMethod*),(svd,method));CHKERRQ(ierr);
-  PetscFunctionReturn(0);
+  PetscAssertPointer(method,2);
+  PetscUseMethod(svd,"SVDPRIMMEGetMethod_C",(SVD,SVDPRIMMEMethod*),(svd,method));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 SLEPC_EXTERN PetscErrorCode SVDCreate_PRIMME(SVD svd)
 {
-  PetscErrorCode ierr;
   SVD_PRIMME     *primme;
 
   PetscFunctionBegin;
-  ierr = PetscNewLog(svd,&primme);CHKERRQ(ierr);
+  PetscCall(PetscNew(&primme));
   svd->data = (void*)primme;
 
   primme_svds_initialize(&primme->primme);
@@ -527,10 +516,9 @@ SLEPC_EXTERN PetscErrorCode SVDCreate_PRIMME(SVD svd)
   svd->ops->reset          = SVDReset_PRIMME;
   svd->ops->view           = SVDView_PRIMME;
 
-  ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMESetBlockSize_C",SVDPRIMMESetBlockSize_PRIMME);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMEGetBlockSize_C",SVDPRIMMEGetBlockSize_PRIMME);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMESetMethod_C",SVDPRIMMESetMethod_PRIMME);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMEGetMethod_C",SVDPRIMMEGetMethod_PRIMME);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
+  PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMESetBlockSize_C",SVDPRIMMESetBlockSize_PRIMME));
+  PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMEGetBlockSize_C",SVDPRIMMEGetBlockSize_PRIMME));
+  PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMESetMethod_C",SVDPRIMMESetMethod_PRIMME));
+  PetscCall(PetscObjectComposeFunction((PetscObject)svd,"SVDPRIMMEGetMethod_C",SVDPRIMMEGetMethod_PRIMME));
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
-
