@@ -21,7 +21,7 @@ typedef struct {
   PetscScalar *vr,*vi;   /* array of vertices (vi not used in complex scalars) */
 } RG_POLYGON;
 
-PetscErrorCode RGComputeBoundingBox_Polygon(RG,PetscReal*,PetscReal*,PetscReal*,PetscReal*);
+static PetscErrorCode RGComputeBoundingBox_Polygon(RG,PetscReal*,PetscReal*,PetscReal*,PetscReal*);
 
 #if !defined(PETSC_USE_COMPLEX)
 static PetscBool CheckSymmetry(PetscInt n,PetscScalar *vr,PetscScalar *vi)
@@ -74,13 +74,13 @@ static PetscErrorCode RGPolygonSetVertices_Polygon(RG rg,PetscInt n,PetscScalar
     ctx->vi[i] = vi[i];
 #endif
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@
    RGPolygonSetVertices - Sets the vertices that define the polygon region.
 
-   Logically Collective on rg
+   Logically Collective
 
    Input Parameters:
 +  rg - the region context
@@ -112,15 +112,15 @@ PetscErrorCode RGPolygonSetVertices(RG rg,PetscInt n,PetscScalar vr[],PetscScala
   PetscFunctionBegin;
   PetscValidHeaderSpecific(rg,RG_CLASSID,1);
   PetscValidLogicalCollectiveInt(rg,n,2);
-  PetscValidScalarPointer(vr,3);
+  PetscAssertPointer(vr,3);
 #if !defined(PETSC_USE_COMPLEX)
-  PetscValidScalarPointer(vi,4);
+  PetscAssertPointer(vi,4);
 #endif
   PetscTryMethod(rg,"RGPolygonSetVertices_C",(RG,PetscInt,PetscScalar*,PetscScalar*),(rg,n,vr,vi));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-static PetscErrorCode RGPolygonGetVertices_Polygon(RG rg,PetscInt *n,PetscScalar **vr,PetscScalar **vi)
+static PetscErrorCode RGPolygonGetVertices_Polygon(RG rg,PetscInt *n,PetscScalar *vr[],PetscScalar *vi[])
 {
   RG_POLYGON     *ctx = (RG_POLYGON*)rg->data;
   PetscInt       i;
@@ -143,7 +143,7 @@ static PetscErrorCode RGPolygonGetVertices_Polygon(RG rg,PetscInt *n,PetscScalar
     }
   }
 #endif
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 /*@C
@@ -168,15 +168,15 @@ static PetscErrorCode RGPolygonGetVertices_Polygon(RG rg,PetscInt *n,PetscScalar
 
 .seealso: RGPolygonSetVertices()
 @*/
-PetscErrorCode RGPolygonGetVertices(RG rg,PetscInt *n,PetscScalar **vr,PetscScalar **vi)
+PetscErrorCode RGPolygonGetVertices(RG rg,PetscInt *n,PetscScalar *vr[],PetscScalar *vi[]) PeNS
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(rg,RG_CLASSID,1);
   PetscUseMethod(rg,"RGPolygonGetVertices_C",(RG,PetscInt*,PetscScalar**,PetscScalar**),(rg,n,vr,vi));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGView_Polygon(RG rg,PetscViewer viewer)
+static PetscErrorCode RGView_Polygon(RG rg,PetscViewer viewer)
 {
   RG_POLYGON     *ctx = (RG_POLYGON*)rg->data;
   PetscBool      isdraw,isascii;
@@ -247,19 +247,19 @@ PetscErrorCode RGView_Polygon(RG rg,PetscViewer viewer)
     PetscCall(PetscDrawSave(draw));
     PetscCall(PetscDrawPause(draw));
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGIsTrivial_Polygon(RG rg,PetscBool *trivial)
+static PetscErrorCode RGIsTrivial_Polygon(RG rg,PetscBool *trivial)
 {
   RG_POLYGON *ctx = (RG_POLYGON*)rg->data;
 
   PetscFunctionBegin;
   *trivial = PetscNot(ctx->n);
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGComputeContour_Polygon(RG rg,PetscInt n,PetscScalar *ucr,PetscScalar *uci)
+static PetscErrorCode RGComputeContour_Polygon(RG rg,PetscInt n,PetscScalar *ucr,PetscScalar *uci)
 {
   RG_POLYGON     *ctx = (RG_POLYGON*)rg->data;
   PetscReal      length,h,d,rem=0.0;
@@ -326,10 +326,10 @@ PetscErrorCode RGComputeContour_Polygon(RG rg,PetscInt n,PetscScalar *ucr,PetscS
   }
   if (!ucr) PetscCall(PetscFree(cr));
   if (!uci) PetscCall(PetscFree(ci));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGComputeBoundingBox_Polygon(RG rg,PetscReal *a,PetscReal *b,PetscReal *c,PetscReal *d)
+static PetscErrorCode RGComputeBoundingBox_Polygon(RG rg,PetscReal *a,PetscReal *b,PetscReal *c,PetscReal *d)
 {
   RG_POLYGON *ctx = (RG_POLYGON*)rg->data;
   PetscInt   i;
@@ -352,10 +352,10 @@ PetscErrorCode RGComputeBoundingBox_Polygon(RG rg,PetscReal *a,PetscReal *b,Pets
     if (d) *d = PetscMax(*d,ctx->vi[i]);
 #endif
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGCheckInside_Polygon(RG rg,PetscReal px,PetscReal py,PetscInt *inout)
+static PetscErrorCode RGCheckInside_Polygon(RG rg,PetscReal px,PetscReal py,PetscInt *inout)
 {
   RG_POLYGON *ctx = (RG_POLYGON*)rg->data;
   PetscReal  val,x[VERTMAX],y[VERTMAX];
@@ -387,13 +387,13 @@ PetscErrorCode RGCheckInside_Polygon(RG rg,PetscReal px,PetscReal py,PetscInt *i
     val = (y[i]*x[j]-x[i]*y[j])/(x[j]-x[i]);
     if (PetscAbs(val)<10*PETSC_MACHINE_EPSILON) {
       *inout = 0;
-      PetscFunctionReturn(0);
+      PetscFunctionReturn(PETSC_SUCCESS);
     } else if (val>0.0) *inout = -*inout;
   }
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGSetFromOptions_Polygon(PetscOptionItems *PetscOptionsObject,RG rg)
+static PetscErrorCode RGSetFromOptions_Polygon(RG rg,PetscOptionItems PetscOptionsObject)
 {
   PetscScalar    array[VERTMAX];
   PetscInt       i,k;
@@ -406,7 +406,7 @@ PetscErrorCode RGSetFromOptions_Polygon(PetscOptionItems *PetscOptionsObject,RG
 #endif
 
   PetscFunctionBegin;
-  PetscCall(PetscOptionsHead(PetscOptionsObject,"RG Polygon Options"));
+  PetscOptionsHeadBegin(PetscOptionsObject,"RG Polygon Options");
 
     k = VERTMAX;
     for (i=0;i<k;i++) array[i] = 0;
@@ -419,11 +419,11 @@ PetscErrorCode RGSetFromOptions_Polygon(PetscOptionItems *PetscOptionsObject,RG
 #endif
     if (flg || flgi) PetscCall(RGPolygonSetVertices(rg,k,array,arrayi));
 
-  PetscCall(PetscOptionsTail());
-  PetscFunctionReturn(0);
+  PetscOptionsHeadEnd();
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode RGDestroy_Polygon(RG rg)
+static PetscErrorCode RGDestroy_Polygon(RG rg)
 {
   RG_POLYGON     *ctx = (RG_POLYGON*)rg->data;
 
@@ -437,7 +437,7 @@ PetscErrorCode RGDestroy_Polygon(RG rg)
   PetscCall(PetscFree(rg->data));
   PetscCall(PetscObjectComposeFunction((PetscObject)rg,"RGPolygonSetVertices_C",NULL));
   PetscCall(PetscObjectComposeFunction((PetscObject)rg,"RGPolygonGetVertices_C",NULL));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 SLEPC_EXTERN PetscErrorCode RGCreate_Polygon(RG rg)
@@ -445,7 +445,7 @@ SLEPC_EXTERN PetscErrorCode RGCreate_Polygon(RG rg)
   RG_POLYGON     *polygon;
 
   PetscFunctionBegin;
-  PetscCall(PetscNewLog(rg,&polygon));
+  PetscCall(PetscNew(&polygon));
   rg->data = (void*)polygon;
 
   rg->ops->istrivial      = RGIsTrivial_Polygon;
@@ -457,5 +457,5 @@ SLEPC_EXTERN PetscErrorCode RGCreate_Polygon(RG rg)
   rg->ops->destroy        = RGDestroy_Polygon;
   PetscCall(PetscObjectComposeFunction((PetscObject)rg,"RGPolygonSetVertices_C",RGPolygonSetVertices_Polygon));
   PetscCall(PetscObjectComposeFunction((PetscObject)rg,"RGPolygonGetVertices_C",RGPolygonGetVertices_Polygon));
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
