@@ -29,7 +29,7 @@ static PetscErrorCode MatMult_Cayley(Mat B,Vec x,Vec y)
   ctx = (ST_CAYLEY*)st->data;
   nu = ctx->nu;
 
-  if (st->matmode == ST_MATMODE_INPLACE) { nu = nu + st->sigma; };
+  if (st->matmode == ST_MATMODE_INPLACE) { nu = nu + st->sigma; }
 
   if (st->nmat>1) {
     /* generalized eigenproblem: y = (A + tB)x */
@@ -55,7 +55,7 @@ static PetscErrorCode MatMultTranspose_Cayley(Mat B,Vec x,Vec y)
   ctx = (ST_CAYLEY*)st->data;
   nu = ctx->nu;
 
-  if (st->matmode == ST_MATMODE_INPLACE) { nu = nu + st->sigma; };
+  if (st->matmode == ST_MATMODE_INPLACE) { nu = nu + st->sigma; }
   nu = PetscConj(nu);
 
   if (st->nmat>1) {
@@ -71,7 +71,7 @@ static PetscErrorCode MatMultTranspose_Cayley(Mat B,Vec x,Vec y)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode STGetBilinearForm_Cayley(ST st,Mat *B)
+static PetscErrorCode STGetBilinearForm_Cayley(ST st,Mat *B)
 {
   PetscFunctionBegin;
   PetscCall(STSetUp(st));
@@ -80,7 +80,7 @@ PetscErrorCode STGetBilinearForm_Cayley(ST st,Mat *B)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode STBackTransform_Cayley(ST st,PetscInt n,PetscScalar *eigr,PetscScalar *eigi)
+static PetscErrorCode STBackTransform_Cayley(ST st,PetscInt n,PetscScalar *eigr,PetscScalar *eigi)
 {
   ST_CAYLEY   *ctx = (ST_CAYLEY*)st->data;
   PetscInt    j;
@@ -110,7 +110,7 @@ PetscErrorCode STBackTransform_Cayley(ST st,PetscInt n,PetscScalar *eigr,PetscSc
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode STPostSolve_Cayley(ST st)
+static PetscErrorCode STPostSolve_Cayley(ST st)
 {
   PetscFunctionBegin;
   if (st->matmode == ST_MATMODE_INPLACE) {
@@ -129,7 +129,7 @@ PetscErrorCode STPostSolve_Cayley(ST st)
    if nmat=1:  (A-sI)^-1 (A+tI)    A-sI      A+tI
    if nmat=2:  (A-sB)^-1 (A+tB)    A-sB      A+tI
 */
-PetscErrorCode STComputeOperator_Cayley(ST st)
+static PetscErrorCode STComputeOperator_Cayley(ST st)
 {
   PetscInt       n,m;
   ST_CAYLEY      *ctx = (ST_CAYLEY*)st->data;
@@ -162,7 +162,7 @@ PetscErrorCode STComputeOperator_Cayley(ST st)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode STSetUp_Cayley(ST st)
+static PetscErrorCode STSetUp_Cayley(ST st)
 {
   PetscFunctionBegin;
   PetscCheck(st->nmat<=2,PetscObjectComm((PetscObject)st),PETSC_ERR_SUP,"Cayley transform cannot be used in polynomial eigenproblems");
@@ -171,7 +171,7 @@ PetscErrorCode STSetUp_Cayley(ST st)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode STSetShift_Cayley(ST st,PetscScalar newshift)
+static PetscErrorCode STSetShift_Cayley(ST st,PetscScalar newshift)
 {
   ST_CAYLEY      *ctx = (ST_CAYLEY*)st->data;
 
@@ -197,7 +197,7 @@ PetscErrorCode STSetShift_Cayley(ST st,PetscScalar newshift)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode STSetFromOptions_Cayley(ST st,PetscOptionItems *PetscOptionsObject)
+static PetscErrorCode STSetFromOptions_Cayley(ST st,PetscOptionItems PetscOptionsObject)
 {
   PetscScalar    nu;
   PetscBool      flg;
@@ -287,12 +287,12 @@ PetscErrorCode STCayleyGetAntishift(ST st,PetscScalar *nu)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(st,ST_CLASSID,1);
-  PetscValidScalarPointer(nu,2);
+  PetscAssertPointer(nu,2);
   PetscUseMethod(st,"STCayleyGetAntishift_C",(ST,PetscScalar*),(st,nu));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode STView_Cayley(ST st,PetscViewer viewer)
+static PetscErrorCode STView_Cayley(ST st,PetscViewer viewer)
 {
   char           str[50];
   ST_CAYLEY      *ctx = (ST_CAYLEY*)st->data;
@@ -307,7 +307,7 @@ PetscErrorCode STView_Cayley(ST st,PetscViewer viewer)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-PetscErrorCode STDestroy_Cayley(ST st)
+static PetscErrorCode STDestroy_Cayley(ST st)
 {
   PetscFunctionBegin;
   PetscCall(PetscFree(st->data));
@@ -328,6 +328,7 @@ SLEPC_EXTERN PetscErrorCode STCreate_Cayley(ST st)
 
   st->ops->apply           = STApply_Generic;
   st->ops->applytrans      = STApplyTranspose_Generic;
+  st->ops->applyhermtrans  = STApplyHermitianTranspose_Generic;
   st->ops->backtransform   = STBackTransform_Cayley;
   st->ops->setshift        = STSetShift_Cayley;
   st->ops->getbilinearform = STGetBilinearForm_Cayley;
